
#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <math.h>

char	OUTPUT_PATH[300];
char	savename[200];
char	filename[200];

int	NULL_flag;

int **tempmati,**tempmati2;
int **gradient, **gradient_x, **gradient_y;
int **EdgeBlock;
int **occlusion;
int **canny, **canny_thred;
int **initDisp;
int **smoothRegion;


//Main functions
void DisparityWithRegularization(UnsignedRGB **left, UnsignedRGB **right, IplImage *left_org_grey, IplImage *right_org_grey,
								 float **Disparity, int  SIZE_W, int SIZE_H, unsigned char hierarchy, unsigned char Block,
								 int Min_Range, int Max_Range, float scale, unsigned char Epsilon,
								 int Region_Th, float beta, int lambda, int iteration, 
								 int canny_Th, char *path, int ini_disp);


//sub functions
void InitDisparityEstimation(UnsignedRGB **left, UnsignedRGB **right, 
							 int **initDisp, int **canny, int SIZE_H, int SIZE_W,
							 unsigned char Block, int Min_Range, int Max_Range, unsigned char Epsilon);
void AmbiguousVectorModification(UnsignedRGB **sub_L, UnsignedRGB **sub_R, 
								 int **initDisp, int **candidate,  
								 int SIZE_H, int SIZE_W, unsigned char Block,
								 int Min_Range, int Max_Range);
void canny_detector(UnsignedRGB **left, int **canny, int **canny_thred, int SIZE_H, int SIZE_W, int canny_Th);

void SmoothRegionRegularization(int **initDisp, int **maxGradOfBlock, 
								int **candidate, double **reliability, 
								int **smoothRegion, int SIZE_H, int SIZE_W,
								unsigned char Block, unsigned char Grad_Th, 
								int Region_Th);
void DenseDisparityRegionDividing(UnsignedRGB **left, UnsignedRGB **right, 
								  int **initDisp, int **Disparity, int **canny, 
								  int **canny_thred, int SIZE_H, int SIZE_W, int Block,
								  int Min_Range, int Max_Range, unsigned char Epsilon);
void DenseDisparityHierarchy(UnsignedRGB **left, UnsignedRGB **right, int **initDisp, int **Disparity, 
							 int SIZE_H, int SIZE_W, int Block, int Min_Range, int Max_Range);
void DenseDisparityHierarchy2(UnsignedRGB **left, UnsignedRGB **right, 
								  int **initDisp, int **Disparity, int **canny, 
								  int **canny_thred, int SIZE_H, int SIZE_W, int Block,
								  int Min_Range, int Max_Range, unsigned char Epsilon);
void DisparityRegularization(unsigned char **left, unsigned char **right, float **Disparity, 
							 int SIZE_H, int SIZE_W, float beta, int lambda, int iteration);

void boundary(int **Disparity, int SIZE_H, int SIZE_W, int Block);
void OcclusionFilling(int **Disparity, int SIZE_H, int SIZE_W);
void Dx(double **input, double **output, int delta, int SIZE_H, int SIZE_W);
void Dy(double **input, double **output, int delta, int SIZE_H, int SIZE_W);
		
void DownURGB(UnsignedRGB **input, UnsignedRGB **output, int SIZE_H, int SIZE_W, int down_ratio);
void TransferURGB(UnsignedRGB **input, UnsignedRGB **output, int SIZE_H, int SIZE_W);

//IO functions
void load_dense_disp(int **DisparityInt, int ysize, int xsize, float scale, unsigned char hierarchy, int Min_Range,int Max_Range);
void SaveBmpScaling(int **output, int ysize, int xsize, char* name);
void SaveBmpScaling(float **output, int ysize, int xsize, char* name);
void SaveBmpScalingException(int **output, int ysize, int xsize, int exception, char* name);

//////////////////////////////////////////////////////////////////////////////////////////////////

void Dx(double **input, double **output, int delta, int SIZE_H, int SIZE_W)
{
	int i,j;

	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
		{
			if(j+1<SIZE_W)
				output[i][j]=(input[i][j+1]-input[i][j])/delta;
			else
				output[i][j]=(input[i][0]-input[i][j])/delta;
		}
}

void Dy(double **input, double **output, int delta, int SIZE_H, int SIZE_W)
{
	int i,j;

	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
		{
			if(i+1<SIZE_H)
				output[i][j]=(input[i+1][j]-input[i][j])/delta;
			else
				output[i][j]=(input[0][j]-input[i][j])/delta;
		}
}

void DownURGB(UnsignedRGB **input, UnsignedRGB **output, int SIZE_H, int SIZE_W, int down_ratio)
{
	int i,j,a,b;
	int sum_r, sum_g, sum_b;

	for(i=0;i<SIZE_H;i+=down_ratio)
		for(j=0;j<SIZE_W;j+=down_ratio)
		{
			sum_r=sum_g=sum_b=0;

			for(a=0;a<down_ratio;a++)
				for(b=0;b<down_ratio;b++)
				{
					sum_r+=input[i+a][j+b].r;
					sum_g+=input[i+a][j+b].g;
					sum_b+=input[i+a][j+b].b;
				}
				output[i/down_ratio][j/down_ratio].r=(int)(sum_r/(down_ratio*down_ratio)+0.5);
				output[i/down_ratio][j/down_ratio].g=(int)(sum_g/(down_ratio*down_ratio)+0.5);
				output[i/down_ratio][j/down_ratio].b=(int)(sum_b/(down_ratio*down_ratio)+0.5);
		}

}

void TransferURGB(UnsignedRGB **input, UnsignedRGB **output, int SIZE_H, int SIZE_W)
{
	int i,j;

	output=Allo2RGB(SIZE_H, SIZE_W);

	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
		{
				output[i][j].r=input[i][j].r;
				output[i][j].g=input[i][j].g;
				output[i][j].b=input[i][j].b;
		}

	Deallo2RGB(input,SIZE_H);

}

void load_dense_disp(int **DisparityInt, int ysize, int xsize, float scale, unsigned char hierarchy, int Min_Range,int Max_Range)
{

	int i,j;
	int tempi;
	CvSize Imagesize, output_size;
	Imagesize.height=ysize;
	Imagesize.width	=xsize;
	IplImage*	result	= cvCreateImage( Imagesize, 8, 1 );

	printf("\nLoading Dense disparity from the file");
	printf("\nConfirm the scale parameters and if the disparity image size is %d by %d\n", ysize, xsize);

	IplImage *ini_org;

	sprintf(filename, "%s%s", BASE_PATH, DENSE_DISP);
	if((ini_org=cvLoadImage(filename)) == 0)
	{
		printf("Cannot open the initial disparity image - %s\n",filename);
		exit(0);
	}

	output_size.width=xsize;
	output_size.height=ysize;
	result	= cvCreateImage( output_size, 8, 1 );

	//left
	cvSplit(ini_org, result, NULL, NULL, NULL);		
	for(i=0; i<output_size.height; i++)
		for(j=0; j<output_size.width; j++)
		{
			tempi=cvGetReal2D(result,i,j);

			if(tempi==0)
				DisparityInt[i][j]=Mark2;
			else
				DisparityInt[i][j]=(int)((tempi/scale+Min_Range));
		}

	cvReleaseImage(&result);
	cvReleaseImage(&ini_org);
}


void SaveBmpScaling(int **output, int ysize, int xsize, char* name)
{
	int a,b,max,min;
	int temp;
   CvSize Imagesize;
   
   Imagesize.height	=ysize;
   Imagesize.width	=xsize;
   
   IplImage*	result	= cvCreateImage( Imagesize, 8, 1 );

	max=-100000;
	min=100000;

	for(a=0;a<ysize;a++){
		for(b=0;b<xsize;b++){
      	if(output[a][b]>max)
         	max=output[a][b];
      	if(output[a][b]<min)
         	min=output[a][b];
         }
      }

	printf("disparity min = %d, max = %d\n", min, max);

	//save disparity
	for(a=0; a<ysize; a++)
		for(b=0; b<xsize; b++)
		{
			for(a=0; a<ysize; a++)
				for(b=0; b<xsize; b++)
					cvSetReal2D(result,a,b,(output[a][b]-min)/(double)(max-min)*255);
		}


	sprintf( savename, "%s%s", OUTPUT_PATH, name);
	cvSaveImage(savename, result);

	cvReleaseImage(&result);
}

void SaveBmpScaling(float **output, int ysize, int xsize, char* name)
{
	int a,b;
	int temp;
	float max,min;
   CvSize Imagesize;
   
   Imagesize.height	=ysize;
   Imagesize.width	=xsize;
   
   IplImage*	result	= cvCreateImage( Imagesize, 8, 1 );

	max=-100000;
	min=100000;

	for(a=0;a<ysize;a++){
		for(b=0;b<xsize;b++){
      	if(output[a][b]>max)
         	max=output[a][b];
      	if(output[a][b]<min)
         	min=output[a][b];
         }
      }

	printf("\ndisparity min = %f, max = %f\n", min, max);

	//save disparity
	for(a=0; a<ysize; a++)
		for(b=0; b<xsize; b++)
		{
			for(a=0; a<ysize; a++)
				for(b=0; b<xsize; b++)
					cvSetReal2D(result,a,b,(output[a][b]-min)/(double)(max-min)*255);
		}

	sprintf( savename, "%s%s", OUTPUT_PATH, name);
	cvSaveImage(savename, result);

	cvReleaseImage(&result);
}

void SaveBmpScalingException(int **output, int ysize, int xsize, int exception, char* name)
{
	int a,b,max,min;
	int temp;
   CvSize Imagesize;
   
   Imagesize.height	=ysize;
   Imagesize.width	=xsize;
   
   IplImage*	result	= cvCreateImage( Imagesize, 8, 1 );

	max=-100000;
	min=100000;

	for(a=0;a<ysize;a++){
		for(b=0;b<xsize;b++){
			if(output[a][b]!=exception)
			{
      			if(output[a][b]>max)
         			max=output[a][b];
      			if(output[a][b]<min)
         			min=output[a][b];
			}
         }
      }
	
	printf("disparity min = %d, max = %d\n", min, max);

	for(a=0; a<ysize; a++)
		for(b=0; b<xsize; b++)
		{
			if(output[a][b]!=exception)
			{
				for(a=0; a<ysize; a++)
					for(b=0; b<xsize; b++)
						cvSetReal2D(result,a,b,(output[a][b]-min)/(double)(max-min)*255);
			}
			else
				cvSetReal2D(result,a,b,0);
		}

	sprintf( savename, "%s%s", OUTPUT_PATH, name);
	cvSaveImage(savename, result);

	cvReleaseImage(&result);
}


/////////////////////////////////////////////////////////////////////////////////
//
//  Sub functions
//
//////////////////////////////////////////////////////////////////////////////////

void AmbiguousVectorModification(UnsignedRGB **sub_L, UnsignedRGB **sub_R, 
								 int **initDisp, int **candidate,  
								 int SIZE_H, int SIZE_W, unsigned char Block,
								 int Min_Range, int Max_Range)
{
	int a,b,i,j,x,y,t,t2,err,err_min,start,end;
	int row, col;

	float Block_sum1, Block_sum2;
	float Block_sq_sum1, Block_sq_sum2;
	float Block_mean1,Block_mean2;
	float NCC;
	float NCC_max;


	row=SIZE_H;///2;
	col=SIZE_W;///2;
	int Block2=Block;///2;

	for(a=0;a<row;a+=Block2)
   		for(b=0;b<col;b+=Block2)
		{
			if(candidate[a/Block2][b/Block2]==0)
			{
				//Search rage 
			   start=Min_Range;///2;
			   end=Max_Range;///2;

			   for(i=0;i<(col-b)/Block2;i++)
			   {
				  if(candidate[a/Block2][b/Block2+i]==1)
				  {
					  if(i*Block2+initDisp[a/Block2][b/Block2+i]<Max_Range)///2)
					  {
						  end=i*Block2+initDisp[a/Block2][b/Block2+i];
					  }
					  break;
				  }
			   }

			   for(i=0;i>(-1)*b/Block2;i--)
			   {
				   if(candidate[a/Block2][b/Block2+i]==1)
				   {
					   if(i*Block2+initDisp[a/Block2][b/Block2+i]>Min_Range)///2)
							start=i*Block2+initDisp[a/Block2][b/Block2+i];
					   break;
				   }
			   }

				// Block matching
				err_min=200000000;
				NCC_max=0.0;

				for(t=start;t<=end;t++)
				{
           			if(b+t+Block2>=col)
              			continue;
           			if(b+t<0)
              			continue;

					if(matching_method==1)
					{

						//NCC of R
						Block_sum1=	Block_sum2=0.0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
							{
								Block_sum1+=(float)sub_L[a+x][b+y].r;
								Block_sum2+=(float)sub_R[a+x][b+t+y].r;
							}
						Block_mean1=Block_sum1/(Block2*Block2);
						Block_mean2=Block_sum2/(Block2*Block2);

						Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
							{
								Block_sum1+=((sub_L[a+x][b+y].r-Block_mean1)*(sub_R[a+x][b+t+y].r-Block_mean2));
								//Block_sum2+=(sub_R[a+x][b+t+y].r-Block_mean2);
								Block_sq_sum1+= pow((sub_L[a+x][b+y].r-Block_mean1),2);
								Block_sq_sum2+= pow((sub_R[a+x][b+t+y].r-Block_mean2),2);
							}

						if(Block_sq_sum1==0)	//avoid devided by 0
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

						NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2);

						//NCC of G
						Block_sum1=	Block_sum2=0.0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
							{
								Block_sum1+=(float)sub_L[a+x][b+y].g;
								Block_sum2+=(float)sub_R[a+x][b+t+y].g;
							}
						Block_mean1=Block_sum1/(Block2*Block2);
						Block_mean2=Block_sum2/(Block2*Block2);

						Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
							{
								Block_sum1+=((sub_L[a+x][b+y].g-Block_mean1)*(sub_R[a+x][b+t+y].g-Block_mean2));
								//Block_sum2+=(sub_R[a+x][b+t+y].g-Block_mean2);
								Block_sq_sum1+= pow((sub_L[a+x][b+y].g-Block_mean1),2);
								Block_sq_sum2+= pow((sub_R[a+x][b+t+y].g-Block_mean2),2);
							}

						if(Block_sq_sum1==0)	//avoid devided by 0
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

						NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2));

						//NCC of B
						Block_sum1=	Block_sum2=0.0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
							{
								Block_sum1+=(float)sub_L[a+x][b+y].b;
								Block_sum2+=(float)sub_R[a+x][b+t+y].b;
							}
						Block_mean1=Block_sum1/(Block2*Block2);
						Block_mean2=Block_sum2/(Block2*Block2);

						Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
							{
								Block_sum1+=((sub_L[a+x][b+y].b-Block_mean1)*(sub_R[a+x][b+t+y].b-Block_mean2));
								//Block_sum2+=(sub_R[a+x][b+t+y].b-Block_mean2);
								Block_sq_sum1+= pow((sub_L[a+x][b+y].b-Block_mean1),2);
								Block_sq_sum2+= pow((sub_R[a+x][b+t+y].b-Block_mean2),2);
							}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

						NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2));

						//Mean of NCC
						NCC /= 3.0;

						if(NCC >= NCC_max)
						{
							NCC_max=NCC;
							t2=t;
						}
					}

					else
					{
						err=0;

						for(x=0;x<Block2;x++)
							for(y=0;y<Block2;y++)
								err+=((abs(sub_L[a+x][b+y].r-sub_R[a+x][b+t+y].r)+abs(sub_L[a+x][b+y].g-sub_R[a+x][b+t+y].g)
								+abs(sub_L[a+x][b+y].b-sub_R[a+x][b+t+y].b))/3);

						if(err<err_min)
						{
							err_min=err;
							t2=t;
						}
					}
				}

				initDisp[a/Block2][b/Block2]=t2;
				candidate[a/Block2][b/Block2]=1;
			}
		}
}

void InitDisparityEstimation(UnsignedRGB **left, UnsignedRGB **right, 
							 int **initDisp, int **canny,  int SIZE_H, int SIZE_W,
							 unsigned char Block, int Min_Range, int Max_Range,unsigned char Epsilon)
{
	//SubSampling κ ʱ disparity 
	int i,j,a,b,c,x,y,t,t2,temp,start,end,err_min,err_min2,err,inverse_t;
	int row, col;
	int max,min,sum;
	int temp1;
	double temp2,temp3;
	int **temp_mat;
	UnsignedRGB **sub_L, **sub_R;
	int *sumGrad, **MAE,**candidate;
	int *barrier,*index;

	float Block_sum1, Block_sum2;
	float Block_sq_sum1, Block_sq_sum2;
	float Block_mean1,Block_mean2;
	float NCC;
	float NCC_max, NCC_max2;

	row=SIZE_H;///2;
	col=SIZE_W;///2;
	int Block2=Block;///2;

	barrier=new int[col/Block2];
	index=new int[col/Block2];
	sumGrad=new int[col/Block2];

	sub_L=Allo2RGB(row,col);
	sub_R=Allo2RGB(row,col);
	temp_mat=Allo2Int(row,col);
	MAE=Allo2Int(row/Block2,col/Block2);
	candidate=Allo2Int(row/Block2,col/Block2);

	//size conversion
	//SizeConversionRGB(SIZE_H,SIZE_W,row,col,left,sub_L);
	//SizeConversionRGB(SIZE_H,SIZE_W,row,col,right,sub_R);

	//20090811
	for(a=0;a<row;a++)
	   for(b=0;b<col;b++)
	   {
		   sub_L[a][b].b=left[a][b].b;
		   sub_L[a][b].g=left[a][b].g;
		   sub_L[a][b].r=left[a][b].r;
		   sub_R[a][b].b=right[a][b].b;
		   sub_R[a][b].g=right[a][b].g;
		   sub_R[a][b].r=right[a][b].r;
	   }
	

	for(i=0;i<row/Block2;i++)          
	   for(j=0;j<col/Block2;j++)
	   {
            initDisp[i][j]=0;
			candidate[i][j]=0;
	   }

	for(a=0;a<row;a+=Block2)
	{

		NULL_flag=1;
		for(b=0;b<col;b++)
		{
			if(left[a][b].r!=0)
				NULL_flag=0;
		}
		if(NULL_flag==1)
		{
			for(b=0;b<col;b+=Block)
			{
				initDisp[a/Block][b/Block]=Min_Range;
				candidate[a/Block][b/Block]=1;
			}
			
			continue;
		}


	   for(b=0;b<col;b+=Block2)
	   {
			max=-1000;
			sum=0;

			for(i=0;i<Block;i++)
				for(j=0;j<Block;j++)
					sum+=canny[a+i][b+j];
					//sum+=canny[2*a+i][2*b+j];
		   
			sumGrad[b/Block2]=sum;
			index[b/Block2]=b;
	   }

		//  ʱȭ
	   for(i=0;i<col/Block2;i++)
	       barrier[i]=0;

		//sorting
	   for(i=0;i<col/Block2-1;i++)
		   for(j=col/Block2-1;j>i;j--)
		   {
			   if(sumGrad[j-1]>sumGrad[j])
			     	continue;
			   temp=sumGrad[j-1];
			   sumGrad[j-1]=sumGrad[j];
               sumGrad[j]=temp;
			   temp=index[j-1];
			   index[j-1]=index[j];
               index[j]=temp;
		   }

		//  
	   for(c=0;c<col/Block2;c++)
	   {
		   b=index[c];

		   start=Min_Range;///2;			
		   end=Max_Range;///2;

		   for(i=0;i<(col-b)/Block2;i++)
		   {
		      if(barrier[b/Block2+i]==1)
			  {
				  if(i*Block2+initDisp[a/Block2][b/Block2+i] < Max_Range)///2)
					  end=i*Block2+initDisp[a/Block2][b/Block2+i];
				  break;
			  }
		   }

		   for(i=0;i>(-1)*b/Block2;i--)
		   {
			   if(barrier[b/Block2+i]==1)
			   {
				   if(i*Block2+initDisp[a/Block2][b/Block2+i] > Min_Range)///2)
				   		start=i*Block2+initDisp[a/Block2][b/Block2+i];
				   break;
			   }
		   }

			// Block matching
			err_min=200000000;
			NCC_max=0.0;

			for(t=start;t<=end;t++)
			{
           		if(b+t+Block2>=col)
              		continue;
           		if(b+t<0)
              		continue;

				if(matching_method==1)
				{
					
					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=(float)sub_L[a+x][b+y].r;
							Block_sum2+=(float)sub_R[a+x][b+t+y].r;
						}
					Block_mean1=Block_sum1/(Block2*Block2);
					Block_mean2=Block_sum2/(Block2*Block2);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=((sub_L[a+x][b+y].r-Block_mean1)*(sub_R[a+x][b+t+y].r-Block_mean2));
							Block_sq_sum1+= pow((sub_L[a+x][b+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((sub_R[a+x][b+t+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=(float)sub_L[a+x][b+y].g;
							Block_sum2+=(float)sub_R[a+x][b+t+y].g;
						}
					Block_mean1=Block_sum1/(Block2*Block2);
					Block_mean2=Block_sum2/(Block2*Block2);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=((sub_L[a+x][b+y].g-Block_mean1)*(sub_R[a+x][b+t+y].g-Block_mean2));
							//Block_sum2+=(sub_R[a+x][b+t+y].g-Block_mean2);
							Block_sq_sum1+= pow((sub_L[a+x][b+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((sub_R[a+x][b+t+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=(float)sub_L[a+x][b+y].b;
							Block_sum2+=(float)sub_R[a+x][b+t+y].b;
						}
					Block_mean1=Block_sum1/(Block2*Block2);
					Block_mean2=Block_sum2/(Block2*Block2);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=((sub_L[a+x][b+y].b-Block_mean1)*(sub_R[a+x][b+t+y].b-Block_mean2));
							//Block_sum2+=(sub_R[a+x][b+t+y].b-Block_mean2);
							Block_sq_sum1+= pow((sub_L[a+x][b+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((sub_R[a+x][b+t+y].b-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2));

					//Mean of NCC
					NCC /= 3.0;

					if(NCC >= NCC_max)
					{
						NCC_max=NCC;
						t2=t;
					}
				}

				else	//MAE
				{
					err=0;
					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
							err+=((abs(sub_L[a+x][b+y].r-sub_R[a+x][b+t+y].r)+abs(sub_L[a+x][b+y].g-sub_R[a+x][b+t+y].g)+
							abs(sub_L[a+x][b+y].b-sub_R[a+x][b+t+y].b))/3);
					if(err<err_min)
					{
						err_min=err;
						t2=t;
						}
				}
			}
			//MAE[a/Block2][b/Block2]=err_min;

			//Bi-directiobal check

			start=Min_Range;///2;
		    end=Max_Range;///2;
			
		   for(i=0;i<(col-b)/Block2;i++)
		      if(barrier[b/Block2+i]==1)
			  {
				 end=i*Block2;
				 break;
			  }
		   for(i=0;i>(-1)*b/Block2;i--)
			   if(barrier[b/Block2+i]==1)
			   {
				   start=i*Block2;
				   break;
			   }

			err_min2=200000000;
			NCC_max2=0.0;

			for(t=start;t<=end;t++)
			{
        		if(b+t+Block2>col)
	           		continue;
        		if(b+t<0)
		       		continue;

				if(matching_method==1)
				{
					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=(float)sub_R[a+x][b+t2+y].r;
							Block_sum2+=(float)sub_L[a+x][b+t+y].r;
						}
					Block_mean1=Block_sum1/(Block2*Block2);
					Block_mean2=Block_sum2/(Block2*Block2);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=((sub_R[a+x][b+t2+y].r-Block_mean1)*(sub_L[a+x][b+t+y].r-Block_mean2));
							//Block_sum2+=(sub_L[a+x][b+t+y].r-Block_mean2);
							Block_sq_sum1+= pow((sub_R[a+x][b+t2+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((sub_L[a+x][b+t+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=(float)sub_R[a+x][b+t2+y].g;
							Block_sum2+=(float)sub_L[a+x][b+t+y].g;
						}
					Block_mean1=Block_sum1/(Block2*Block2);
					Block_mean2=Block_sum2/(Block2*Block2);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=((sub_R[a+x][b+t2+y].g-Block_mean1)*(sub_L[a+x][b+t+y].g-Block_mean2));
							//Block_sum2+=(sub_L[a+x][b+t+y].g-Block_mean2);
							Block_sq_sum1+= pow((sub_R[a+x][b+t2+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((sub_L[a+x][b+t+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2);

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=(float)sub_R[a+x][b+t2+y].b;
							Block_sum2+=(float)sub_L[a+x][b+t+y].b;
						}
					Block_mean1=Block_sum1/(Block2*Block2);
					Block_mean2=Block_sum2/(Block2*Block2);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
						{
							Block_sum1+=((sub_R[a+x][b+t2+y].b-Block_mean1)*(sub_L[a+x][b+t+y].b-Block_mean2));
							//Block_sum2+=(sub_L[a+x][b+t+y].b-Block_mean2);
							Block_sq_sum1+= pow((sub_R[a+x][b+t2+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((sub_L[a+x][b+t+y].b-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block2*Block2);

					//Mean of NCC
					NCC /= 3.0;
					
					if(NCC>=NCC_max2)
					{
						NCC_max2=NCC;
						inverse_t=t;
					}
				}

				else
				{
					err=0;

					for(x=0;x<Block2;x++)
						for(y=0;y<Block2;y++)
							err+=((abs(sub_R[a+x][b+t2+y].r-sub_L[a+x][b+t+y].r)+abs(sub_R[a+x][b+t2+y].g-sub_L[a+x][b+t+y].g)
							+abs(sub_R[a+x][b+t2+y].b-sub_L[a+x][b+t+y].b))/3);

					if(err<err_min2)
					{
						err_min2=err;
						inverse_t=t;
					}
				}
			}

			if(abs(inverse_t)<=Epsilon)
			{
				initDisp[a/Block2][b/Block2]=t2;
				candidate[a/Block2][b/Block2]=1;

				if(c<col/Block2/4)
					barrier[b/Block2]=1;
			}
			else
				initDisp[a/Block2][b/Block2]=t2;

	   }
	}

	// ĺ   ¿ Ȯ  Ͽ  
	AmbiguousVectorModification(sub_L, sub_R, initDisp, candidate, SIZE_H, SIZE_W, 
								Block, Min_Range, Max_Range);

	//median filtering
	Median33(row/Block2,col/Block2,initDisp,initDisp);

	delete barrier;
	delete index;
	delete sumGrad;
	Deallo2RGB(sub_L,row);
	Deallo2RGB(sub_R,row);
	Deallo2Int(temp_mat,row);
	Deallo2Int(MAE,row/Block2);
	Deallo2Int(candidate,row/Block2);
}


void canny_detector(UnsignedRGB **left, int **canny, int **canny_thred,int SIZE_H, int SIZE_W, int canny_Th)
{
	int i,j;
	CvSize size = cvSize(SIZE_W,SIZE_H);

	//Canny edge detector & Thresholding
	IplImage	*tempimg = cvCreateImage(size,IPL_DEPTH_8U,1);
	IplImage	*tempimg2 = cvCreateImage(size,IPL_DEPTH_8U,1);
	IplImage	*sobel_x = cvCreateImage(size,IPL_DEPTH_16S,1);
	IplImage	*sobel_y = cvCreateImage(size,IPL_DEPTH_16S,1);
	IplImage	*sobel = cvCreateImage(size,IPL_DEPTH_16S,1);
	IplImage	*canny_r = cvCreateImage(size,IPL_DEPTH_8U,1);
	IplImage	*canny_g = cvCreateImage(size,IPL_DEPTH_8U,1);
	IplImage	*canny_b = cvCreateImage(size,IPL_DEPTH_8U,1);

	//R channel
	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
			cvSetReal2D(tempimg,i,j,left[i][j].r);

	cvCanny( tempimg, tempimg2, 20, 35);

	cvSobel(tempimg,sobel_x,1,0);
	cvSobel(tempimg,sobel_y,0,1);
	cvAdd(sobel_x,sobel_y,sobel);
	cvConvertScaleAbs(sobel,tempimg);

	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
		{
			if(cvGetReal2D(tempimg2,i,j)>128)
				cvSetReal2D(canny_r,i,j,cvGetReal2D(tempimg,i,j));
			else
				cvSetReal2D(canny_r,i,j,0);
		}

	//G channel
	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
			cvSetReal2D(tempimg,i,j,left[i][j].g);

	cvCanny( tempimg, tempimg2, 20, 35);

	cvSobel(tempimg,sobel_x,1,0);
	cvSobel(tempimg,sobel_y,0,1);
	cvAdd(sobel_x,sobel_y,sobel);
	cvConvertScaleAbs(sobel,tempimg);

	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
		{
			if(cvGetReal2D(tempimg2,i,j)>128)
				cvSetReal2D(canny_g,i,j,cvGetReal2D(tempimg,i,j));
			else
				cvSetReal2D(canny_g,i,j,0);
		}

	//b channel
	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
			cvSetReal2D(tempimg,i,j,left[i][j].b);

	cvCanny( tempimg, tempimg2, 20, 35);

	cvSobel(tempimg,sobel_x,1,0);
	cvSobel(tempimg,sobel_y,0,1);
	cvAdd(sobel_x,sobel_y,sobel);
	cvConvertScaleAbs(sobel,tempimg);

	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
		{
			if(cvGetReal2D(tempimg2,i,j)>128)
				cvSetReal2D(canny_b,i,j,cvGetReal2D(tempimg,i,j));
			else
				cvSetReal2D(canny_b,i,j,0);
		}

	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
			canny[i][j]=(int)((cvGetReal2D(canny_r,i,j)+cvGetReal2D(canny_g,i,j)+cvGetReal2D(canny_b,i,j))/3.0+0.5);

	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
		{
			if(canny[i][j]>	canny_Th)
				canny_thred[i][j]=255;
			else
				canny_thred[i][j]=0;
		}

	cvReleaseImage(&sobel_x);
	cvReleaseImage(&sobel_y);
	cvReleaseImage(&sobel);
	cvReleaseImage(&canny_r);
	cvReleaseImage(&canny_g);
	cvReleaseImage(&canny_b);
	cvReleaseImage(&tempimg);
	cvReleaseImage(&tempimg2);
}


void SmoothRegionRegularization(int **initDisp, int **maxGradOfBlock, 
								int **candidate, double **reliability, 
								int **smoothRegion, int SIZE_H, int SIZE_W,
								unsigned char Block, unsigned char Grad_Th, 
								int Region_Th)
{
	//Ȱ ã Ȱ   max reliability  ̷ ä
	int k,l,i,j,a,b,count,old_count;
	int **temp2;
	int row=SIZE_H/2, col=SIZE_W/2;
	double maxReliability;
	int realDisparity;
	int Block2=Block/2;

	temp2=Allo2Int(row/Block2+2,col/Block2+2);

	//  ʱȭ
	for(i=0;i<row/Block2+2;i++)
   		for(j=0;j<col/Block2+2;j++)
			temp2[i][j]=0;
	
	for(i=0;i<row/Block2;i++)
   		for(j=0;j<col/Block2;j++)
			if(maxGradOfBlock[i][j]<Grad_Th)	
				temp2[i+1][j+1]=255;

		
	//Region growing
	for(k=0;k<row/Block2;k++)
   		for(l=0;l<col/Block2;l++)
		{
			if(temp2[k+1][l+1]==255)
			{
				temp2[k+1][l+1]=2;
				old_count=0;
				maxReliability=-1000000.0;

				while(1)
				{
					count=0;

					for(i=0;i<row/Block2;i++)
						for(j=0;j<col/Block2;j++)
						{
							if(temp2[i+1][j+1]==2)
								count++;
						}


					//̻   max reliability ̷ 
					//	 ̸ üϰ ٸ seedã....		
					if(old_count==count)
					{
						//max reliability disparity ã
						for(a=0;a<row/Block2;a++)
							for(b=0;b<col/Block2;b++)
							{
								if(temp2[a+1][b+1]==2 && reliability[a][b]>maxReliability)
								{
									maxReliability=reliability[a][b];
									realDisparity=initDisp[a][b];
								}
							}

						for(a=0;a<row/Block2;a++)
							for(b=0;b<col/Block2;b++)
							{
								if(temp2[a+1][b+1]==2)
									{
										initDisp[a][b]=realDisparity;
										temp2[a+1][b+1]=3;
										candidate[a][b]=1;
										if(count>Region_Th)		
											smoothRegion[a][b]=1;
									}
							}
						
						break;
					}


					//Growing
					for(i=0;i<row/Block2;i++)
						for(j=0;j<col/Block2;j++)
						{
							if(temp2[i+1][j+1]==2)
								continue;
							if(( (temp2[i][j+1]==2)||(temp2[i+1][j]==2)||(temp2[i+1][j+2]==2)
								||(temp2[i+2][j+1]==2)) && (temp2[i+1][j+1]!=0) )
								temp2[i+1][j+1]=2;
						}

					old_count=count;
				}
			}
		}
}


void DenseDisparityRegionDividing(UnsignedRGB **left, UnsignedRGB **right, 
								  int **initDisp, int **Disparity, int **canny, 
								  int **canny_thred, int SIZE_H, int SIZE_W, int Block,
								  int Min_Range, int Max_Range, unsigned char Epsilon)
{
	//Disparity estimation by bidirectional matching with shape adaptive window
   int i,j,a,b,c,x,y,t,temp,start,end,start2,end2,err_min,second_min,err,inverse_t,center;
   int t2=0;
   int t3=0;
   int mode;		//⺻ mode=1, mode=2 ̸   ι Ȯ
   int **vector;   
   int *barrier,*index,*edgeblock;
   int min, max;
   int mask[9][9];
   int count, old_count;

	//For NCC
	float Block_sum1, Block_sum2;
	float Block_sq_sum1, Block_sq_sum2;
	float Block_mean1,Block_mean2;
	float NCC;
	float NCC_max, NCC_second_max, NCC_max2;

   //Ÿ Ķ 
   int alpha=1;
	int maskSize=Block/2;
	int Edge_Th=8;
	int Filterlength=5;
	int Block1=Block/2;
	int Block2=Block/4;

   barrier=new int[SIZE_W];
   vector=Allo2Int(SIZE_H, SIZE_W);
   index=new int[SIZE_W-Block];
   edgeblock=new int[SIZE_W-Block];

   for(i=0;i<SIZE_H;i++)          //Initialisation
	   for(j=0;j<SIZE_W;j++)
	   {
		   if(i<Block/2 || i>=SIZE_H-Block/2 || j<Block/2 || j>=SIZE_W-Block/2 )
			   vector[i][j]=initDisp[i/Block][j/Block];
			   //vector[i][j]=2*initDisp[i/Block][j/Block];
		   else
	           vector[i][j]=Mark2;
	   }

	min=2000;
	max=-2000;
	for(a=0;a<SIZE_H/Block;a++)
		for(b=0;b<SIZE_W/Block;b++)
		{
			//if(2*initDisp[a][b]>max)
			//	max=2*initDisp[a][b];
			//if(2*initDisp[a][b]<min)
			//	min=2*initDisp[a][b];
			if(initDisp[a][b]>max)
				max=initDisp[a][b];
			if(initDisp[a][b]<min)
				min=initDisp[a][b];

		}

   for(a=Block/2;a<SIZE_H-Block/2;a++)
   {
	   if(a%50 == 0)
		   printf("\nDense estimation line: %d", a);


		NULL_flag=1;
		for(b=0;b<SIZE_W;b++)
		{
			if(left[a][b].r!=0)
				NULL_flag=0;
		}
		if(NULL_flag==1)
		{
			for(b=0;b<SIZE_W;b++)
				Disparity[a][b]=Min_Range;
			
			continue;
		}


		//edge has twice of weight if the canny_thred is on
	   for(i=0;i<SIZE_W-Block;i++)
	   {
		   if(canny_thred[a][i+Block/2]==255 && 
			   (canny_thred[a-1][i+Block/2-1]==255 || canny_thred[a-1][i+Block/2]==255 
			   || canny_thred[a-1][i+Block/2+1]==255 || canny_thred[a+1][i+Block/2-1]==255 
			   || canny_thred[a+1][i+Block/2]==255 || canny_thred[a+1][i+Block/2+1]==255))

				edgeblock[i]=2*canny[a][i+Block/2];
		   else
			   edgeblock[i]=canny[a][i+Block/2];

		   index[i]=i+Block/2;
	   }

	   for(i=0;i<SIZE_W;i++)
	       barrier[i]=0;

		//sorting
	   for(i=0;i<SIZE_W-Block-1;i++)
		   for(j=SIZE_W-Block-1;j>i;j--)
		   {
			   if(edgeblock[j-1]>edgeblock[j])
			     	continue;
			   temp=edgeblock[j-1];
			   edgeblock[j-1]=edgeblock[j];
               edgeblock[j]=temp;
			   temp=index[j-1];
			   index[j-1]=index[j];
               index[j]=temp;
		   }

	   for(c=0;c<SIZE_W-Block;c++)
	   {
			b=index[c];
			mode=1;
			
			//Search rage  : ֺ 9 initial vector ִ  +-alpha
			end2=Min_Range;
			start2=Max_Range;

			for(i=-1;i<=1;i++)
				for(j=-1;j<=1;j++)
				{
					if(a/Block+i<0 || a/Block+i>SIZE_H/Block-1 || b/Block+j<0 || b/Block+j>SIZE_W/Block-1)
						continue;

					//temp=2*initDisp[a/Block+i][b/Block+j];
					temp=initDisp[a/Block+i][b/Block+j];
					if(temp+alpha>end2)
						end2=temp+alpha;
					if(temp-alpha<start2)
						start2=temp-alpha;
				}
			center=(start2+end2)/2;
			start=start2;
			end=end2;

		   for(i=0;i<SIZE_W-b-1;i++)
		      if(barrier[b+i]==1)
			  {
				 if(i+vector[a][b+i]<end)
					end=i+vector[a][b+i];
				 break;
			  }
		   for(i=0;i>(-1)*b;i--)
			   if(barrier[b+i]==1)
			   {
				   if(i+vector[a][b+i]>start)
						start=i+vector[a][b+i];
				   break;
			   }

		//matching
			err_min=2000000;
			second_min=1000000;

			NCC_max=NCC_second_max=0.0;

			for(t=start;t<=end;t++)
			{
           		if(b+t+Block/2>=SIZE_W)
              		continue;
           		if(b+t-Block/2<0)
              		continue;

				if(matching_method==1)
				{

					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].r;
							Block_sum2+=(float)right[a+x][b+t+y].r;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].r-Block_mean1)*(right[a+x][b+t+y].r-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].r-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].g;
							Block_sum2+=(float)right[a+x][b+t+y].g;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].g-Block_mean1)*(right[a+x][b+t+y].g-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].g-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].b;
							Block_sum2+=(float)right[a+x][b+t+y].b;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].b-Block_mean1)*(right[a+x][b+t+y].b-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].b-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].b-Block_mean2),2);
						}

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC/=3.0;

					if(NCC > NCC_max)
					{
            			NCC_second_max=NCC_max;
						t3=t2;
						NCC_max=NCC;
						t2=t;
					}
					else if(NCC>=NCC_second_max)
					{
            			NCC_second_max=NCC;
						t3=t;
					}
				}

				else	//MAE
				{
					err=0;
					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
							//err+=abs(left_t[a+x][b+y]-right_t[a+x][b+t+y]);
							err+=(abs(left[a+x][b+y].r-right[a+x][b+t+y].r)+abs(left[a+x][b+y].g-right[a+x][b+t+y].g)
							+abs(left[a+x][b+y].b-right[a+x][b+t+y].b));

					if(err<err_min)
					{
						if(abs(t3-t)>Block/2)		//should be far from the best match
						{
            				second_min=err_min;
							t3=t2;
						}
						err_min=err;
						t2=t;
					}
					else if(err<second_min)
					{
						if(abs(t3-t)>Block/2)		//should be far from the best match
						{
            				second_min=err_min;
							t3=t;
						}
					}

				}
			}

		//Bi-directiobal check for the first match

			start=t2-max;
			end=t2-min;
//			end=Range;
//			start=(-1)*Range;
		   for(i=0;i<(SIZE_W-b);i++)
		      if(barrier[b+i]==1)
			  {
				 end=i;
				 break;
			  }
		   for(i=0;i>(-1)*b;i--)
			   if(barrier[b+i]==1)
			   {
				   start=i;
				   break;
			   }


			err_min=20000000;
			NCC_max2=0.0;

			for(t=start;t<=end;t++)
			{
        		if(b+t+mode*Block/2>=SIZE_W)
           		continue;
        		if(b+t-mode*Block/2<0)
           		continue;

				if(matching_method==1)
				{
					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+t+y].r;
							Block_sum2+=(float)right[a+x][b+t2+y].r;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+t+y].r-Block_mean1)*(right[a+x][b+t2+y].r-Block_mean2));
							//Block_sum2+=(right[a+x][b+t2+y].r-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+t+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t2+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+t+y].g;
							Block_sum2+=(float)right[a+x][b+t2+y].g;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+t+y].g-Block_mean1)*(right[a+x][b+t2+y].g-Block_mean2));
							//Block_sum2+=(right[a+x][b+t2+y].g-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+t+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t2+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+t+y].b;
							Block_sum2+=(float)right[a+x][b+t2+y].b;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+t+y].b-Block_mean1)*(right[a+x][b+t2+y].b-Block_mean2));
							//Block_sum2+=(right[a+x][b+t2+y].b-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+t+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t2+y].b-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					NCC/=3.0;

					if(NCC>=NCC_max2)
					{
						NCC_max2=NCC;
						inverse_t=t;
					}
				}

				else
				{
					err=0;

					for(x=(-1)*mode*Block/2;x<mode*Block/2;x++)
						for(y=(-1)*mode*Block/2;y<mode*Block/2;y++)
							//err+=abs(left_t[a+x][b+t+y]-right_t[a+x][b+t2+y]);
							err+=(abs(left[a+x][b+t+y].r-right[a+x][b+t2+y].r)+abs(left[a+x][b+t+y].g-right[a+x][b+t2+y].g)
							+abs(left[a+x][b+t+y].b-right[a+x][b+t2+y].b));

					if(err<err_min)
					{
						err_min=err;
						inverse_t=t;
					}
				}
			}

			if(abs(inverse_t)<=Epsilon)
			{
				vector[a][b]=t2;
				barrier[b]=1;
			}

			else
			{
				if(second_best_matching ==1)
				{
					//Bi-directiobal check for the second-best match
					start=t3-max;
					end=t3-min;

					for(i=0;i<(SIZE_W-b);i++)
						if(barrier[b+i]==1)
						{
							 end=i;
							 break;
						}
					for(i=0;i>(-1)*b;i--)
						   if(barrier[b+i]==1)
					{
						   start=i;
						   break;
					}

					NCC_max2=0.0;
					err_min=20000000;

					for(t=start;t<=end;t++)
					{
        				if(b+t+mode*Block/2>=SIZE_W)
           				continue;
        				if(b+t-mode*Block/2<0)
           				continue;

						if(matching_method==1)
						{
							//NCC of R
							Block_sum1=	Block_sum2=0.0;

							for(x=(-1)*Block/2;x<Block/2;x++)
								for(y=(-1)*Block/2;y<Block/2;y++)
								{
									Block_sum1+=(float)left[a+x][b+t+y].r;
									Block_sum2+=(float)right[a+x][b+t3+y].r;
								}
							Block_mean1=Block_sum1/(Block*Block);
							Block_mean2=Block_sum2/(Block*Block);

							Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

							for(x=(-1)*Block/2;x<Block/2;x++)
								for(y=(-1)*Block/2;y<Block/2;y++)
								{
									Block_sum1+=((left[a+x][b+t+y].r-Block_mean1)*(right[a+x][b+t3+y].r-Block_mean2));
									//Block_sum2+=(right[a+x][b+t3+y].r-Block_mean2);
									Block_sq_sum1+= pow((left[a+x][b+t+y].r-Block_mean1),2);
									Block_sq_sum2+= pow((right[a+x][b+t3+y].r-Block_mean2),2);
								}

							if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
								Block_sq_sum1=1;
							if(Block_sq_sum2==0)
								Block_sq_sum2=1;

							NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

							//NCC of G
							Block_sum1=	Block_sum2=0.0;

							for(x=(-1)*Block/2;x<Block/2;x++)
								for(y=(-1)*Block/2;y<Block/2;y++)
								{
									Block_sum1+=(float)left[a+x][b+t+y].g;
									Block_sum2+=(float)right[a+x][b+t3+y].g;
								}
							Block_mean1=Block_sum1/(Block*Block);
							Block_mean2=Block_sum2/(Block*Block);

							Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

							for(x=(-1)*Block/2;x<Block/2;x++)
								for(y=(-1)*Block/2;y<Block/2;y++)
								{
									Block_sum1+=((left[a+x][b+t+y].g-Block_mean1)*(right[a+x][b+t3+y].g-Block_mean2));
									//Block_sum2+=(right[a+x][b+t3+y].g-Block_mean2);
									Block_sq_sum1+= pow((left[a+x][b+t+y].g-Block_mean1),2);
									Block_sq_sum2+= pow((right[a+x][b+t3+y].g-Block_mean2),2);
								}

							if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
								Block_sq_sum1=1;
							if(Block_sq_sum2==0)
								Block_sq_sum2=1;

							NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

							//NCC of B
							Block_sum1=	Block_sum2=0.0;

							for(x=(-1)*Block/2;x<Block/2;x++)
								for(y=(-1)*Block/2;y<Block/2;y++)
								{
									Block_sum1+=(float)left[a+x][b+t+y].b;
									Block_sum2+=(float)right[a+x][b+t3+y].b;
								}
							Block_mean1=Block_sum1/(Block*Block);
							Block_mean2=Block_sum2/(Block*Block);

							Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

							for(x=(-1)*Block/2;x<Block/2;x++)
								for(y=(-1)*Block/2;y<Block/2;y++)
								{
									Block_sum1+=((left[a+x][b+t+y].b-Block_mean1)*(right[a+x][b+t3+y].b-Block_mean2));
									//Block_sum2+=(right[a+x][b+t3+y].b-Block_mean2);
									Block_sq_sum1+= pow((left[a+x][b+t+y].b-Block_mean1),2);
									Block_sq_sum2+= pow((right[a+x][b+t3+y].b-Block_mean2),2);
								}

							if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
								Block_sq_sum1=1;
							if(Block_sq_sum2==0)
								Block_sq_sum2=1;

							NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

							NCC/=3.0;

							if(NCC>NCC_max2)
							{
								NCC_max2=NCC;
								inverse_t=t;
							}
						}

						else	//MAE
						{
							err=0;

							for(x=(-1)*mode*Block/2;x<mode*Block/2;x++)
								for(y=(-1)*mode*Block/2;y<mode*Block/2;y++)
									//err+=abs(left_t[a+x][b+t+y]-right_t[a+x][b+t3+y]);
									err+=(abs(left[a+x][b+t+y].r-right[a+x][b+t3+y].r)+abs(left[a+x][b+t+y].g-right[a+x][b+t3+y].g)
									+abs(left[a+x][b+t+y].b-right[a+x][b+t3+y].b));

							if(err<err_min)
							{
								err_min=err;
								inverse_t=t;
							}
						}
					}

					if(abs(inverse_t)<=Epsilon)
					{
						vector[a][b]=t3;
						barrier[b]=1;
					}
				}
			}

	   }

	}

	printf("\nDense disparity transfer");
	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
			Disparity[a][b]=vector[a][b];


//Error refinement

	for(a=0;a<SIZE_H/Block;a++)
		for(b=0;b<SIZE_W/Block;b++)
		{
			end2=-1000;
			start2=1000;

			if(Disparity[a][b]==Mark2)
				continue;

			for(i=-1;i<=1;i++)
				for(j=-1;j<=1;j++)
				{
					if(a+i<0 || a+i>SIZE_H/Block-1 || b+j<0 || b+j>SIZE_W/Block-1)
						continue;

//					temp=2*initDisp[a+i][b+j];
					temp=initDisp[a+i][b+j];
					if(temp+alpha>end2)
						end2=temp+alpha;
					if(temp-alpha<start2)
						start2=temp-alpha;
				}

			for(i=0;i<Block; i++)
				for(j=0;j<Block;j++)
				{
					if((Disparity[a*Block+i][b*Block+j]!=Mark2)&&(Disparity[a*Block+i][b*Block+j]>end2 || Disparity[a*Block+i][b*Block+j]<start2))
						Disparity[a*Block+i][b*Block+j]=initDisp[a][b];//2*initDisp[a][b];
				}
		}
  
	Deallo2Int(vector,SIZE_H);

	delete edgeblock;
	delete barrier;
	delete index;
}


void DenseDisparityHierarchy(UnsignedRGB **left, UnsignedRGB **right, int **initDisp, int **Disparity, 
							 int SIZE_H, int SIZE_W, int Block, int Min_Range, int Max_Range)
{
	//Disparity estimation by bidirectional matching with shape adaptive window
   int a,b,x,y,t,start,end,err_min,err;

	//For NCC
	float Block_sum1, Block_sum2;
	float Block_sq_sum1, Block_sq_sum2;
	float Block_mean1,Block_mean2;
	float NCC;
	float NCC_max, NCC_second_max, NCC_max2;

	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
			Disparity[a][b]=initDisp[a][b];

   for(a=Block/2;a<SIZE_H-Block/2;a++)
   {
	   if(a%50 == 0)
		   printf("\nDense estimation line: %d", a);
		
	   for(b=Block/2;b<SIZE_W-Block/2;b++)
	   {
		   if( initDisp[a][b] == Mark2)
			   continue;

			start=end=initDisp[a][b];

			//matching range setting
			if(a%2==0 && b%2==0)
			{
				if(initDisp[a-1][b]!=Mark2 && initDisp[a-1][b]<initDisp[a][b])
					start=initDisp[a-1][b];
				if(initDisp[a][b-1]!=Mark2 && initDisp[a][b-1]<initDisp[a][b])
					start=initDisp[a][b-1];
				if(initDisp[a-1][b]!=Mark2 && initDisp[a-1][b]>initDisp[a][b])
					end=initDisp[a-1][b];
				if(initDisp[a][b-1]!=Mark2 && initDisp[a][b-1]>initDisp[a][b])
					end=initDisp[a][b-1];
			}

			else if(a%2==0 && b%2==1)
			{
				if(initDisp[a-1][b]!=Mark2 && initDisp[a-1][b]<initDisp[a][b])
					start=initDisp[a-1][b];
				if(initDisp[a][b+1]!=Mark2 && initDisp[a][b+1]<initDisp[a][b])
					start=initDisp[a][b+1];
				if(initDisp[a-1][b]!=Mark2 && initDisp[a-1][b]>initDisp[a][b])
					end=initDisp[a-1][b];
				if(initDisp[a][b+1]!=Mark2 && initDisp[a][b+1]>initDisp[a][b])
					end=initDisp[a][b+1];
			}

			else if(a%2==1 && b%2==0)
			{
				if(initDisp[a+1][b]!=Mark2 && initDisp[a+1][b]<initDisp[a][b])
					start=initDisp[a+1][b];
				if(initDisp[a][b-1]!=Mark2 && initDisp[a][b-1]<initDisp[a][b])
					start=initDisp[a][b-1];
				if(initDisp[a+1][b]!=Mark2 && initDisp[a+1][b]>initDisp[a][b])
					end=initDisp[a+1][b];
				if(initDisp[a][b-1]!=Mark2 && initDisp[a][b-1]>initDisp[a][b])
					end=initDisp[a][b-1];
			}
			
			else
			{
				if(initDisp[a+1][b]!=Mark2 && initDisp[a+1][b]<initDisp[a][b])
					start=initDisp[a+1][b];
				if(initDisp[a][b+1]!=Mark2 && initDisp[a][b+1]<initDisp[a][b])
					start=initDisp[a][b+1];
				if(initDisp[a+1][b]!=Mark2 && initDisp[a+1][b]>initDisp[a][b])
					end=initDisp[a+1][b];
				if(initDisp[a][b+1]!=Mark2 && initDisp[a][b+1]>initDisp[a][b])
					end=initDisp[a][b+1];
			}

			start--;
			end++;


			err_min=2000000;
			NCC_max=NCC_second_max=0.0;

			for(t=start;t<=end;t++)
			{
           		if(b+t+Block/2>=SIZE_W)
              		continue;
           		if(b+t-Block/2<0)
              		continue;

				if(matching_method==1)
				{
					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].r;
							Block_sum2+=(float)right[a+x][b+t+y].r;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].r-Block_mean1)*(right[a+x][b+t+y].r-Block_mean2));
							Block_sq_sum1+= pow((left[a+x][b+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].g;
							Block_sum2+=(float)right[a+x][b+t+y].g;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].g-Block_mean1)*(right[a+x][b+t+y].g-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].g-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].b;
							Block_sum2+=(float)right[a+x][b+t+y].b;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].b-Block_mean1)*(right[a+x][b+t+y].b-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].b-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].b-Block_mean2),2);
						}

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC/=3.0;

					if(NCC > NCC_max)
					{
						NCC_max=NCC;
						Disparity[a][b]=t;
					}
				}

				else	//MAE
				{
					err=0;
					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
							err+=(abs(left[a+x][b+y].r-right[a+x][b+t+y].r)+abs(left[a+x][b+y].g-right[a+x][b+t+y].g)
							+abs(left[a+x][b+y].b-right[a+x][b+t+y].b));

					if(err<err_min)
					{
						err_min=err;
						Disparity[a][b]=t;
					}
				}
			}
		}
	}
}

void DenseDisparityHierarchy2(UnsignedRGB **left, UnsignedRGB **right, 
								  int **initDisp, int **Disparity, int **canny, 
								  int **canny_thred, int SIZE_H, int SIZE_W, int Block,
								  int Min_Range, int Max_Range, unsigned char Epsilon)
{
	//Disparity estimation by bidirectional matching with shape adaptive window
   int i,j,a,b,c,x,y,t,temp,start,end,start2,end2,err_min,second_min,err,inverse_t,center;
   int t2=0;
   int t3=0;
   int **vector;   
   int *barrier,*index,*edgeblock;
   int count, old_count;

	//For NCC
	float Block_sum1, Block_sum2;
	float Block_sq_sum1, Block_sq_sum2;
	float Block_mean1,Block_mean2;
	float NCC;
	float NCC_max, NCC_second_max, NCC_max2;

   int alpha=1;
	int maskSize=Block/2;
	int Edge_Th=8;

   barrier=new int[SIZE_W];
   vector=Allo2Int(SIZE_H, SIZE_W);
   index=new int[SIZE_W-Block];
   edgeblock=new int[SIZE_W-Block];

   for(i=0;i<SIZE_H;i++)          //Initialisation
	   for(j=0;j<SIZE_W;j++)
			   vector[i][j]=initDisp[i][j];

   for(a=Block/2;a<SIZE_H-Block/2;a++)
   {
	   if(a%50 == 0)
		   printf("\nDense estimation line: %d", a);

	   for(i=0;i<SIZE_W-Block;i++)
	   {
		   if(canny_thred[a][i+Block/2]==255 && 
			   (canny_thred[a-1][i+Block/2-1]==255 || canny_thred[a-1][i+Block/2]==255 
			   || canny_thred[a-1][i+Block/2+1]==255 || canny_thred[a+1][i+Block/2-1]==255 
			   || canny_thred[a+1][i+Block/2]==255 || canny_thred[a+1][i+Block/2+1]==255))

				edgeblock[i]=2*canny[a][i+Block/2];
		   else
			   edgeblock[i]=canny[a][i+Block/2];

		   index[i]=i+Block/2;
	   }

	   for(i=0;i<SIZE_W;i++)
	       barrier[i]=0;

		//sorting
	   for(i=0;i<SIZE_W-Block-1;i++)
		   for(j=SIZE_W-Block-1;j>i;j--)
		   {
			   if(edgeblock[j-1]>edgeblock[j])
			     	continue;
			   temp=edgeblock[j-1];
			   edgeblock[j-1]=edgeblock[j];
               edgeblock[j]=temp;
			   temp=index[j-1];
			   index[j-1]=index[j];
               index[j]=temp;
		   }

	   for(c=0;c<SIZE_W-Block;c++)
	   {
			b=index[c];
			
			if(initDisp[a][b]==Mark2)
			{
				barrier[b]=1;
				continue;
			}
			
			//Search rage  : ֺ 9 initial vector ִ  +-alpha
			end2=Min_Range;
			start2=Max_Range;

			for(i=-1;i<=1;i++)
				for(j=-1;j<=1;j++)
				{
					if(a+i<0 || a+i>SIZE_H-1 || b+j<0 || b+j>SIZE_W-1)
						continue;
					if(initDisp[a+i][b+j]==Mark2)
						continue;

					temp=initDisp[a+i][b+j];
					if(temp+alpha>end2)
						end2=temp+alpha;
					if(temp-alpha<start2)
						start2=temp-alpha;
				}
			start=start2;
			end=end2;

		   for(i=0;i<SIZE_W-b-1;i++)
		      if(barrier[b+i]==1)
			  {
				 if(i+vector[a][b+i]<end)
					end=i+vector[a][b+i];
				 break;
			  }
		   for(i=0;i>(-1)*b;i--)
			   if(barrier[b+i]==1)
			   {
				   if(i+vector[a][b+i]>start)
						start=i+vector[a][b+i];
				   break;
			   }

		//matching
			err_min=2000000;
			second_min=1000000;

			NCC_max=NCC_second_max=0.0;

			for(t=start;t<=end;t++)
			{
           		if(b+t+Block/2>=SIZE_W)
              		continue;
           		if(b+t-Block/2<0)
              		continue;

				if(matching_method==1)
				{

					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].r;
							Block_sum2+=(float)right[a+x][b+t+y].r;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].r-Block_mean1)*(right[a+x][b+t+y].r-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].r-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].g;
							Block_sum2+=(float)right[a+x][b+t+y].g;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].g-Block_mean1)*(right[a+x][b+t+y].g-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].g-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].b;
							Block_sum2+=(float)right[a+x][b+t+y].b;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].b-Block_mean1)*(right[a+x][b+t+y].b-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].b-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].b-Block_mean2),2);
						}

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC/=3.0;

					if(NCC > NCC_max)
					{
            			NCC_second_max=NCC_max;
						t3=t2;
						NCC_max=NCC;
						t2=t;
					}
					else if(NCC>=NCC_second_max)
					{
            			NCC_second_max=NCC;
						t3=t;
					}
				}

				else	//MAE
				{
					err=0;
					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
							err+=(abs(left[a+x][b+y].r-right[a+x][b+t+y].r)+abs(left[a+x][b+y].g-right[a+x][b+t+y].g)
							+abs(left[a+x][b+y].b-right[a+x][b+t+y].b));

					if(err<err_min)
					{
						err_min=err;
						t2=t;
					}
				}
			}

			vector[a][b]=t2;
			barrier[b]=1;

	   }
	}

	printf("\nDense disparity transfer");
	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
			Disparity[a][b]=vector[a][b];


//Error refinement

	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
		{
			end2=-1000;
			start2=1000;

			if(Disparity[a][b]==Mark2)
				continue;

			for(i=-1;i<=1;i++)
				for(j=-1;j<=1;j++)
				{
					if(a+i<0 || a+i>SIZE_H-1 || b+j<0 || b+j>SIZE_W-1)
						continue;

					if(Disparity[a+i][b+j]==Mark2)
						continue;

					temp=initDisp[a+i][b+j];
					if(temp+alpha>end2)
						end2=temp+alpha;
					if(temp-alpha<start2)
						start2=temp-alpha;
				}

			if((Disparity[a][b]!=Mark2)&&(Disparity[a][b]>end2 || Disparity[a][b]<start2))
					Disparity[a][b]=initDisp[a][b];//2*initDisp[a][b];

		}
  
	Deallo2Int(vector,SIZE_H);

	delete edgeblock;
	delete barrier;
	delete index;
}
/*
void DenseDisparityHierarchy2(UnsignedRGB **left, UnsignedRGB **right, 
								  int **initDisp, int **Disparity, int **canny, 
								  int **canny_thred, int SIZE_H, int SIZE_W, int Block,
								  int Min_Range, int Max_Range, unsigned char Epsilon)
{
	//Disparity estimation by bidirectional matching with shape adaptive window
   int i,j,a,b,c,x,y,t,temp,start,end,start2,end2,err_min,second_min,err,inverse_t,center;
   int t2=0;
   int t3=0;
   int **vector;   
   int *barrier,*index,*edgeblock;
   int min, max;
   int count, old_count;

	//For NCC
	float Block_sum1, Block_sum2;
	float Block_sq_sum1, Block_sq_sum2;
	float Block_mean1,Block_mean2;
	float NCC;
	float NCC_max, NCC_second_max, NCC_max2;

   int alpha=1;
	int maskSize=Block/2;
	int Edge_Th=8;

   barrier=new int[SIZE_W];
   vector=Allo2Int(SIZE_H, SIZE_W);
   index=new int[SIZE_W-Block];
   edgeblock=new int[SIZE_W-Block];

   for(i=0;i<SIZE_H;i++)          //Initialisation
	   for(j=0;j<SIZE_W;j++)
			   vector[i][j]=initDisp[i][j];

	min=2000;
	max=-2000;
	for(a=0;a<SIZE_H/Block;a++)
		for(b=0;b<SIZE_W/Block;b++)
		{
			if(initDisp[a][b]>max)
				max=initDisp[a][b];
			if(initDisp[a][b]<min)
				min=initDisp[a][b];
		}

   for(a=Block/2;a<SIZE_H-Block/2;a++)
   {
	   if(a%50 == 0)
		   printf("\nDense estimation line: %d", a);

	   for(i=0;i<SIZE_W-Block;i++)
	   {
		   if(canny_thred[a][i+Block/2]==255 && 
			   (canny_thred[a-1][i+Block/2-1]==255 || canny_thred[a-1][i+Block/2]==255 
			   || canny_thred[a-1][i+Block/2+1]==255 || canny_thred[a+1][i+Block/2-1]==255 
			   || canny_thred[a+1][i+Block/2]==255 || canny_thred[a+1][i+Block/2+1]==255))

				edgeblock[i]=2*canny[a][i+Block/2];
		   else
			   edgeblock[i]=canny[a][i+Block/2];

		   index[i]=i+Block/2;
	   }

	   for(i=0;i<SIZE_W;i++)
	       barrier[i]=0;

		//sorting
	   for(i=0;i<SIZE_W-Block-1;i++)
		   for(j=SIZE_W-Block-1;j>i;j--)
		   {
			   if(edgeblock[j-1]>edgeblock[j])
			     	continue;
			   temp=edgeblock[j-1];
			   edgeblock[j-1]=edgeblock[j];
               edgeblock[j]=temp;
			   temp=index[j-1];
			   index[j-1]=index[j];
               index[j]=temp;
		   }

	   for(c=0;c<SIZE_W-Block;c++)
	   {
			b=index[c];
			
			//Search rage  : ֺ 9 initial vector ִ  +-alpha
			end2=Min_Range;
			start2=Max_Range;

			for(i=-1;i<=1;i++)
				for(j=-1;j<=1;j++)
				{
					if(a+i<0 || a+i>SIZE_H-1 || b+j<0 || b+j>SIZE_W-1)
						continue;

					temp=initDisp[a+i][b+j];
					if(temp+alpha>end2)
						end2=temp+alpha;
					if(temp-alpha<start2)
						start2=temp-alpha;
				}
			center=(start2+end2)/2;
			start=start2;
			end=end2;

		   for(i=0;i<SIZE_W-b-1;i++)
		      if(barrier[b+i]==1)
			  {
				 if(i+vector[a][b+i]<end)
					end=i+vector[a][b+i];
				 break;
			  }
		   for(i=0;i>(-1)*b;i--)
			   if(barrier[b+i]==1)
			   {
				   if(i+vector[a][b+i]>start)
						start=i+vector[a][b+i];
				   break;
			   }

		//matching
			err_min=2000000;
			second_min=1000000;

			NCC_max=NCC_second_max=0.0;

			for(t=start;t<=end;t++)
			{
           		if(b+t+Block/2>=SIZE_W)
              		continue;
           		if(b+t-Block/2<0)
              		continue;

				if(matching_method==1)
				{

					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].r;
							Block_sum2+=(float)right[a+x][b+t+y].r;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].r-Block_mean1)*(right[a+x][b+t+y].r-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].r-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].g;
							Block_sum2+=(float)right[a+x][b+t+y].g;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].g-Block_mean1)*(right[a+x][b+t+y].g-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].g-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+y].b;
							Block_sum2+=(float)right[a+x][b+t+y].b;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+y].b-Block_mean1)*(right[a+x][b+t+y].b-Block_mean2));
							//Block_sum2+=(right[a+x][b+t+y].b-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t+y].b-Block_mean2),2);
						}

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC/=3.0;

					if(NCC > NCC_max)
					{
            			NCC_second_max=NCC_max;
						t3=t2;
						NCC_max=NCC;
						t2=t;
					}
					else if(NCC>=NCC_second_max)
					{
            			NCC_second_max=NCC;
						t3=t;
					}
				}

				else	//MAE
				{
					err=0;
					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
							err+=(abs(left[a+x][b+y].r-right[a+x][b+t+y].r)+abs(left[a+x][b+y].g-right[a+x][b+t+y].g)
							+abs(left[a+x][b+y].b-right[a+x][b+t+y].b));

					if(err<err_min)
					{
						if(abs(t3-t)>Block/2)		//should be far from the best match
						{
            				second_min=err_min;
							t3=t2;
						}
						err_min=err;
						t2=t;
					}
					else if(err<second_min)
					{
						if(abs(t3-t)>Block/2)		//should be far from the best match
						{
            				second_min=err_min;
							t3=t;
						}
					}

				}
			}

		//Bi-directiobal check for the first match

			start=t2-max;
			end=t2-min;

		   for(i=0;i<(SIZE_W-b);i++)
		      if(barrier[b+i]==1)
			  {
				 end=i;
				 break;
			  }
		   for(i=0;i>(-1)*b;i--)
			   if(barrier[b+i]==1)
			   {
				   start=i;
				   break;
			   }


			err_min=20000000;
			NCC_max2=0.0;

			for(t=start;t<=end;t++)
			{
        		if(b+t+Block/2>=SIZE_W)
           		continue;
        		if(b+t-Block/2<0)
           		continue;

				if(matching_method==1)
				{
					//NCC of R
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+t+y].r;
							Block_sum2+=(float)right[a+x][b+t2+y].r;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+t+y].r-Block_mean1)*(right[a+x][b+t2+y].r-Block_mean2));
							//Block_sum2+=(right[a+x][b+t2+y].r-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+t+y].r-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t2+y].r-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC = Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block);

					//NCC of G
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+t+y].g;
							Block_sum2+=(float)right[a+x][b+t2+y].g;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+t+y].g-Block_mean1)*(right[a+x][b+t2+y].g-Block_mean2));
							//Block_sum2+=(right[a+x][b+t2+y].g-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+t+y].g-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t2+y].g-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					//NCC of B
					Block_sum1=	Block_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=(float)left[a+x][b+t+y].b;
							Block_sum2+=(float)right[a+x][b+t2+y].b;
						}
					Block_mean1=Block_sum1/(Block*Block);
					Block_mean2=Block_sum2/(Block*Block);

					Block_sum1=Block_sum2=Block_sq_sum1=Block_sq_sum2=0.0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
						{
							Block_sum1+=((left[a+x][b+t+y].b-Block_mean1)*(right[a+x][b+t2+y].b-Block_mean2));
							//Block_sum2+=(right[a+x][b+t2+y].b-Block_mean2);
							Block_sq_sum1+= pow((left[a+x][b+t+y].b-Block_mean1),2);
							Block_sq_sum2+= pow((right[a+x][b+t2+y].b-Block_mean2),2);
						}

						if(Block_sq_sum1==0)	//avoid devided by 0 from saturation
							Block_sq_sum1=1;
						if(Block_sq_sum2==0)
							Block_sq_sum2=1;

					NCC += (Block_sum1/(sqrt(Block_sq_sum1)*sqrt(Block_sq_sum2)*Block*Block));

					NCC/=3.0;

					if(NCC>=NCC_max2)
					{
						NCC_max2=NCC;
						inverse_t=t;
					}
				}

				else
				{
					err=0;

					for(x=(-1)*Block/2;x<Block/2;x++)
						for(y=(-1)*Block/2;y<Block/2;y++)
							//err+=abs(left_t[a+x][b+t+y]-right_t[a+x][b+t2+y]);
							err+=(abs(left[a+x][b+t+y].r-right[a+x][b+t2+y].r)+abs(left[a+x][b+t+y].g-right[a+x][b+t2+y].g)
							+abs(left[a+x][b+t+y].b-right[a+x][b+t2+y].b));

					if(err<err_min)
					{
						err_min=err;
						inverse_t=t;
					}
				}
			}

			if(abs(inverse_t)<=Epsilon)
			{
				vector[a][b]=t2;
				barrier[b]=1;
			}

			else
			{
				vector[a][b]=Mark2;
			}

	   }

	}

	printf("\nDense disparity transfer");
	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
			Disparity[a][b]=vector[a][b];


//Error refinement

	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
		{
			end2=-1000;
			start2=1000;

			if(Disparity[a][b]==Mark2)
				continue;

			for(i=-1;i<=1;i++)
				for(j=-1;j<=1;j++)
				{
					if(a+i<0 || a+i>SIZE_H-1 || b+j<0 || b+j>SIZE_W-1)
						continue;

					temp=initDisp[a+i][b+j];
					if(temp+alpha>end2)
						end2=temp+alpha;
					if(temp-alpha<start2)
						start2=temp-alpha;
				}

			if((Disparity[a][b]!=Mark2)&&(Disparity[a][b]>end2 || Disparity[a][b]<start2))
					Disparity[a][b]=initDisp[a][b];//2*initDisp[a][b];

		}
  
	Deallo2Int(vector,SIZE_H);

	delete edgeblock;
	delete barrier;
	delete index;
}
*/
void boundary(int **Disparity, int SIZE_H, int SIZE_W, int Block)
{
	//迭 ʰ Ʒ  ä
	int a,b;

	for(a=0;a<Block/2;a++)
		for(b=0;b<SIZE_W;b++)
      	Disparity[a][b]=Disparity[Block/2][b];
	for(a=SIZE_H-Block/2;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
      	Disparity[a][b]=Disparity[SIZE_H-Block/2-1][b];
}

void OcclusionFilling(int **Disparity, int SIZE_H, int SIZE_W)
{
	int i,j;
	
	// 5ȼ иƼ ̵ ä	
	const int L=5;
	int N;
	int a,b;
	int P[L];
	int temp,d;

	for(i=0;i<SIZE_H;i++)
		for(j=1;j<SIZE_W-1;j++)
			if(Disparity[i][j]==Mark2)
			{
				if(j<L)
					N=j;
				else
					N=L;

				for(a=0;a<N;a++)
					P[a]=Disparity[i][j-a-1];

				for(a=0;a<N-1;a++)
					for(b=N-1;b>a;b--)
					{
  						if(P[b-1]>P[b])
     						continue;
						temp=P[b-1];
						P[b-1]=P[b];
						P[b]=temp;
					}
				d=P[N/2];

				for(a=j;a<SIZE_W-2;a++)
				{
					Disparity[i][a]=d;
					if(Disparity[i][a+2]!=Mark2)
						break;
				}
			}
}


void DisparityRegularization(unsigned char **left, unsigned char **right, float **Disparity, 
							 int SIZE_H, int SIZE_W, float beta, int lambda, int iteration)
{
	//energy function  regularization

	int i,j,t2,t;
	double **f, **fn,**bx, **by, **temp1dmat, **temp2dmat, **updated, **laplacian, **fidelity;
	double **Dxf, **Dyf, **grad_r;
	int **tempimat;
	double tempd,tempd2,error,old_error;
	int tempi,sign;
	double mean, var;

	old_error=100.0;

	fn=Allo2Double(SIZE_H,SIZE_W);
	f=Allo2Double(SIZE_H,SIZE_W);
	bx=Allo2Double(SIZE_H,SIZE_W);
	by=Allo2Double(SIZE_H,SIZE_W);
	temp1dmat=Allo2Double(SIZE_H,SIZE_W);
	temp2dmat=Allo2Double(SIZE_H,SIZE_W);
	updated=Allo2Double(SIZE_H,SIZE_W);
	laplacian=Allo2Double(SIZE_H,SIZE_W);
	fidelity=Allo2Double(SIZE_H,SIZE_W);
	Dxf=Allo2Double(SIZE_H,SIZE_W);
	Dyf=Allo2Double(SIZE_H,SIZE_W);
	grad_r=Allo2Double(SIZE_H,SIZE_W);
	tempimat=Allo2Int(SIZE_H,SIZE_W);

	for(i=0;i<SIZE_H;i++)			//    disparity  Ƿ extrapolation
	{
		Disparity[i][0]=Disparity[i][1];
		Disparity[i][SIZE_W-1]=Disparity[i][SIZE_W-2];
	}

	//ʱġ
	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
			f[i][j]=Disparity[i][j];

	// gradient
	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
			temp2dmat[i][j]=(double)left[i][j];
	Dx(temp2dmat,Dxf,3, SIZE_H, SIZE_W);
	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
			temp2dmat[i][j]=(double)left[i][j];
	Dy(temp2dmat,Dyf,3, SIZE_H, SIZE_W);

	//diffusion tensor ϱ
	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
		{
			tempd=Dxf[i][j];
			bx[i][j]=1.0/((1+tempd*tempd)*(1+tempd*tempd));
			//bx[i][j]=1.0/(sqrt(1+tempd*tempd));
			tempd=Dyf[i][j];
			by[i][j]=1.0/((1+tempd*tempd)*(1+tempd*tempd));
			//by[i][j]=1.0/(sqrt(1+tempd*tempd));
		}

	{
		//f(n+1) by steepest decent
		//iteration

		for(i=0;i<SIZE_H;i++)
			for(j=0;j<SIZE_W;j++)
			{
				fn[i][j]=f[i][j];
			}

			for(t2=0;t2<iteration;t2++)
			{
				if(t2%50 == 0)
					printf("\nitenration = %d",t2);

				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
					{
						tempi=(int)(fn[i][j]);
						if(j+tempi>=SIZE_W || j+tempi<0)
							temp2dmat[i][j]=(double)right[i][j];
						else
							temp2dmat[i][j]=(double)(right[i][j+tempi]);
					}
				Dx(temp2dmat,grad_r,3, SIZE_H, SIZE_W);

				for(i=0;i<SIZE_H;i++)		//(left[i][j]-right[i][j+disparity])grad(right[i][j+disparity])
					for(j=0;j<SIZE_W;j++)
					{
						tempi=(int)(fn[i][j]);			//disparity  κ
						tempd=fn[i][j]-tempi;		//disparity Ҽ κ

						if(j+tempi+1>=SIZE_W)
							fidelity[i][j]=0;
						else if (j+tempi<0)
							fidelity[i][j]=0;
						else
							// Ʋ...   ߳⵵ . Ʒ update İ  ̷
							fidelity[i][j]=grad_r[i][j]*(left[i][j]-((1.0-tempd)*right[i][j+tempi]+tempd*right[i][j+tempi+1]));
							// ..  Ʒ updateİ  ̷
							//fidelity[i][j]=grad_r[i][j]*(left[i][j]-((1.0-tempd)*right[i][j+tempi]+tempd*right[i][j+tempi+1])+fn[i][j]*grad_r[i][j]);
					}

				//div(b grad(f))
				Dx(fn,Dxf,1, SIZE_H, SIZE_W);
				Dy(fn,Dyf,1, SIZE_H, SIZE_W);
				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
						//laplacian[i][j]=Dxf[i][j];	// öþ
						laplacian[i][j]=bx[i][j]*Dxf[i][j];

				for(i=0;i<SIZE_H;i++)		//Laplacian   shift
					for(j=0;j<SIZE_W;j++)
					{
						if(j-1<0)
							temp1dmat[i][j]=laplacian[i][SIZE_W-j-1];
						else
							temp1dmat[i][j]=laplacian[i][j-1];
					}

				Dx(temp1dmat,Dxf,1, SIZE_H, SIZE_W);

				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
						//laplacian[i][j]=Dyf[i][j];	// öþ
						laplacian[i][j]=by[i][j]*Dyf[i][j];

				for(i=0;i<SIZE_H;i++)		//Laplacian   shift
					for(j=0;j<SIZE_W;j++)
					{
						if(i-1<0)
							temp1dmat[i][j]=laplacian[SIZE_H-i-1][j];
						else
							temp1dmat[i][j]=laplacian[i-1][j];
					}

				Dy(temp1dmat,Dyf,1, SIZE_H, SIZE_W);

				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
						laplacian[i][j]=Dxf[i][j]+Dyf[i][j];

				//update
				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
					{
						//׳ expliciteϰ °
						//updated[i][j]=fn[i][j]+beta*(0.003*fidelity[i][j]+lambda*laplacian[i][j]);
						//  Ʋ..  smoothing.. ٵ ̰  ߳⵵ 
						updated[i][j]=fn[i][j] + (beta/(1+beta*grad_r[i][j]*grad_r[i][j])) * (0.003*fidelity[i][j]+lambda*laplacian[i][j]);
						// Ĵ...
						//updated[i][j]=(1.0/(1+beta*grad_r[i][j]*grad_r[i][j])) * (fn[i][j] + beta*fidelity[i][j]+beta*lambda*laplacian[i][j]);
					}

				//converge Ȯ
				error=0.0;
				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
					{
						error+=abs(updated[i][j]-fn[i][j]);
					}

				for(i=0;i<SIZE_H;i++)
					for(j=0;j<SIZE_W;j++)
					{
						fn[i][j]=updated[i][j];
					}

				old_error=error;
		
			}

		for(i=0;i<SIZE_H;i++)
			for(j=0;j<SIZE_W;j++)
				Disparity[i][j]=fn[i][j];
	}
	
	Deallo2Double(fn,SIZE_H);
	Deallo2Double(f,SIZE_H);
	Deallo2Double(bx,SIZE_H);
	Deallo2Double(by,SIZE_H);
	Deallo2Double(temp1dmat,SIZE_H);
	Deallo2Double(temp2dmat,SIZE_H);
	Deallo2Double(updated,SIZE_H);
	Deallo2Double(laplacian,SIZE_H);
	Deallo2Double(fidelity,SIZE_H);
	Deallo2Double(Dxf,SIZE_H);
	Deallo2Double(Dyf,SIZE_H);
	Deallo2Double(grad_r,SIZE_H);
	Deallo2Int(tempimat,SIZE_H);

}




/////////////////////////////////////////////
/// Main function
////////////////////////////////////////////////

void DisparityWithRegularization(UnsignedRGB **left, UnsignedRGB **right, IplImage *left_org_grey, IplImage *right_org_grey,
								 float **Disparity, int  SIZE_W, int SIZE_H, unsigned char hierarchy, unsigned char Block,
								 int Min_Range, int Max_Range, float scale, unsigned char Epsilon,
								 int Region_Th, float beta, int lambda, int iteration, 
								 int canny_Th, char *path, int ini_disp)
{

	int i,j,a,b,sum;
	int down_ratio;
	int tempi;
	double tempd;
	UnsignedRGB **left_down, **right_down;
	int **DisparityInt;
	CvSize output_size;

	sprintf(OUTPUT_PATH, "%s", path);

	//Initial disparity estimation
	down_ratio	= pow(2.0,hierarchy);
	initDisp	= Allo2Int(SIZE_H/(down_ratio*Block),SIZE_W/(down_ratio*Block));

	left_down	= Allo2RGB(SIZE_H/down_ratio,SIZE_W/down_ratio);
	right_down	= Allo2RGB(SIZE_H/down_ratio,SIZE_W/down_ratio);
	
	DownURGB(left, left_down, SIZE_H, SIZE_W, down_ratio);
	DownURGB(right, right_down, SIZE_H, SIZE_W, down_ratio);


	//Feature extraction (Canny)
	printf("\nFeature extraction");
	canny		= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);
	canny_thred	= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);
	canny_detector(left_down, canny, canny_thred,SIZE_H/down_ratio, SIZE_W/down_ratio, canny_Th);
	
	CvSize size=cvSize(SIZE_W/down_ratio, SIZE_H/down_ratio);
	IplImage*		result		= cvCreateImage( size, 8, 1 );

	for(i=0; i<SIZE_H/down_ratio; i++)
		for(j=0; j<SIZE_W/down_ratio; j++)
		{
			tempi=canny[i][j];

			if (tempi<0)
				cvSetReal2D(result,i,j,0);
			else if(tempi> 255)
				cvSetReal2D(result,i,j,255);
			else
				cvSetReal2D(result,i,j,tempi);
		}
	sprintf( filename, "%s//00canny.png", BASE_PATH);
	cvSaveImage(filename, result);
	cvReleaseImage(&result);


	//Initial disparity

	if(initial_disp_from_file==1)
	{
		printf("\nLoading Initial disparity from the file");
		printf("\nConfirm the scale parameters and if the disparity image size is %d by %d\n", SIZE_W/(down_ratio*Block), SIZE_H/(down_ratio*Block));

		IplImage *ini_org;
		sprintf(filename, "%s%s", BASE_PATH, INITIAL_IMAGE);
		if((ini_org=cvLoadImage(filename)) == 0)
		{
			printf("Cannot open the initial disparity image - %s\n",filename);
			exit(0);
		}

		output_size.width=SIZE_W/(down_ratio*Block);
		output_size.height=SIZE_H/(down_ratio*Block);
		result	= cvCreateImage( output_size, 8, 1 );

		cvSplit(ini_org, result, NULL, NULL, NULL);


		for(i=0; i<output_size.height; i++)
			for(j=0; j<output_size.width; j++)
			{
				tempi=cvGetReal2D(result,i,j);
				tempi=(int)((tempi/scale+Min_Range)/(down_ratio));///(2*down_ratio));
				initDisp[i][j]=tempi;
			}
		cvReleaseImage(&result);
		cvReleaseImage(&ini_org);
	}

	else if(initial_disp_from_file==0)
	{
		printf("\nInitial disparity estimation");

		InitDisparityEstimation(left_down, right_down, initDisp, canny, SIZE_H/down_ratio, SIZE_W/down_ratio,
								 Block, Min_Range/down_ratio, Max_Range/down_ratio, Epsilon);

		output_size.width=SIZE_W/(down_ratio*Block);
		output_size.height=SIZE_H/(down_ratio*Block);

		result	= cvCreateImage( output_size, 8, 1 );

		for(i=0; i<output_size.height; i++)
			for(j=0; j<output_size.width; j++)
			{
				tempi=(int)(scale* (initDisp[i][j]*down_ratio - Min_Range));

				if (tempi<0)
					cvSetReal2D(result,i,j,0);
				else if(tempi> 255)
					cvSetReal2D(result,i,j,255);
				else
					cvSetReal2D(result,i,j,tempi);
			}

		sprintf( filename, "%s//01Initial-g_%d_%d.png", BASE_PATH, (int)(scale*down_ratio), (int)Min_Range);
		cvSaveImage(filename, result);
		cvReleaseImage(&result);
		//SaveBmpScaling(initDisp,SIZE_H/Block,SIZE_W/Block, "01Initial.bmp");
	}
	Deallo2RGB(left_down, SIZE_H/down_ratio);
	Deallo2RGB(right_down, SIZE_H/down_ratio);
	
	//Dense disparity estimation

	if(initial_disp_from_file==2)		//read dense disparity from file
	{
		DisparityInt=	Allo2Int(SIZE_H,SIZE_W);
		load_dense_disp(DisparityInt, SIZE_H, SIZE_W, scale, hierarchy, Min_Range, Max_Range );
	}
	else	//dense disparity extimation
	{
		printf("\nDense disparity");
		printf("\nHierarchy level: %d", hierarchy);

		down_ratio	= pow(2.0,hierarchy);
		DisparityInt	= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);

		left_down	= Allo2RGB(SIZE_H/down_ratio,SIZE_W/down_ratio);
		right_down	= Allo2RGB(SIZE_H/down_ratio,SIZE_W/down_ratio);
		
		DownURGB(left, left_down, SIZE_H, SIZE_W, down_ratio);
		DownURGB(right, right_down, SIZE_H, SIZE_W, down_ratio);

		DenseDisparityRegionDividing(left_down, right_down, initDisp, DisparityInt, canny, 
			canny_thred, SIZE_H/down_ratio, SIZE_W/down_ratio, Block, Min_Range/down_ratio, Max_Range/down_ratio, Epsilon);

		Deallo2Int(canny, SIZE_H/down_ratio);
		Deallo2Int(canny_thred, SIZE_H/down_ratio);
		Deallo2Int(initDisp, SIZE_H/(down_ratio*Block));
		Deallo2RGB(left_down, SIZE_H/down_ratio);
		Deallo2RGB(right_down, SIZE_H/down_ratio);

		output_size.width=SIZE_W/down_ratio;
		output_size.height=SIZE_H/down_ratio;

		result	= cvCreateImage( output_size, 8, 1 );

		for(i=0; i<output_size.height; i++)
			for(j=0; j<output_size.width; j++)
			{
				if(DisparityInt[i][j]==Mark2)	//exception
				{
					cvSetReal2D(result,i,j,0);
					continue;
				}

				tempi=(int)(scale* (DisparityInt[i][j]*down_ratio - Min_Range) + 0.5);

				if (tempi<0)
					cvSetReal2D(result,i,j,0);
				else if(tempi> 255)
					cvSetReal2D(result,i,j,255);
				else
					cvSetReal2D(result,i,j,tempi);
			}

		sprintf( filename, "%s//02Dense_%d-g_%d_%d.png", BASE_PATH, 0,(int)(scale*down_ratio), (int)Min_Range);
		cvSaveImage(filename, result);
		cvReleaseImage(&result);

		//Save intermediate output
		if(rotation==1)
		{
			IplImage*		result1		= cvCreateImage( Imagesize, 8, 1 );
			cvSet(result1, cvScalar(0));

			for(i=Imagesize.height/20; i<Imagesize.height-(Imagesize.height/8); i++)
				for(j=8; j<Imagesize.width-8; j++)
				{
					int temp=(int)(scale * (DisparityInt[Imagesize_ext.width-j-1][i]*down_ratio - Min_Range) + 0.5);

					if (temp<0)
						cvSetReal2D(result1,i,j,0);
					else if(temp> 255)
						cvSetReal2D(result1,i,j,255);
					else
						cvSetReal2D(result1,i,j,temp);
				}
			sprintf( filename, "%s//05DenseDisparity-g_%d_%d.png", BASE_PATH, (int)(scale*down_ratio), (int)Min_Range);
			cvSaveImage(filename, result1);
			cvReleaseImage(&result1);
		}


		//Hierarchical processing
		for (int t=hierarchy-1; t>=0; t--)
		{
			printf("\n\nHierarchy level: %d", t);
			down_ratio	= pow(2.0,t);
			initDisp	= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);

			for(i=0;i<SIZE_H/down_ratio;i++)
				for(j=0;j<SIZE_W/down_ratio;j++)
				{
					if(DisparityInt[i/2][j/2]==Mark2)
						initDisp[i][j] = Mark2;
					else
						initDisp[i][j] = 2*DisparityInt[i/2][j/2];
				}
			
			LowfilteringExc(SIZE_H/down_ratio, SIZE_W/down_ratio, 1, Mark2, initDisp, initDisp);

			Deallo2Int(DisparityInt, SIZE_H/(down_ratio*2));
			
			DisparityInt= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);
			left_down	= Allo2RGB(SIZE_H/down_ratio,SIZE_W/down_ratio);
			right_down	= Allo2RGB(SIZE_H/down_ratio,SIZE_W/down_ratio);
			
			DownURGB(left, left_down, SIZE_H, SIZE_W, down_ratio);
			DownURGB(right, right_down, SIZE_H, SIZE_W, down_ratio);

			canny		= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);
			canny_thred	= Allo2Int(SIZE_H/down_ratio,SIZE_W/down_ratio);
			canny_detector(left_down, canny, canny_thred,SIZE_H/down_ratio, SIZE_W/down_ratio, canny_Th);
	
			//DenseDisparityHierarchy(left_down, right, initDisp, DisparityInt, SIZE_H/down_ratio, 
			//	SIZE_W/down_ratio, Block, Min_Range/down_ratio, Max_Range/down_ratio);

			DenseDisparityHierarchy2(left_down, right_down, initDisp, DisparityInt, canny, 
				canny_thred, SIZE_H/down_ratio, SIZE_W/down_ratio, Block, Min_Range/down_ratio, Max_Range/down_ratio, Epsilon);

			Deallo2Int(initDisp, SIZE_H/down_ratio);

			output_size.width=SIZE_W/down_ratio;
			output_size.height=SIZE_H/down_ratio;

			result	= cvCreateImage( output_size, 8, 1 );

			for(i=0; i<output_size.height; i++)
				for(j=0; j<output_size.width; j++)
				{
					if(DisparityInt[i][j]==Mark2)	//exception
					{
						cvSetReal2D(result,i,j,0);
						continue;
					}

					tempi=(int)(scale* (DisparityInt[i][j]*down_ratio - Min_Range) + 0.5);

					if (tempi<0)
						cvSetReal2D(result,i,j,0);
					else if(tempi> 255)
						cvSetReal2D(result,i,j,255);
					else
						cvSetReal2D(result,i,j,tempi);
				}

			sprintf( filename, "%s//02Dense_%d-g_%d_%d.png", BASE_PATH, hierarchy-t,(int)(scale*down_ratio), (int)Min_Range);
			cvSaveImage(filename, result);
			cvReleaseImage(&result);


			Deallo2Int(canny, SIZE_H/(down_ratio));
			Deallo2Int(canny_thred, SIZE_H/(down_ratio));
		}

	}

	//Disparity Assignment
	printf("\nOcclusion assingment");
	occlusion=	Allo2Int(SIZE_H, SIZE_W);
	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
		{
			if(DisparityInt[i][j]==Mark2)
				occlusion[i][j]=1;
			else
				occlusion[i][j]=0;
		}
	
	//Disparity setting ???
	printf("\nDisparity setting");		
	boundary(DisparityInt, SIZE_H, SIZE_W, Block);
	/*
	for(i=0;i<SIZE_H;i++)
   		for(j=0;j<SIZE_W;j++)
			if(DisparityInt[i][j]>-5 && DisparityInt[i][j]<4)
				DisparityInt[i][j]=-5;
	*/

	printf("\nOcclusion filling");
	//Occlusion filling with background disparity (from left)
	OcclusionFilling(DisparityInt,SIZE_H, SIZE_W);

	for(i=0;i<SIZE_H;i++)
   		for(j=1;j<SIZE_W;j++)
		{
			if(DisparityInt[i][j]==Mark2)
				DisparityInt[i][j]=DisparityInt[i][j-1];
		}


	output_size.width=SIZE_W;
	output_size.height=SIZE_H;

	result	= cvCreateImage( output_size, 8, 1 );

	for(i=0; i<output_size.height; i++)
		for(j=0; j<output_size.width; j++)
		{
			if(DisparityInt[i][j]==Mark2)	//exception
			{
				cvSetReal2D(result,i,j,0);
				continue;
			}

			tempi=(int)(scale* (DisparityInt[i][j] - Min_Range) + 0.5);

			if (tempi<0)
				cvSetReal2D(result,i,j,0);
			else if(tempi> 255)
				cvSetReal2D(result,i,j,255);
			else
				cvSetReal2D(result,i,j,tempi);
		}

	sprintf( filename, "%s//03Occlusion-g_%d_%d.png", BASE_PATH, (int)(scale*pow(2.0,hierarchy)), (int)Min_Range);
	cvSaveImage(filename, result);
	cvReleaseImage(&result);
	//SaveBmpScalingException(DisparityInt,SIZE_H,SIZE_W,Mark2,"03Occlusion.bmp");
	Deallo2Int(occlusion, SIZE_H);


	//Regularization
	printf("\n\n\nRegularization");

	unsigned char **left_grey, **right_grey;
	IplImage *left_g_down, *right_g_down;
	float **disp_temp, **disp_old;
	CvSize org_size, low_size;

	org_size	= cvGetSize( left_org_grey );

	hierarchy=0;

	for (int t=hierarchy; t>=0; t--)
	{
		printf("\nHierarchy level: %d", t);

		disp_temp	=	Allo2Float(SIZE_H, SIZE_W);
		left_grey	=	Allo2UC(SIZE_H, SIZE_W);
		right_grey	=	Allo2UC(SIZE_H, SIZE_W);

		if(t==hierarchy)
		{
			low_size.height=org_size.height/(pow(2.0,t));
			low_size.width=org_size.width/(pow(2.0,t));

			for(i=0;i<SIZE_H;i++)
   				for(j=0;j<SIZE_W;j++)
				{
					left_grey[i][j]		= (unsigned char)(0.299*left[i][j].r + 0.587*left[i][j].g + 0.114*left[i][j].b);
					right_grey[i][j]	= (unsigned char)(0.299*right[i][j].r + 0.587*right[i][j].g + 0.114*right[i][j].b);
					disp_temp[i][j]		= (float)DisparityInt[i][j];
				}
		}
		else
		{
			low_size.height=org_size.height/(pow(2.0,t));
			low_size.width=org_size.width/(pow(2.0,t));
			left_g_down		= cvCreateImage( low_size, 8, 1 );
			right_g_down		= cvCreateImage( low_size, 8, 1 );

			cvResize(left_org_grey,left_g_down);
			cvResize(right_org_grey,right_g_down);

			for(i=0;i<SIZE_H;i++)
   				for(j=0;j<SIZE_W;j++)
				{
					left_grey[i][j] = right_grey[i][j] = 0;
					disp_temp[i][j] = 2.0*disp_old[i/2][j/2];
				}

			Lowfiltering(SIZE_H, SIZE_W, 1, disp_temp, disp_temp);

			for(i=0;i<low_size.height;i++)
				for(j=0;j<low_size.width;j++)
				{
					left_grey[i][j]=cvGetReal2D(left_g_down,i,j);
					right_grey[i][j]=cvGetReal2D(right_g_down,i,j);
				}
			cvReleaseImage(&left_g_down);
			cvReleaseImage(&right_g_down);
		}

		Deallo2Int(DisparityInt, SIZE_H);


		NULL_flag=1;
		for(j=0;j<SIZE_W;j++)
		{
			if(left_grey[0][j]!=0)
				NULL_flag=0;
		}

		if(NULL_flag==0) //Regularisation for whole image
			DisparityRegularization(left_grey, right_grey, disp_temp, SIZE_H, SIZE_W, beta, lambda, iteration);

		else  //Regularisation for ROI
		{
			int start_r, end_r;
			int temp_SIZE_H;

			for(i=0;i<SIZE_H;i++)
			{
				NULL_flag=1;
				for(j=0;j<SIZE_W;j++)
					if(left_grey[i][j]!=0)
						NULL_flag=0;
				if(NULL_flag==0)
				{
					start_r=i;
					break;
				}
			}

			for(i=SIZE_H-1;i>0;i--)
			{
				NULL_flag=1;
				for(j=0;j<SIZE_W;j++)
					if(left_grey[i][j]!=0)
						NULL_flag=0;
				if(NULL_flag==0)
				{
					end_r=i;
					break;
				}
			}
				
			temp_SIZE_H = end_r-start_r+1;

			printf("Size of interest: %d %d \n",temp_SIZE_H, SIZE_W);

			unsigned char **temp_left = Allo2UC(temp_SIZE_H, SIZE_W);
			unsigned char **temp_right = Allo2UC(temp_SIZE_H, SIZE_W);
			float **temp_Disp = Allo2Float(temp_SIZE_H, SIZE_W);

			for(i=0;i<temp_SIZE_H;i++)
				for(j=0;j<SIZE_W;j++)
				{
					temp_left[i][j]=left_grey[i+start_r][j];
					temp_right[i][j]=right_grey[i+start_r][j];
					temp_Disp[i][j]=disp_temp[i+start_r][j];
				}

			DisparityRegularization(temp_left, temp_right, temp_Disp, temp_SIZE_H, SIZE_W, beta, lambda, iteration);

			for(i=0;i<temp_SIZE_H;i++)
				for(j=0;j<SIZE_W;j++)
					disp_temp[i+start_r][j]= temp_Disp[i][j];

			Deallo2UC(temp_left,temp_SIZE_H);
			Deallo2UC(temp_right,temp_SIZE_H);
			Deallo2Float(temp_Disp,temp_SIZE_H);
		}



		if(t!=hierarchy)
			Deallo2Float(disp_old, SIZE_H/2);
		disp_old	=	Allo2Float(SIZE_H, SIZE_W);

		for(i=0;i<SIZE_H;i++)
			for(j=0;j<SIZE_W;j++)
				disp_old[i][j]=disp_temp[i][j];


		Deallo2UC(left_grey, SIZE_H);
		Deallo2UC(right_grey, SIZE_H);
		Deallo2Float(disp_temp, SIZE_H);

		output_size.width=SIZE_W;
		output_size.height=SIZE_H;

		result	= cvCreateImage( output_size, 8, 1 );

		for(i=0; i<output_size.height; i++)
			for(j=0; j<output_size.width; j++)
			{
				tempi=(int)(scale* (disp_old[i][j] - Min_Range)*pow(2.0,t) + 0.5);

				if (tempi<0)
					cvSetReal2D(result,i,j,0);
				else if(tempi> 255)
					cvSetReal2D(result,i,j,255);
				else
					cvSetReal2D(result,i,j,tempi);
			}

		sprintf( filename, "%s//04Regularization_%d-g_%d_%d.png", BASE_PATH, hierarchy-t,(int)(scale*pow(2.0,t)), (int)Min_Range);
		cvSaveImage(filename, result);
		cvReleaseImage(&result);

		//sprintf( filename, "04Regularization_%d.bmp", hierarchy-t );
		//SaveBmpScaling(disp_old,SIZE_H,SIZE_W, filename);

		SIZE_H*=2;
		SIZE_W*=2;
		Min_Range*=2;
	}

	for(i=0;i<SIZE_H/2;i++)
   		for(j=0;j<SIZE_W/2;j++)
			Disparity[i][j]		= disp_old[i][j];


	Deallo2Float(disp_old, SIZE_H/2);

	printf("\nRegularization done\n");

	//Deallo2Int(gradient_x, SIZE_H);
	//Deallo2Int(gradient_y, SIZE_H);
	//Deallo2Int(smoothRegion, SIZE_H/Block);

}
