
/////////////////////////////////////////////////////////////////////
//
// util.h: Memory allocation and iamge processing
// Programmer: Hansung Kim
// Version 1.0 (2004. 2. 6) 
//
//////////////////////////////////////////////////////////////////////

#include <math.h>
#include <string.h>
#include <stdio.h>
#include <opencv/cv.h>
#include <opencv/highgui.h>

//Properties
int		MatMax(int row, int col, int **input);
int		MatMin(int row, int col, int **input);
double	MatMean(int row, int col, int **input);
double	MatVar(int row, int col, int **input);
void	calc_histo(int **image, int Histo[], int row, int col);
void	Bubble(int N, int *P);

//filtering
void	Lowfiltering(int m_nH, int m_nW, int weight, int **input, int **output);
void	LowfilteringExc(int m_nH, int m_nW, int weight, int exc, int **input, int **output);
void	Lowfiltering(int m_nH, int m_nW, int weight, float **input, float **output);
void	LowfilteringUC(int m_nH, int m_nW, int weight, 
                     unsigned char **input, unsigned char **output);
void	Median33(int row, int col, int** input, int** output);
void	Median33UC(int row, int col, unsigned char** input, unsigned char** output);
void	Sobel(int SIZE_H, int SIZE_W, int **input, int **output);
void	Gradient(int SIZE_H, int SIZE_W, int **input, int **grad_x,
		 int **grad_y, int **grad_mag);
void	GradientUC(int SIZE_H, int SIZE_W, unsigned char **input, int **grad_x,
		 int **grad_y, int **grad_mag);
void	GradientRGB(int SIZE_H, int SIZE_W, UnsignedRGB **input, int **grad_x, int **grad_y, int **grad_mag);
void	Laplacian(int m_nH, int m_nW, int **input, int **output);

//Image processing
void	Thresholding(int m_nH, int m_nW, int Thres, int **m_pBuff, int **output);
void	Equlization(int row, int col, int **input, int **output);
void	Stretching(int SIZE_H, int SIZE_W, int **input, int **output);
void	Invert(int SIZE_H,int SIZE_W, int **input,int **output);
void	Erosion(int ysize, int xsize, int **input, int **output);
void	Dialation(int ysize, int xsize, int **input, int **output);
void	ErosionUC(int ysize, int xsize, unsigned char **input, unsigned char **output);
void	DialationUC(int ysize, int xsize, unsigned char **input, unsigned char **output);

//size conversion
void	SizeConversion(int org_H, int org_W, int tar_H, int tar_W,
		       int **input, int **output);
void	SizeConversionUC(int org_H, int org_W, int tar_H, int tar_W, 
					  unsigned char **input, unsigned char **output);
void	SizeConversionUC(int org_H, int org_W, int tar_H, int tar_W, 
					  unsigned char **input, int **output);
void SizeConversionRGB(int org_H, int org_W, int tar_H, int tar_W, 
					  UnsignedRGB **input, UnsignedRGB **output);
void	Resampling(int SIZE_H, int SIZE_W, double SamplingRate,
		   int **input, int **output);


//////////////////////////////////////////////////////////////////////////////////////////////


void Thresholding(int m_nH, int m_nW, int Thres, int **m_pBuff, int **output)
{
	int i,j;

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
			if(m_pBuff[i][j]>=Thres)
				output[i][j]=255;
			else
				output[i][j]=0;

}

void Lowfiltering(int m_nH, int m_nW, int weight, int **input, int **output)
{
	//3*3 weighted lowpass filtering
	//weight : weighting factor ofcenter pixel

	int i,j,temp2;
	int **temp;

	temp=Allo2Int(m_nH,m_nW);

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
		{
			if(i==0 || j==0 || i==m_nH-1 || j==m_nW-1)
				temp[i][j]=input[i][j];
			else
			{
				temp2=input[i-1][j-1]+input[i-1][j]+input[i-1][j+1]
				  +input[i][j-1]+weight*input[i][j]+input[i][j+1]
				  +input[i+1][j-1]+input[i+1][j]+input[i+1][j+1];
				temp[i][j]=(int)(temp2/(weight+8) +0.5);
			}
		}

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
			output[i][j]=temp[i][j];

	Deallo2Int(temp,m_nH);
}

void LowfilteringExc(int m_nH, int m_nW, int weight, int exc, int **input, int **output)
{
	//3*3 weighted lowpass filtering
	//weight : weighting factor ofcenter pixel

	int i,j,temp2;
	int a,b;
	int count,sum;
	int **temp;

	temp=Allo2Int(m_nH,m_nW);

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
		{
			if(i==0 || j==0 || i==m_nH-1 || j==m_nW-1)
				temp[i][j]=input[i][j];
			else if(input[i][j]==exc)
				temp[i][j]=input[i][j];
			else
			{
				count=0;
				sum=0;
				for(a=-1;a<2;a++)
					for(b=-1;b<2;b++)
					{
						if(input[i+a][j+b]==exc)
							continue;
						else
						{
							sum+=input[i+a][j+b];
							count++;
						}
					}
				sum+=((weight-1)*input[i][j]);
				count+=(weight-1);
				temp[i][j]=(int)(sum/(float)(count) +0.5);
			}
		}

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
			output[i][j]=temp[i][j];

	Deallo2Int(temp,m_nH);
}


void Lowfiltering(int m_nH, int m_nW, int weight, float **input, float **output)
{
	//3*3 weighted lowpass filtering
	//weight : weighting factor of center pixel

	int i,j;
	float temp2;
	float **temp;

	temp=Allo2Float(m_nH,m_nW);

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
		{
			if(i==0 || j==0 || i==m_nH-1 || j==m_nW-1)
				temp[i][j]=input[i][j];
			else
			{
				temp2=input[i-1][j-1]+input[i-1][j]+input[i-1][j+1]
				  +input[i][j-1]+weight*input[i][j]+input[i][j+1]
				  +input[i+1][j-1]+input[i+1][j]+input[i+1][j+1];
				temp[i][j]=temp2/(weight+8);
			}
		}

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
			output[i][j]=temp[i][j];

	Deallo2Float(temp,m_nH);
}

void LowfilteringUC(int m_nH, int m_nW, int weight, unsigned char **input, unsigned char **output)
{
	//weight : weighting factor ofcenter pixel

	int i,j,temp2;
	unsigned char **temp;

	temp=Allo2UC(m_nH,m_nW);

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
		{
			if(i==0 || j==0 || i==m_nH-1 || j==m_nW-1)
				temp[i][j]=input[i][j];
			else
			{
				temp2=input[i-1][j-1]+input[i-1][j]+input[i-1][j+1]
				  +input[i][j-1]+weight*input[i][j]+input[i][j+1]
				  +input[i+1][j-1]+input[i+1][j]+input[i+1][j+1];
				temp[i][j]=(unsigned char)(temp2/(weight+8) +0.5);
			}
		}

	for(i=0;i<m_nH;i++)
		for(j=0;j<m_nW;j++)
			output[i][j]=temp[i][j];

	Deallo2UC(temp,m_nH);
}

void SizeConversion(int org_H, int org_W, int tar_H, int tar_W, int **input, int **output)
{
	// ũ ϴ  ũ⸦ ޾  ũ ȯ

  int i, j;
  int temp;
  double ratio_H, ratio_W, position_H, position_W;
  double weight_H, weight_W;

  //Ȯ/ 
  ratio_H=(double)org_H/(double)tar_H;
  ratio_W=(double)org_W/(double)tar_W;

  //interpolation
  for(i=0;i<tar_H;i++){
    position_H=i*ratio_H;
    weight_H=position_H-(int)position_H;

    for(j=0;j<tar_W;j++){
      position_W=j*ratio_W;
      weight_W=position_W-(int)position_W;
      
      if(position_H>=tar_H-1 || position_W>=tar_W-1)
	output[i][j]=input[(int)position_H][(int)position_W];

      else{
	temp=(int)((1-weight_H)*(1-weight_W)*input[(int)position_H][(int)position_W]
		   +weight_H*(1-weight_W)*input[(int)position_H+1][(int)position_W]
		   +(1-weight_H)*weight_W*input[(int)position_H][(int)position_W+1]
		   +weight_H*weight_W*input[(int)position_H+1][(int)position_W+1]+0.5);
	output[i][j]=temp;
      }
    }
  }
}

void SizeConversionUC(int org_H, int org_W, int tar_H, int tar_W, 
					  unsigned char **input, unsigned char **output)
{
	// ũ ϴ  ũ⸦ ޾  ũ ȯ

  int i, j;
  int temp;
  double ratio_H, ratio_W, position_H, position_W;
  double weight_H, weight_W;

  //Ȯ/ 
  ratio_H=(double)org_H/(double)tar_H;
  ratio_W=(double)org_W/(double)tar_W;

  //interpolation
  for(i=0;i<tar_H;i++){
    position_H=i*ratio_H;
    weight_H=position_H-(int)position_H;

    for(j=0;j<tar_W;j++){
      position_W=j*ratio_W;
      weight_W=position_W-(int)position_W;
      
      if(position_H>=tar_H-1 || position_W>=tar_W-1)
	output[i][j]=(int)input[(int)position_H][(int)position_W];

      else{
	temp=(int)((1-weight_H)*(1-weight_W)*input[(int)position_H][(int)position_W]
		   +weight_H*(1-weight_W)*input[(int)position_H+1][(int)position_W]
		   +(1-weight_H)*weight_W*input[(int)position_H][(int)position_W+1]
		   +weight_H*weight_W*input[(int)position_H+1][(int)position_W+1]+0.5);
	output[i][j]=(unsigned char)temp;
      }
    }
  }
}


void SizeConversionUC(int org_H, int org_W, int tar_H, int tar_W, 
					  unsigned char **input, int **output)
{
	// ũ ϴ  ũ⸦ ޾  ũ ȯ

  int i, j;
  int temp;
  double ratio_H, ratio_W, position_H, position_W;
  double weight_H, weight_W;

  //Ȯ/ 
  ratio_H=(double)org_H/(double)tar_H;
  ratio_W=(double)org_W/(double)tar_W;

  //interpolation
  for(i=0;i<tar_H;i++){
    position_H=i*ratio_H;
    weight_H=position_H-(int)position_H;

    for(j=0;j<tar_W;j++){
      position_W=j*ratio_W;
      weight_W=position_W-(int)position_W;
      
      if(position_H>=tar_H-1 || position_W>=tar_W-1)
	output[i][j]=(int)input[(int)position_H][(int)position_W];

      else{
	temp=(int)((1-weight_H)*(1-weight_W)*input[(int)position_H][(int)position_W]
		   +weight_H*(1-weight_W)*input[(int)position_H+1][(int)position_W]
		   +(1-weight_H)*weight_W*input[(int)position_H][(int)position_W+1]
		   +weight_H*weight_W*input[(int)position_H+1][(int)position_W+1]+0.5);
	output[i][j]=temp;
      }
    }
  }
}

void SizeConversionRGB(int org_H, int org_W, int tar_H, int tar_W, 
					  UnsignedRGB **input, UnsignedRGB **output)
{
	// ũ ϴ  ũ⸦ ޾  ũ ȯ

  int i, j;
  int temp;
  double ratio_H, ratio_W, position_H, position_W;
  double weight_H, weight_W;

  //Ȯ/ 
  ratio_H=(double)org_H/(double)tar_H;
  ratio_W=(double)org_W/(double)tar_W;

  //interpolation
  for(i=0;i<tar_H;i++){
    position_H=i*ratio_H;
    weight_H=position_H-(int)position_H;

    for(j=0;j<tar_W;j++){
      position_W=j*ratio_W;
      weight_W=position_W-(int)position_W;
      
      if(position_H>=tar_H-1 || position_W>=tar_W-1)
	  {
		  output[i][j].r=input[(int)position_H][(int)position_W].r;
		  output[i][j].g=input[(int)position_H][(int)position_W].g;
		  output[i][j].b=input[(int)position_H][(int)position_W].b;
	  }

      else
	  {
		  temp=(int)((1-weight_H)*(1-weight_W)*input[(int)position_H][(int)position_W].r
			  +weight_H*(1-weight_W)*input[(int)position_H+1][(int)position_W].r
			  +(1-weight_H)*weight_W*input[(int)position_H][(int)position_W+1].r
			  +weight_H*weight_W*input[(int)position_H+1][(int)position_W+1].r+0.5);
		  output[i][j].r=(unsigned char)temp;
		  temp=(int)((1-weight_H)*(1-weight_W)*input[(int)position_H][(int)position_W].g
			  +weight_H*(1-weight_W)*input[(int)position_H+1][(int)position_W].g
			  +(1-weight_H)*weight_W*input[(int)position_H][(int)position_W+1].g
			  +weight_H*weight_W*input[(int)position_H+1][(int)position_W+1].g+0.5);
		  output[i][j].g=(unsigned char)temp;
		  temp=(int)((1-weight_H)*(1-weight_W)*input[(int)position_H][(int)position_W].b
			  +weight_H*(1-weight_W)*input[(int)position_H+1][(int)position_W].b
			  +(1-weight_H)*weight_W*input[(int)position_H][(int)position_W+1].b
			  +weight_H*weight_W*input[(int)position_H+1][(int)position_W+1].b+0.5);
		  output[i][j].b=(unsigned char)temp;
      }
    }
  }
}

void Median33(int row, int col, int** input, int** output)
{
	int **tempmat;
	int a,b,x,y,t,temp,sort[9];

	tempmat=Allo2Int(row,col);
	
	for(a=0;a<row;a++)
	{
		output[a][0]=input[a][0];
		output[a][col-1]=input[a][col-1];
	}

	for(a=1;a<col-1;a++)
	{
		output[0][a]=input[0][a];
		output[row-1][a]=input[row-1][a];
	}

	for(a=1;a<row-1;a++)
   		for(b=1;b<col-1;b++)
   		{
      		t=0;
			for(x=-1;x<2;x++)
      			for(y=-1;y<2;y++)
        		   sort[t++]=input[a+x][b+y];

     		for(x=0;x<8;x++)
				for(y=0;y<8-x;y++)
         		{
            		if(sort[y]<sort[y+1])
            		{
				   		temp=sort[y];
               			sort[y]=sort[y+1];
						sort[y+1]=temp;
					}
				}
			
      		tempmat[a][b]=sort[4];
		}

	for(a=0;a<row;a++)
   		for(b=0;b<col;b++)
		{
			if(a==0 || a== row-1 || b==0 || b==col-1)
				output[a][b]=input[a][b];
			else
				output[a][b]=tempmat[a][b];
		}

	Deallo2Int(tempmat,row);
}

void Median33UC(int row, int col, unsigned char** input, unsigned char** output)
{
	unsigned char **tempmat;
	int a,b,x,y,t;
	unsigned char temp, sort[9];

	tempmat=Allo2UC(row,col);
	
	for(a=0;a<row;a++)
	{
		output[a][0]=input[a][0];
		output[a][col-1]=input[a][col-1];
	}

	for(a=1;a<col-1;a++)
	{
		output[0][a]=input[0][a];
		output[row-1][a]=input[row-1][a];
	}

	for(a=1;a<row-1;a++)
   		for(b=1;b<col-1;b++)
   		{
      		t=0;
			for(x=-1;x<2;x++)
      			for(y=-1;y<2;y++)
        		   sort[t++]=input[a+x][b+y];

     		for(x=0;x<8;x++)
				for(y=0;y<8-x;y++)
         		{
            		if(sort[y]<sort[y+1])
            		{
				   		temp=sort[y];
               			sort[y]=sort[y+1];
						sort[y+1]=temp;
					}
				}
			
      		tempmat[a][b]=sort[4];
		}

	for(a=0;a<row;a++)
   		for(b=0;b<col;b++)
		{
			output[a][b]=tempmat[a][b];
		}

	Deallo2UC(tempmat,row);
}


void Bubble(int N, int *P)	  	//ũ Ȯ ġ
{

}

int MatMax(int row, int col, int **input)
{
	int i,j;
   int max=-100000000;

	for(i=0;i<row;i++)
   	for(j=0;j<col;j++)
      {
         if(input[i][j]>max)
         	max=input[i][j];
      }

	return max;
}

int MatMin(int row, int col, int **input)
{
	int i,j;
   int min=100000000;

	for(i=0;i<row;i++)
   	for(j=0;j<col;j++)
      {
         if(input[i][j]<min)
         	min=input[i][j];
      }

	return min;
}


void calc_histo(int **image, int Histo[], int row, int col)
{
	int x,y,temp;

//Histo[] ʱȭ
	for(x = 0; x <256; x++)
	{
		Histo[x] = 0;
	}

// 󵵼 
 	for(y = 0; y < row; y++)
 		for(x=0; x < col; x++)
		{
 			temp = image[y][x];
 			Histo[temp]++;
 		}

}


void Equlization(int row, int col, int **input, int **output)
{
  int temp, Histo_CDF[256];
  int Histo[256];
  int total_pixel = row * col; 
  int y,x;
	
//Initialization	
  for(y = 0; y <= 255; y++)
    Histo[y] = 0;
    
  for(y = 0; y < row; y++)
    for(x=0; x < col; x++)
      output[y][x] = 0;
      

// 󵵼 	
  calc_histo(input, Histo, row, col);
  temp = 0;

// ׷ ϱ
  for(y = 0; y <= 255; y++){
    temp += Histo[y];
    Histo_CDF[y] = temp;
  }

// ׷ ȭ
  temp = 0;
  for(y = 0; y < row; y++)
    for(x=0; x < col; x++){
      temp = input[y][x];
      output[y][x] = (int)((float)Histo_CDF[temp] * ((float)255/(float)total_pixel));
    }
}

void Gradient(int SIZE_H, int SIZE_W, int **input, int **grad_x, int **grad_y, int **grad_mag)
{

  int i,j;
  
  for(i=0;i<SIZE_H;i++)
    for(j=0;j<SIZE_W;j++){
      if(i==SIZE_H-1)
	grad_y[i][j]=0;
      else
	grad_y[i][j]=input[i+1][j]-input[i][j];
      if(j==SIZE_W-1)
	grad_x[i][j]=0;
      else
	grad_x[i][j]=input[i][j+1]-input[i][j];
      
      grad_mag[i][j]=(int)(sqrt((double)grad_x[i][j]*grad_x[i][j]+grad_y[i][j]*grad_y[i][j])+0.5);
    }
}

void GradientRGB(int SIZE_H, int SIZE_W, UnsignedRGB **input, int **grad_x, int **grad_y, int **grad_mag)
{

  int i,j;
  
  for(i=0;i<SIZE_H;i++)
    for(j=0;j<SIZE_W;j++){
      if(i==SIZE_H-1)
		grad_y[i][j]=0;
      else
		  grad_y[i][j]=abs(input[i+1][j].r-input[i][j].r)+abs(input[i+1][j].g-input[i][j].g)+abs(input[i+1][j].b-input[i][j].b);
      if(j==SIZE_W-1)
		grad_x[i][j]=0;
      else
		grad_x[i][j]=abs(input[i][j+1].r-input[i][j].r)+abs(input[i][j+1].g-input[i][j].g)+abs(input[i][j+1].b-input[i][j].b);
      
      grad_mag[i][j]=(int)(sqrt((double)grad_x[i][j]*grad_x[i][j]+grad_y[i][j]*grad_y[i][j])+0.5);
    }
}

void GradientUC(int SIZE_H, int SIZE_W, unsigned char **input, int **grad_x, int **grad_y, int **grad_mag)
{

  int i,j;
  
  for(i=0;i<SIZE_H;i++)
    for(j=0;j<SIZE_W;j++){
      if(i==SIZE_H-1)
	grad_y[i][j]=0;
      else
	grad_y[i][j]=(int)input[i+1][j]-(int)input[i][j];
      if(j==SIZE_W-1)
	grad_x[i][j]=0;
      else
	grad_x[i][j]=(int)input[i][j+1]-(int)input[i][j];
      
      grad_mag[i][j]=(int)(sqrt((double)grad_x[i][j]*grad_x[i][j]+grad_y[i][j]*grad_y[i][j])+0.5);
    }

}


double MatMean(int row, int col, int **input)
{
	int i,j;
	double mean;
   int sum=0;

	for(i=0;i<row;i++)
   	for(j=0;j<col;j++)
      {
		sum+=input[i][j];
      }
	mean=(double)sum/(row*col);

	return mean;
}


double MatVar(int row, int col, int **input)
{
	int i,j;
	double mean,var;
   int sum=0;
   double sum2=0.0;

	for(i=0;i<row;i++)
   	for(j=0;j<col;j++)
      {
		sum+=input[i][j];
      }
	mean=(double)sum/(row*col);

	for(i=0;i<row;i++)
   	for(j=0;j<col;j++)
      {
		sum2+=(input[i][j]-mean)*(input[i][j]-mean);
      }
	var=sum2/(row*col);

	return var;
}

void Stretching(int SIZE_H, int SIZE_W, int **input, int **output)
{
	int a,b, min=1000000, max=-1000000;

	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
		{
      	if(input[a][b]>max)
         	max=input[a][b];
      	if(input[a][b]<min)
         	min=input[a][b];
         }
      
	for(a=0;a<SIZE_H;a++)
		for(b=0;b<SIZE_W;b++)
			output[a][b]=(int)((input[a][b]-min)/(double)(max-min)*255+0.5);

}

void Sobel(int SIZE_H, int SIZE_W, int **input, int **output)
{
  int i,j;
  int temp;

  for(i=0;i<SIZE_H;i++)
    for(j=0;j<SIZE_W;j++){
      if(i==0||j==0||i==SIZE_H-1||j==SIZE_W-1)
	output[i][j]=0;
      else{
	temp=abs(input[i+1][j+1]+2*input[i+1][j]+input[i+1][j-1]
		 -input[i-1][j+1]-2*input[i-1][j]-input[i-1][j-1])
	  +abs(input[i+1][j+1]+2*input[i][j+1]+input[i-1][j+1]-
	       input[i+1][j-1]-2*input[i][j-1]-input[i-1][j-1]);
	if(temp>255)
	  output[i][j]=255;
	else
	  output[i][j]=temp;
      }
    }
}

void Resampling(int SIZE_H, int SIZE_W, double SamplingRate, int **input, int **output)
{
  int i,j,a,b;
  int rate,sum;

  if(SamplingRate<=1.0){
    rate=(int)(1/SamplingRate);
    for(i=0;i<SIZE_H;i++)
      for(j=0;j<SIZE_W;j++){
	for(a=0;a<rate;a++)
	  for(b=0;b<rate;b++){
	    output[i*rate+a][j*rate+b]=input[i][j];
	  }
      }
  }
  else{
    rate=(int)SamplingRate;
    
    for(i=0;i<SIZE_H;i+=rate)
      for(j=0;j<SIZE_W;j+=rate){
	sum=0;
	for(a=0;a<rate;a++)
	  for(b=0;b<rate;b++)
	    sum+=input[i+a][j+b];
	output[i/rate][j/rate]=(int)((double)sum/(rate*rate)+0.5);
      }
  }
}

void Invert(int SIZE_H,int SIZE_W, int **input,int **output)
{
	int i,j;

	for(i=0;i<SIZE_H;i++)
		for(j=0;j<SIZE_W;j++)
			output[i][j]=255-input[i][j];
}

void Laplacian(int m_nH, int m_nW, int **input, int **output)
{
  int i,j;

  for(i=0;i<m_nH;i++)
    for(j=0;j<m_nW;j++){
      if(i==0 || j==0 || i==m_nH-1 || j==m_nW-1)
	output[i][j]=0;
      else
	output[i][j]=(-1)*input[i-1][j]-input[i][j-1]
	  +4*input[i][j]-input[i][j+1]-input[i+1][j];
    }

  for(i=0;i<m_nH;i++)
    for(j=0;j<m_nW;j++){
      if(output[i][j]>255)
	output[i][j]=255;
      if(output[i][j]<0)
	output[i][j]=0;
    }
}

void Erosion(int ysize, int xsize, int **input, int **output)
{
  int i,j;
  
  for(i=0; i<ysize; i++){
    for(j=0; j<xsize; j++){

      if(i==0 || j==0 || i==ysize-1 || j==xsize-1){
	output[i][j]=0;
	continue;
      }

      if(input[i][j]==0)
	output[i-1][j-1]=output[i-1][j]=output[i-1][j+1]
	    =output[i][j-1]=output[i][j]=output[i][j+1]
	    =output[i+1][j-1]=output[i+1][j]=output[i+1][j+1]=0;
    }
  }
}

void Dialation(int ysize, int xsize, int **input, int **output)
{
  int i,j;
  
  for(i=0; i<ysize; i++){
    for(j=0; j<xsize; j++){

      if(i==0 || j==0 || i==ysize-1 || j==xsize-1){
	output[i][j]=0;
	continue;
      }

      if(input[i][j]==1)
	output[i-1][j-1]=output[i-1][j]=output[i-1][j+1]
	    =output[i][j-1]=output[i][j]=output[i][j+1]
	    =output[i+1][j-1]=output[i+1][j]=output[i+1][j+1]=1;
    }
  }
}

void ErosionUC(int ysize, int xsize, unsigned char **input, unsigned char **output)
{
  int i,j;
  
  for(i=0; i<ysize; i++){
    for(j=0; j<xsize; j++){

      if(i==0 || j==0 || i==ysize-1 || j==xsize-1){
	output[i][j]=0;
	continue;
      }

      if(input[i][j]==0)
	output[i-1][j-1]=output[i-1][j]=output[i-1][j+1]
	    =output[i][j-1]=output[i][j]=output[i][j+1]
	    =output[i+1][j-1]=output[i+1][j]=output[i+1][j+1]=0;
    }
  }
}

void DialationUC(int ysize, int xsize, unsigned char **input, unsigned char **output)
{
  int i,j;
  
  for(i=0; i<ysize; i++){
    for(j=0; j<xsize; j++){

      if(i==0 || j==0 || i==ysize-1 || j==xsize-1){
	output[i][j]=0;
	continue;
      }

      if(input[i][j]==1)
	output[i-1][j-1]=output[i-1][j]=output[i-1][j+1]
	    =output[i][j-1]=output[i][j]=output[i][j+1]
	    =output[i+1][j-1]=output[i+1][j]=output[i+1][j+1]=1;
    }
  }
}


