/////////////////////////////////////////////////////////
//
//  Cubic Projection - Generate Cubic Projection images with depth map
//  V1.2 
//  V1.3 - with more input options for grey disparity and different size output.
//         Have problem in depth conversion
//  ManhattanAlignment
//  V1.0 - Only for Mahattan Alignment
////////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <math.h>
#include <time.h>
#include <direct.h>
#include "variable.h"
#include "param.h"
#include "houghlines.h"
#include "spherical.h"



void on_mouse_canny(int event, int x, int y, int flags, void* param);


int main(int argc, char **argv)
{

	//For commnad line input /////////////////////////////////////////////
	if (argc < 4)
	{
		printf("Syntax is: %s [Top_Image] [Bottom_Image] [Disparity image]\n", argv[0]);
		return (0);
	}

	sprintf(TOP_IMAGE, "%s", argv[1]);
	sprintf(BOT_IMAGE, "%s", argv[2]);
	sprintf(DISPARITY_IMAGE, "%s", argv[3]);
	
	sprintf(filename, "%s", TOP_IMAGE);
	if ((org_image_t = cvLoadImage(filename, 1)) == 0)
	{
		printf("Cannot open the original image - %s\n", filename);
		return 0;
	}
	sprintf(filename, "%s", BOT_IMAGE);
	if ((org_image_b = cvLoadImage(filename, 1)) == 0)
	{
		printf("Cannot open the original image - %s\n", filename);
		return 0;
	}
	sprintf(filename, "%s", DISPARITY_IMAGE);
	if ((org_disparity = cvLoadImage(filename, colour_disparity_flag)) == 0)
	{
		printf("Cannot open the disparity image - %s\n", filename);
		return 0;
	}

	if (org_disparity->nChannels == 1)
		disp_scale = 2.0;
	else
		disp_scale = 1000.0;


	img_size = cvGetSize(org_image_t);
	cube_size = cvSize(img_size.height * 3, img_size.height / 2);
	face_size = cvSize(cube_size.height, cube_size.height);

	img_t = cvCreateImage(img_size, 8, 3);
	img_b = cvCreateImage(img_size, 8, 3);
	img_disparity = cvCreateImage(img_size, org_disparity->depth, org_disparity->nChannels);
	printf("disparity info- depth: %d channel: %d\n", org_disparity->depth, org_disparity->nChannels);

	img_cube = cvCreateImage(cube_size, 8, 3);		//6 face images of the cube projection (top,front,right,back,left,bottom)
	seg_cube = cvCreateImage(cube_size, 8, 3);
	img_disp_cube = cvCreateImage(cube_size, org_disparity->depth, org_disparity->nChannels);

	for (int i = 0; i<6; i++)
	{
		cube_face[i] = cvCreateImage(face_size, 8, 3);
		Hough_face[i] = cvCreateImage(face_size, 8, 3);
	}

	pixel_size = cvSize2D32f(2 * (float)PI / img_size.width, (float)PI / img_size.height);	//pixel size in radian in the org image

	disparity = cvCreateMat(img_size.height, img_size.width, CV_32FC1);

	CvPoint4D32f Hough_result;
	CvPoint Hough_line[300][3];

		//canny edge detector
		org_canny = cvCreateImage(img_size, 8, 1);
		cvCvtColor(org_image_t, org_canny, CV_RGB2GRAY);
		cvCanny(org_canny, org_canny, canny1, canny2);
		cvNamedWindow("Canny", 1);
		cvShowImage("Canny", org_canny);

		//Mouse click for the nearest point to facade on the canny image (should be left side of the lowest point)
		printf("Click a point near to facade on the canny image\n");
		cvSetMouseCallback("Canny", on_mouse_canny, 0);

		cvWaitKey(0);
		cvDestroyWindow("Canny");

		//Spherical Image initial alignment
		int shift = 0;
		double prev1 = 0.0, prev2 = 0.0;

		int init_shift = img_size.width / 4 - curr_pt.x;

		double	max = 0.0, avg;
		int		max_shift;
		double	Hough_poss[100];

		for (int i = (-50); i<50; i++)
		{
			spherical_image_shift(org_image_t, img_t, init_shift + i);
			spherical2cube(img_t, img_cube, cube_face, pixel_size);

			Hough_result = HoughTr(cube_face[2], Hough_face[2], Hough_line, 1);

			if (Hough_result.x > 7 && Hough_result.y != 0)
				Hough_poss[i + 50] = Hough_result.w * Hough_result.x * Hough_result.z / Hough_result.y;
			else
				Hough_poss[i + 50] = Hough_result.w * Hough_result.x * Hough_result.z / (Hough_result.y + 0.5);

		}



		for (int i = 2; i<98; i++)
		{
			avg = (Hough_poss[i - 2] + Hough_poss[i - 1] + 2 * Hough_poss[i] + Hough_poss[i + 1] + Hough_poss[i + 2]) / 6.0;
			if (avg>max)
			{
				max = avg;
				max_shift = i - 50;
			}
		}

		//fclose(output);

		spherical_image_shift(org_image_t, img_t, init_shift + max_shift);
		spherical2cube(img_t, img_cube, cube_face, pixel_size);
		spherical_image_shift(org_image_b, img_b, init_shift + max_shift);
		spherical_image_shift(org_disparity, img_disparity, init_shift + max_shift);

		printf("initial shift: %d, final shift: %d\n", init_shift, init_shift + max_shift);

		sprintf(filename, "shifted_t.png");
		cvSaveImage(filename, img_t);
		sprintf(filename, "shifted_b.png");
		cvSaveImage(filename, img_b);
		sprintf(filename, "shifted-disparity.png");
		cvSaveImage(filename, img_disparity);

	return 1;
}


void on_mouse_canny(int event, int x, int y, int flags, void* param)
{
	if (!org_canny)
		return;

	if (event == CV_EVENT_LBUTTONDOWN)
	{
		printf("point: %d %d selected\n", x, y);

		CvPoint pt = cvPoint(x, y);
		curr_pt = pt;
		if (cvGetReal2D(org_canny, y, x)>128)
			curr_pt = cvPoint(pt.x, pt.y);
		else
		{
			for (int t = 1; t<100; t++)
			{
				if (cvGetReal2D(org_canny, y + t, x)>128)
				{
					curr_pt = cvPoint(pt.x, pt.y + t);
					break;
				}
				if (cvGetReal2D(org_canny, y, x + t)>128)
				{
					curr_pt = cvPoint(pt.x + t, pt.y);
					break;
				}
				if (cvGetReal2D(org_canny, y, x - t)>128)
				{
					curr_pt = cvPoint(pt.x - t, pt.y);
					break;
				}
				if (cvGetReal2D(org_canny, y - t, x)>128)
				{
					curr_pt = cvPoint(pt.x, pt.y - t);
					break;
				}
			}
		}

		printf("Nearest edge point: %d %d \n", curr_pt.x, curr_pt.y);
		printf("OK! now press SPACE bar to get aligned cube\n");
	}
}
