#define PI 3.14159

CvPoint3D32f Cartesian2Spherical(CvPoint3D32f point_3D);

void spherical2cube(IplImage *spherical, IplImage *cube, IplImage *cube_face[6], CvSize2D32f pixel_size);

CvPoint spherical2cube_point(CvPoint sph_pt);		//point projection from spherical point into cube point

void cube2spherical(IplImage *cube, IplImage *spherical, CvSize2D32f pixel_size);

void cube2spherical_two_points (int index[], CvPoint cube[][2], CvPoint spherical[][2], int number);

void persp_depth_cube_generation(CvMat *spherical, CvMat *cube_face[6], IplImage *normal_face[6], IplImage *kinect_normal_face[6], CvSize2D32f pixel_size);

void cube_composition(IplImage *cube_face[6], IplImage *cube);

void cube_decomposition(IplImage *cube, IplImage *cube_face[6]);

void save_depth_images(CvMat *persp_depth_face[6], IplImage *img_disp_face[6], IplImage *kinect_depth_face[6] , int Rot);

void save_normal_images(IplImage *normal_face[6], IplImage *kinect_normal_face[6], CvSize face_size , int Rot);

void spherical_image_shift(IplImage *org_image, IplImage *image, int shift);

void face_normal_calculation(CvPoint3D32f **vertice, CvPoint3D32f **vertice_normal, IplImage *normal_face);

void cube_normal_calculation(CvMat *depth_face[6], IplImage *normal_face[6]);

CvPoint3D32f vertex_calculation(double disp, int i, int j );

CvPoint3D32f calculate_normal(CvPoint3D32f vetice_point[]);

void normal_classification(IplImage *input, IplImage *output);

IplImage* img_resize(IplImage* src_img, int new_width,int new_height);


////////Implementation

CvPoint3D32f Cartesian2Spherical(CvPoint3D32f point_3D)
{
	CvPoint3D32f spherical_3D;

	spherical_3D.x= sqrt(point_3D.x * point_3D.x + point_3D.y * point_3D.y +point_3D.z * point_3D.z);		//r
	spherical_3D.y = acos(point_3D.z / spherical_3D.x);														//theta
	spherical_3D.z = CV_PI - atan(point_3D.y/point_3D.x);													//phi
	
	return spherical_3D;
}


void cube_composition(IplImage *cube_face[6], IplImage *cube)
{
	CvSize face_size = cvGetSize(cube_face[0]);

	for(int i=0; i<6; i++)
	{
		cvSetImageROI( cube, cvRect( face_size.width*i, 0, face_size.width, face_size.height ) );
		cvCopy( cube_face[i], cube );
		cvResetImageROI( cube );
	}

}

void cube_decomposition(IplImage *cube, IplImage *cube_face[6])
{
	CvSize face_size = cvGetSize(cube_face[0]);

	for(int i=0; i<6; i++)
	{
		cvSetImageROI( cube, cvRect( face_size.width*i, 0, face_size.width, face_size.height ) );
		cvCopy( cube, cube_face[i]);
		cvResetImageROI( cube );
	}
}

void spherical_image_shift(IplImage *org_image, IplImage *image, int shift)
{
	IplImage *temp1, *temp2;

	if(shift>=0)
	{
		temp1 = cvCreateImage(cvSize(img_size.width -shift, img_size.height),org_image->depth, org_image->nChannels);
		temp2 = cvCreateImage(cvSize(shift, img_size.height),org_image->depth, org_image->nChannels);
		
		cvSetImageROI( org_image, cvRect( 0, 0, img_size.width -shift, img_size.height ) );
		cvCopy( org_image, temp1 );
		cvResetImageROI( org_image );

		cvSetImageROI( org_image, cvRect( img_size.width -shift, 0, shift, img_size.height ) );
		cvCopy( org_image, temp2 );
		cvResetImageROI( org_image );

		cvSetImageROI( image, cvRect( shift, 0, img_size.width -shift, img_size.height ) );
		cvCopy( temp1, image );
		cvResetImageROI( image );

		cvSetImageROI( image, cvRect( 0, 0, shift, img_size.height ) );
		cvCopy( temp2, image );
		cvResetImageROI( image );

	}

	else
	{
		shift = (-1)*shift;
		temp1 = cvCreateImage(cvSize(shift, img_size.height),org_image->depth, org_image->nChannels);
		temp2 = cvCreateImage(cvSize(img_size.width -shift, img_size.height),org_image->depth, org_image->nChannels);

		cvSetImageROI( org_image, cvRect( 0, 0, shift, img_size.height ) );
		cvCopy( org_image, temp1 );
		cvResetImageROI( org_image );

		cvSetImageROI( org_image, cvRect( shift, 0, img_size.width -shift, img_size.height ) );
		cvCopy( org_image, temp2 );
		cvResetImageROI( org_image );

		cvSetImageROI( image, cvRect( img_size.width -shift, 0, shift, img_size.height ) );
		cvCopy( temp1, image );
		cvResetImageROI( image );

		cvSetImageROI( image, cvRect( 0, 0, img_size.width -shift, img_size.height ) );
		cvCopy( temp2, image );
		cvResetImageROI( image );		


	}

	cvReleaseImage(&temp1);
	cvReleaseImage(&temp2);
	
}


void marker_expansion(IplImage *marker, IplImage *image)
{
	uchar *curr, *prev, *curr_i, *prev_i;

	CvSize cube_size = cvGetSize(marker);
	CvSize face_size = cvSize(cube_size.height, cube_size.height);

	IplImage* marker_face = cvCreateImage( face_size, 8, 3 );
	IplImage* image_face = cvCreateImage( face_size, 8, 3 );

	for(int t=0;t<6;t++)
	{
		cvSetImageROI( marker, cvRect( face_size.width*t, 0, face_size.width, face_size.height ) );
		cvCopy( marker, marker_face );
		cvSetImageROI( image, cvRect( face_size.width*t, 0, face_size.width, face_size.height ) );
		cvCopy( image, image_face );

		//Top
		for(int i=Expansion_range-1;i>0;i--)
			for(int j=0; j<face_size.width;j++)
			{
				curr = &CV_IMAGE_ELEM( marker_face, uchar, i, j*3 );
				prev = &CV_IMAGE_ELEM( marker_face, uchar, i+1, j*3 );

				if(curr[0]==0 && prev[0]!=0)
				{
					curr[0]=prev[0]; curr[1]=prev[1]; curr[2]=prev[2];
					
					curr_i = &CV_IMAGE_ELEM( image_face, uchar, i, j*3 );
					prev_i = &CV_IMAGE_ELEM( image_face, uchar, i+1, j*3 );

					curr_i[0]=prev_i[0]; curr_i[1]=prev_i[1]; curr_i[2]=prev_i[2];
				}
			}
		
		//Left
		for(int i=0; i<face_size.height;i++)
			for(int j=Expansion_range-1;j>0;j--)
			{
				curr = &CV_IMAGE_ELEM( marker_face, uchar, i, j*3 );
				prev = &CV_IMAGE_ELEM( marker_face, uchar, i, (j+1)*3 );

				if(curr[0]==0 && prev[0]!=0)
				{
					curr[0]=prev[0]; curr[1]=prev[1]; curr[2]=prev[2];
					
					curr_i = &CV_IMAGE_ELEM( image_face, uchar, i, j*3 );
					prev_i = &CV_IMAGE_ELEM( image_face, uchar, i, (j+1)*3 );

					curr_i[0]=prev_i[0]; curr_i[1]=prev_i[1]; curr_i[2]=prev_i[2];
				}
			}

		//Bottom
		for(int i=face_size.height-Expansion_range+1;i<face_size.height;i++)
			for(int j=0; j<face_size.width;j++)
			{
				curr = &CV_IMAGE_ELEM( marker_face, uchar, i, j*3 );
				prev = &CV_IMAGE_ELEM( marker_face, uchar, i-1, j*3 );

				if(curr[0]==0 && prev[0]!=0)
				{
					curr[0]=prev[0]; curr[1]=prev[1]; curr[2]=prev[2];
					
					curr_i = &CV_IMAGE_ELEM( image_face, uchar, i, j*3 );
					prev_i = &CV_IMAGE_ELEM( image_face, uchar, i-1, j*3 );

					curr_i[0]=prev_i[0]; curr_i[1]=prev_i[1]; curr_i[2]=prev_i[2];
				}
			}

		//Right
		for(int i=0; i<face_size.height;i++)
			for(int j=face_size.width-Expansion_range+1;j<face_size.width;j++)
			{
				curr = &CV_IMAGE_ELEM( marker_face, uchar, i, j*3 );
				prev = &CV_IMAGE_ELEM( marker_face, uchar, i, (j-1)*3 );

				if(curr[0]==0 && prev[0]!=0)
				{
					curr[0]=prev[0]; curr[1]=prev[1]; curr[2]=prev[2];
					
					curr_i = &CV_IMAGE_ELEM( image_face, uchar, i, j*3 );
					prev_i = &CV_IMAGE_ELEM( image_face, uchar, i, (j-1)*3 );

					curr_i[0]=prev_i[0]; curr_i[1]=prev_i[1]; curr_i[2]=prev_i[2];
				}
			}

		cvSetImageROI( marker, cvRect( face_size.width*t, 0, face_size.width, face_size.height ) );
		cvCopy( marker_face, marker);

		cvSetImageROI( image, cvRect( face_size.width*t, 0, face_size.width, face_size.height ) );
		cvCopy( image_face, image);
	}

}


void cube2spherical_two_points (int index[], CvPoint cube[][2], CvPoint spherical[][2], int number)
{
	int k,t;
	int i,j;				//pixel position on face image
	int a,b;				//pixel position on spherical image
	float x, y, z;			//Cartesian coord
	float r, theta, phi;	//Spherical coord
	int mode;				//0:Top 1:Left 2:Front 3:Back 4:Right 5:Bottom

	CvSize sph_size = cvGetSize(markers);
	CvSize cube_size = cvGetSize(markers_cube);
	CvSize face_size = cvSize(cube_size.height, cube_size.height);

	for(k=1;k<number;k++)
	{
		if(index[k]==2)		//Line
		{
			for(t=0;t<2;t++)	//Both for source and target points
			{
				i=cube[k][t].y;
				j=cube[k][t].x;

				if(j<face_size.width)		//Top
				{
					mode=0;
					j-=(mode*face_size.width);
					x=((float)i/face_size.height * (-2.0)) + 1.0;
					y=((float)j/face_size.width * (2.0)) - 1.0;
					z=1.0;
				}

				else if(j< 2*face_size.width)		//Front
				{
					mode=1;
					j-=(mode*face_size.width);
					x=1.0;
					y=((float)j/face_size.width * (-2.0)) + 1.0;
					z=((float)i/face_size.height * (-2.0)) + 1.0;
				}

				else if(j< 3*face_size.width)		//Left
				{
					mode =2;
					j-=(mode*face_size.width);
					y = -1.0;
					z=((float)i/face_size.height * (-2.0)) + 1.0;
					x=((float)j/face_size.width * (-2.0)) + 1.0;
				}

				else if(j< 4*face_size.width)		//Back
				{
					mode =3;
					j-=(mode*face_size.width);
					x = -1.0;
					z=((float)i/face_size.height * (-2.0)) + 1.0;
					y=((float)j/face_size.width * (2.0)) - 1.0;
				}

				else if(j< 5*face_size.width)		//Right
				{
					mode =4;
					j-=(mode*face_size.width);
					y=1.0;
					z=((float)i/face_size.height * (-2.0)) + 1.0;
					x=((float)j/face_size.width * (2.0)) - 1.0;
				}
				
				else								//Bottom
				{
					mode =5;
					j-=(mode*face_size.width);
					z = -1.0;
					x=((float)i/face_size.height * (2.0)) - 1.0;
					y=((float)j/face_size.width * (2.0)) - 1.0;

				}


				r = cvSqrt(x*x+y*y+z*z);
				theta = acos(z/r);

				if (x==0)
					phi= 0.5*PI;
				else
					phi = atan(y/x);


				switch (mode)
				{
					case 0:
						if(i>=face_size.height/2)	phi+=PI;
						break;
					case 1:
						break;
					case 2:
						if(phi>0) phi+=PI;
						break;
					case 3:
						phi += PI;
						break;
					case 4:
						if (phi<0) phi += PI;
						break;
					case 5:
						if(i<=face_size.height/2)	phi+=PI;
						break;
				}

				a = (int)(theta / PI * sph_size.height + 0.5);
				b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

				if(b<0)	b+=sph_size.width;
				if(b>sph_size.width-1)	b-=sph_size.width;
				if(a<0)	a=0;
				if(a>sph_size.height-1)	a=sph_size.height-1;

				spherical[k][t].x = b;
				spherical[k][t].y = a;

			}

		}

	}


}

void persp_depth_cube_generation(CvMat *spherical, CvMat *cube_face[6], IplImage *normal_face[6], IplImage *kinect_normal_face[6], CvSize2D32f pixel_size)
{
	int i,j;				//pixel position on face image
	int a,b;				//pixel position on spherical image
	float x, y, z;			//Cartesian coord
	float r, theta, phi;	//Spherical coord
	double src;
	CvPoint3D32f dst;

	CvSize sph_size = cvGetSize(spherical);
	CvSize face_size = cvGetSize(cube_face[0]);

	CvPoint3D32f **vertice;
	CvPoint3D32f **vertice_normal;

	vertice = new CvPoint3D32f*[face_size.height];
	vertice_normal = new CvPoint3D32f*[face_size.height];

	for(i=0; i<face_size.height; i++)
	{
		vertice[i] = new CvPoint3D32f[face_size.width];
		vertice_normal[i] = new CvPoint3D32f[face_size.width];
	}

	CvPoint3D32f input_points[3];
	//CvPoint3D32f vertex_normal;
	uchar *curr, *prec;

	//For top image
	cvZero( cube_face[0] );
	z=1.0;
	for(i=0; i<face_size.height; i++)
	{
		x=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);

			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			if(i>=face_size.height/2)
				phi+=PI;


			a = floor(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - floor(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;
			if(a<0)	a=0;
			if(a>sph_size.height-1)	a=sph_size.height-1;

			src = cvGetReal2D( spherical, a, b );
			if(src ==0)
			{
				cvSetReal2D(cube_face[0], i, j, max_radius); 
				vertice[i][j] = cvPoint3D32f( 0.0, 0.0, 0.0 );
				continue;
			}
			vertice[i][j] = vertex_calculation(src, a, b );
			cvSetReal2D(cube_face[0], i, j, vertice[i][j].z); 
		}
	}

	//normal calculation
	face_normal_calculation(vertice, vertice_normal, normal_face[0]);
	
	for (int i=0; i<face_size.height; i++)
		for (int j=0; j<face_size.width; j++)
		{
			curr = &CV_IMAGE_ELEM( kinect_normal_face[0] , uchar, i, j*3 );
			curr[2] = (uchar)(((1)*(vertice_normal[i][j].y)+1)*127);			// OpenCV uses BGR
			curr[1] = (uchar)(((1)*(vertice_normal[i][j].z)+1)*127);
			curr[0] = (uchar)(((1)*(vertice_normal[i][j].x)+1)*127);
		}


	//For front image
	cvZero( cube_face[1] );
	x=1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (-2.0)) + 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			a = floor(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - floor(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = cvGetReal2D( spherical, a, b );
			if(src ==0)
			{
				cvSetReal2D(cube_face[1], i, j, max_radius);
				vertice[i][j] = cvPoint3D32f( 0.0, 0.0, 0.0 );
				continue;
			}
			vertice[i][j] = vertex_calculation(src, a, b );
			cvSetReal2D(cube_face[1], i, j, vertice[i][j].x); 
		}
	}

	//normal calculation
	face_normal_calculation(vertice, vertice_normal, normal_face[1]);
	
	for (int i=0; i<face_size.height; i++)
		for (int j=0; j<face_size.width; j++)
		{
			curr = &CV_IMAGE_ELEM( kinect_normal_face[1] , uchar, i, j*3 );
			curr[2] = (uchar)(((-1)*(vertice_normal[i][j].y)+1)*127);			// OpenCV uses BGR
			curr[1] = (uchar)(((1)*(vertice_normal[i][j].x)+1)*127);
			curr[0] = (uchar)(((1)*(vertice_normal[i][j].z)+1)*127);

		}


	//For left image
	cvZero( cube_face[2] );
	y = -1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			x=((float)j/face_size.width * (-2.0)) + 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			if(phi>0) phi+=PI;

			a = floor(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - floor(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = cvGetReal2D( spherical, a, b );
			if(src ==0)
			{
				cvSetReal2D(cube_face[2], i, j, max_radius); 
				vertice[i][j] = cvPoint3D32f( 0.0, 0.0, 0.0 );
				continue;
			}
			vertice[i][j] = vertex_calculation(src, a, b );
			cvSetReal2D(cube_face[2], i, j, vertice[i][j].y); 


		}
	}
	//normal calculation
	face_normal_calculation(vertice, vertice_normal, normal_face[2]);
	
	for (int i=0; i<face_size.height; i++)
		for (int j=0; j<face_size.width; j++)
		{
			curr = &CV_IMAGE_ELEM( kinect_normal_face[2] , uchar, i, j*3 );
			curr[2] = (uchar)(((-1)*(vertice_normal[i][j].x)+1)*127);			// OpenCV uses BGR
			curr[1] = (uchar)(((-1)*(vertice_normal[i][j].y)+1)*127);
			curr[0] = (uchar)(((1)*(vertice_normal[i][j].z)+1)*127);
		}


	//For back image
	cvZero( cube_face[3] );
	x = -1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			phi += PI;

			a = floor(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - floor(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = cvGetReal2D( spherical, a, b );
			if(src ==0)
			{
				cvSetReal2D(cube_face[3], i, j, max_radius); 
				vertice[i][j] = cvPoint3D32f( 0.0, 0.0, 0.0 );
				continue;
			}
			vertice[i][j] = vertex_calculation(src, a, b );
			cvSetReal2D(cube_face[3], i, j, vertice[i][j].x); 
		}
	}


	//normal calculation
	face_normal_calculation(vertice, vertice_normal, normal_face[3]);
	
	for (int i=0; i<face_size.height; i++)
		for (int j=0; j<face_size.width; j++)
		{
			curr = &CV_IMAGE_ELEM( kinect_normal_face[3] , uchar, i, j*3 );
			curr[2] = (uchar)(((1)*(vertice_normal[i][j].y)+1)*127);			// OpenCV uses BGR
			curr[1] = (uchar)(((-1)*(vertice_normal[i][j].x)+1)*127);
			curr[0] = (uchar)(((1)*(vertice_normal[i][j].z)+1)*127);
		}


	//For right image
	cvZero( cube_face[4] );
	y=1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			x=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			if (phi<0) phi += PI;

			a = floor(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - floor(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = cvGetReal2D( spherical, a, b );
			if(src ==0)
			{
				cvSetReal2D(cube_face[4], i, j, max_radius); 
				vertice[i][j] = cvPoint3D32f( 0.0, 0.0, 0.0 );
				continue;
			}
			vertice[i][j] = vertex_calculation(src, a, b );
			cvSetReal2D(cube_face[4], i, j, vertice[i][j].y); 
		}
	}

	//normal calculation
	face_normal_calculation(vertice, vertice_normal, normal_face[4]);
	
	for (int i=0; i<face_size.height; i++)
		for (int j=0; j<face_size.width; j++)
		{
			curr = &CV_IMAGE_ELEM( kinect_normal_face[4] , uchar, i, j*3 );
			curr[2] = (uchar)(((1)*(vertice_normal[i][j].x)+1)*127);			// OpenCV uses BGR
			curr[1] = (uchar)(((1)*(vertice_normal[i][j].y)+1)*127);
			curr[0] = (uchar)(((1)*(vertice_normal[i][j].z)+1)*127);
		}



	//For bottom image
	cvZero( cube_face[5] );
	z = -1.0;
	for(i=0; i<face_size.height; i++)
	{
		x=((float)i/face_size.height * (2.0)) - 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);

			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			if(i<=face_size.height/2)
				phi+=PI;

			a = floor(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - floor(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			if(a<0)	a=0;
			if(a>sph_size.height-1)	a=sph_size.height-1;

			src = cvGetReal2D( spherical, a, b );
			if(src ==0)
			{
				cvSetReal2D(cube_face[5], i, j, max_radius); 
				vertice[i][j] = cvPoint3D32f( 0.0, 0.0, 0.0 );
				continue;
			}
			vertice[i][j] = vertex_calculation(src, a, b );
			cvSetReal2D(cube_face[5], i, j, vertice[i][j].z); 
		}
	}

	//normal calculation
	face_normal_calculation(vertice, vertice_normal, normal_face[5]);

	for (int i=0; i<face_size.height; i++)
		for (int j=0; j<face_size.width; j++)
		{
			curr = &CV_IMAGE_ELEM( kinect_normal_face[5] , uchar, i, j*3 );
			curr[2] = (uchar)(((-1)*(vertice_normal[i][j].y)+1)*127);			// OpenCV uses BGR
			curr[1] = (uchar)(((-1)*(vertice_normal[i][j].z)+1)*127);
			curr[0] = (uchar)(((1)*(vertice_normal[i][j].x)+1)*127);
		}



	for(i=0; i<face_size.height; i++)
	{
		delete vertice[i];
		delete vertice_normal[i];
	}

	delete vertice;
	delete vertice_normal;



}

void spherical2cube(IplImage *spherical, IplImage *cube, IplImage *cube_face[6], CvSize2D32f pixel_size)
{
	int i,j;				//pixel position on face image
	int a,b;				//pixel position on spherical image
	float x, y, z;			//Cartesian coord
	float r, theta, phi;	//Spherical coord
	uchar *src, *dst;

	CvSize sph_size = cvGetSize(spherical);
	CvSize cube_size = cvGetSize(cube);
	CvSize face_size = cvGetSize(cube_face[0]);

	cvZero( cube );

	//For top image
	cvZero( cube_face[0] );
	z=1.0;
	for(i=0; i<face_size.height; i++)
	{
		x=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);

			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			if(i>=face_size.height/2)
				phi+=PI;


			a = (int)(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;
			if(a<0)	a=0;
			if(a>sph_size.height-1)	a=sph_size.height-1;

			src = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
			dst = &CV_IMAGE_ELEM( cube_face[0], uchar, i, j*3 );
			
			dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
		}
	}
	cvSetImageROI( cube, cvRect( 0, 0, face_size.width, face_size.height ) );
	cvCopy( cube_face[0], cube );
	cvResetImageROI( cube );
	
	//For front image
	cvZero( cube_face[1] );
	x=1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (-2.0)) + 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			a = (int)(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
			dst = &CV_IMAGE_ELEM( cube_face[1], uchar, i, j*3 );
			
			dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
		}
	}
	cvSetImageROI( cube, cvRect( face_size.width, 0, face_size.width, face_size.height ) );
	cvCopy( cube_face[1], cube );
	cvResetImageROI( cube );


	//For left image
	cvZero( cube_face[2] );
	y = -1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			x=((float)j/face_size.width * (-2.0)) + 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			if(phi>0) phi+=PI;

			a = (int)(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
			dst = &CV_IMAGE_ELEM( cube_face[2], uchar, i, j*3 );
			
			dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
		}
	}
	cvSetImageROI( cube, cvRect( face_size.width*2, 0, face_size.width, face_size.height ) );
	cvCopy( cube_face[2], cube );
	cvResetImageROI( cube );

	//For back image
	cvZero( cube_face[3] );
	x = -1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			phi += PI;

			a = (int)(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
			dst = &CV_IMAGE_ELEM( cube_face[3], uchar, i, j*3 );
			
			dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
		}
	}
	cvSetImageROI( cube, cvRect( face_size.width*3, 0, face_size.width, face_size.height ) );
	cvCopy( cube_face[3], cube );
	cvResetImageROI( cube );

	//For right image
	cvZero( cube_face[4] );
	y=1.0;
	for(i=0; i<face_size.height; i++)
	{
		z=((float)i/face_size.height * (-2.0)) + 1.0;

		for(j=0; j<face_size.width; j++)
		{
			x=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);
			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			//if(i==face_size.width/2)
			//	printf("%d  ", (int)(phi*180/PI));

			if (phi<0) phi += PI;

			a = (int)(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			src = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
			dst = &CV_IMAGE_ELEM( cube_face[4], uchar, i, j*3 );
			
			dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
		}
	}
	cvSetImageROI( cube, cvRect( face_size.width*4, 0, face_size.width, face_size.height ) );
	cvCopy( cube_face[4], cube );
	cvResetImageROI( cube );

	//For bottom image
	cvZero( cube_face[5] );
	z = -1.0;
	for(i=0; i<face_size.height; i++)
	{
		x=((float)i/face_size.height * (2.0)) - 1.0;

		for(j=0; j<face_size.width; j++)
		{
			y=((float)j/face_size.width * (2.0)) - 1.0;

			r = cvSqrt(x*x+y*y+z*z);
			theta = acos(z/r);

			if (x==0)
				phi= 0.5*PI;
			else
				phi = atan(y/x);

			if(i<=face_size.height/2)
				phi+=PI;

			a = (int)(theta / PI * sph_size.height + 0.5);
			b = sph_size.width -1 - (int)(phi / (2*PI) * sph_size.width + 0.5);

			if(b<0)	b+=sph_size.width;
			if(b>sph_size.width-1)	b-=sph_size.width;

			if(a<0)	a=0;
			if(a>sph_size.height-1)	a=sph_size.height-1;

			src = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
			dst = &CV_IMAGE_ELEM( cube_face[5], uchar, i, j*3 );
			
			dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
		}
	}
	cvSetImageROI( cube, cvRect( face_size.width*5, 0, face_size.width, face_size.height ) );
	cvCopy( cube_face[5], cube );
	cvResetImageROI( cube );
	
}


void cube2spherical(IplImage *cube, IplImage *spherical, CvSize2D32f pixel_size)
{
	int i,j;				//pixel position on face image
	int a,b;				//pixel position on spherical image
	float x, y, z;			//Cartesian coord
	float r, theta, phi;	//Spherical coord

	uchar *src, *dst;

	CvSize sph_size = cvGetSize(spherical);
	CvSize cube_size = cvGetSize(cube);
	CvSize face_size = cvSize(cube_size.height, cube_size.height);
	IplImage *face = cvCreateImage( face_size, 8, 3 );

	cvZero( spherical );

	//For top image
	cvSetImageROI( cube, cvRect( 0, 0, face_size.width, face_size.height ) );
	cvCopy( cube, face);
	cvResetImageROI( cube );

	z=1.0;

	for(a=0; a<sph_size.height/3; a++)
	{
		theta = (float)a/sph_size.height * PI;

		for(b=0; b<sph_size.width; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= z/cos(theta);

			x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			//z= r * cos(theta)

			i= (int)((x - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y + 1.0)/2.0 * face_size.width );

			if(j<0) j=0;
			if(j>face_size.width-1) j=face_size.width-1;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}
	}

	//For front image
	cvSetImageROI( cube, cvRect( face_size.width*1, 0, face_size.width, face_size.height ) );
	cvCopy( cube, face);
	cvResetImageROI( cube );

	x = 1.0;

	for(a=sph_size.height/4; a<sph_size.height*3/4; a++)
	{
		theta = (float)a/sph_size.height * PI;

		for(b=0; b<sph_size.width/8; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= x / sin(theta) / cos(phi);

			//x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y - 1.0)/(-2.0) * face_size.width );

			if(j<0) j=0;
			if(j>face_size.width-1) j=face_size.width-1;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}

		for(b=sph_size.width*7/8; b<sph_size.width; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= x / sin(theta) / cos(phi);

			//x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y - 1.0)/(-2.0) * face_size.width );

			if(j<0) j=0;
			if(j>face_size.width-1) j=face_size.width-1;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}

	}

	//For left image
	cvSetImageROI( cube, cvRect( face_size.width*2, 0, face_size.width, face_size.height ) );
	cvCopy( cube, face);
	cvResetImageROI( cube );

	y = -1.0;

	for(a=sph_size.height/4; a<sph_size.height*3/4; a++)
	{
		theta = (float)a/sph_size.height * PI;

		for(b=sph_size.width/8; b<sph_size.width*3/8; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= y / sin(theta) /sin(phi);

			x= r * sin(theta) * cos(phi);
			//y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (x - 1.0)/(-2.0) * face_size.width );

			if(j<0) j=0;
			if(j>face_size.width-1) j=face_size.width-1;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}
	}

	//For back image
	cvSetImageROI( cube, cvRect( face_size.width*3, 0, face_size.width, face_size.height ) );
	cvCopy( cube, face);
	cvResetImageROI( cube );

	x = -1.0;

	for(a=sph_size.height/4; a<sph_size.height*3/4; a++)
	{
		theta = (float)a/sph_size.height * PI;

		for(b=sph_size.width*3/8; b<sph_size.width*5/8; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= x / sin(theta) /cos(phi);

			//x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y + 1.0)/(2.0) * face_size.width );

			if(j<0) j=0;
			if(j>face_size.width-1) j=face_size.width-1;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}
	}

	//For right image
	cvSetImageROI( cube, cvRect( face_size.width*4, 0, face_size.width, face_size.height ) );
	cvCopy( cube, face);
	cvResetImageROI( cube );

	y = 1.0;

	for(a=sph_size.height/4; a<sph_size.height*3/4; a++)
	{
		theta = (float)a/sph_size.height * PI;

		for(b=sph_size.width*5/8; b<sph_size.width*7/8; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= y / sin(theta) / sin(phi);

			x= r * sin(theta) * cos(phi);
			//y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (x + 1.0)/(2.0) * face_size.width );

			if(j<0) j=0;
			if(j>face_size.width-1) j=face_size.width-1;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}
	}

	//For bottom image
	cvSetImageROI( cube, cvRect( face_size.width*5, 0, face_size.width, face_size.height ) );
	cvCopy( cube, face);
	cvResetImageROI( cube );

	z = -1.0;

	for(a=sph_size.height*2/3; a<sph_size.height; a++)
	{
		theta = (float)a/sph_size.height * PI;

		for(b=0; b<sph_size.width; b++)
		{
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= z/cos(theta);

			x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			//z= r * cos(theta)

			i= (int)((x + 1.0)/(2.0) * face_size.height);
			j= (int)( (y + 1.0)/2.0 * face_size.width );

			if(j<0 || j>face_size.width-1) continue;

			if(i>=0 && i<face_size.height)
			{
				src = &CV_IMAGE_ELEM( face, uchar, i, j*3 );
				dst = &CV_IMAGE_ELEM( spherical, uchar, a, b*3 );
				
				dst[0]=src[0]; dst[1]=src[1]; dst[2]=src[2];
			}
		}
	}
}

CvPoint spherical2cube_point(CvPoint sph_pt)
{
	int i=(-1), j=(-1);		//pixel position on face image
	int a,b;				//pixel position on spherical image
	CvPoint cube_pt;
	float x, y, z;			//Cartesian coord
	float r, theta, phi;	//Spherical coord

	uchar *src, *dst;

	CvSize sph_size = img_size;
	CvSize face_size = cvSize(cube_size.height, cube_size.height);

	a=sph_pt.y;
	b=sph_pt.x;

	if( a<sph_size.height/3 )	//top image
	{
		z=1.0;

		theta = (float)a/sph_size.height * PI;
		phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;
		r= z/cos(theta);

		x= r * sin(theta) * cos(phi);
		y= r * sin(theta) * sin(phi);

		i= (int)((x - 1.0)/(-2.0) * face_size.height);
		j= (int)( (y + 1.0)/2.0 * face_size.width );


		if(i>=0 && i<face_size.height && j>=0 && j<face_size.width )
		{
			cube_pt.y = i;
			cube_pt.x = j;

			return cube_pt;
		}
	}

	if(a>sph_size.height/4 && a<sph_size.height*3/4)	//middle images
	{
		if(b<sph_size.width/8) 			//left face (half)
		{
			x = 1.0;

			theta = (float)a/sph_size.height * PI;
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;
			r= x / sin(theta) / cos(phi);

			//x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y - 1.0)/(-2.0) * face_size.width );

			if(i>=0 && i<face_size.height && j>=0 && j<face_size.width)
			{
				cube_pt.y = i;
				cube_pt.x = face_size.width + j;
				return cube_pt;
			}
		}

		if(b>=sph_size.width*7/8 && b<sph_size.width)	//left face (half)
		{
			x = 1.0;

			theta = (float)a/sph_size.height * PI;
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;
			r= x / sin(theta) / cos(phi);

			//x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y - 1.0)/(-2.0) * face_size.width );

			if(i>=0 && i<face_size.height && j>=0 && j<face_size.width)
			{
				cube_pt.y = i;
				cube_pt.x = face_size.width + j;
				return cube_pt;
			}
		}

		if(b>=sph_size.width/8 && b<sph_size.width*3/8) //front face
		{
			y = -1.0;
			theta = (float)a/sph_size.height * PI;
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= y / sin(theta) /sin(phi);

			x= r * sin(theta) * cos(phi);
			//y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (x - 1.0)/(-2.0) * face_size.width );

			if(i>=0 && i<face_size.height && j>=0 && j<face_size.width)
			{
				cube_pt.y = i;
				cube_pt.x = face_size.width * 2 + j;
				return cube_pt;
			}
		}

		if(b>=sph_size.width*3/8 && b<sph_size.width*5/8) //right face
		{
			x = -1.0;
			theta = (float)a/sph_size.height * PI;
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= x / sin(theta) /cos(phi);

			//x= r * sin(theta) * cos(phi);
			y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (y + 1.0)/(2.0) * face_size.width );

			if(i>=0 && i<face_size.height && j>=0 && j<face_size.width)
			{
				cube_pt.y = i;
				cube_pt.x = face_size.width * 3 + j;
				return cube_pt;
			}
		}

		if(b>=sph_size.width*5/8 && b<sph_size.width*7/8) //back face
		{
			y = 1.0;
			theta = (float)a/sph_size.height * PI;
			phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;

			r= y / sin(theta) / sin(phi);

			x= r * sin(theta) * cos(phi);
			//y= r * sin(theta) * sin(phi);
			z= r * cos(theta);

			i= (int)((z - 1.0)/(-2.0) * face_size.height);
			j= (int)( (x + 1.0)/(2.0) * face_size.width );

			if(i>=0 && i<face_size.height && j>=0 && j<face_size.width)
			{
				cube_pt.y = i;
				cube_pt.x = face_size.width * 4 + j;
				return cube_pt;
			}
		}
	}

	if(a>=sph_size.height*2/3 && a<sph_size.height)		//bottom face
	{
		z = -1.0;
		theta = (float)a/sph_size.height * PI;
		phi = 2*PI - ((float)b/sph_size.width * 2 * PI) - 0.00001 ;
		r= z/cos(theta);

		x= r * sin(theta) * cos(phi);
		y= r * sin(theta) * sin(phi);
		//z= r * cos(theta)

		i= (int)((x + 1.0)/(2.0) * face_size.height);
		j= (int)( (y + 1.0)/2.0 * face_size.width );

		if(i>=0 && i<face_size.height && j>=0 && j<face_size.width)
		{
			cube_pt.y = i;
			cube_pt.x = face_size.width * 5 + j;
			return cube_pt;
		}
	}

	cube_pt.y =0;
	cube_pt.x =0;
	return cube_pt;		// no projection point. error!!!

}

void face_normal_calculation(CvPoint3D32f **vertice, CvPoint3D32f **vertice_normal, IplImage *normal_face)
{
	int i,j,k;
	CvSize face_size = cvGetSize(normal_face);
	CvPoint3D32f input_points[3];
	//CvPoint3D32f vertex_normal;
	uchar *curr, *prec;
	bool FAIL_TAG;

	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			input_points[1]= vertice[i][j];
			if(input_points[1].x ==0 && input_points[1].y ==0 && input_points[1].z ==0)
			{	
				vertice_normal[i][j] = cvPoint3D32f(-1.0, -1.0, -1.0);
				curr = &CV_IMAGE_ELEM( normal_face, uchar, i, j*3 );
				curr[0] = (uchar)((vertice_normal[i][j].x+1)*127);
				curr[1] = (uchar)((vertice_normal[i][j].y+1)*127);
				curr[2] = (uchar)((vertice_normal[i][j].z+1)*127);
				continue;
			}

			input_points[0]= vertice[i+1][j];
			FAIL_TAG=0;
			if(input_points[0].x==input_points[1].x && input_points[0].y==input_points[1].y && input_points[0].z==input_points[1].z )
			{
				FAIL_TAG=1;
				for(k=2; k<face_size.height-i; k++)
				{
					input_points[0]= vertice[i+k][j];
					if(input_points[0].x!=input_points[1].x || input_points[0].y!=input_points[1].y || input_points[0].z!=input_points[1].z )
					{
						FAIL_TAG=0;
						break;
					}
				}
			}
			if(FAIL_TAG==1)
			{
				prec =  &CV_IMAGE_ELEM( normal_face, uchar, i-1, j*3 );
				curr = &CV_IMAGE_ELEM( normal_face, uchar, i, j*3 );
				curr[0] = prec[0]; curr[1] = prec[1]; curr[1] = prec[1]; 
				continue;
			}

			input_points[2]= vertice[i][j+1];
			FAIL_TAG=0;
			if(input_points[2].x==input_points[1].x && input_points[2].y==input_points[1].y && input_points[2].z==input_points[1].z )
			{
				FAIL_TAG=1;
				for(k=2; k<face_size.width-i; k++)
				{
					input_points[2]= vertice[i][j+k];
					if(input_points[2].x!=input_points[1].x || input_points[2].y!=input_points[1].y || input_points[2].z!=input_points[1].z )
					{
						FAIL_TAG=0;
						break;
					}
				}
			}
			if(FAIL_TAG==1)
			{
				prec =  &CV_IMAGE_ELEM( normal_face, uchar, i, (j-1)*3 );
				curr = &CV_IMAGE_ELEM( normal_face, uchar, i, j*3 );
				curr[0] = prec[0]; curr[1] = prec[1]; curr[1] = prec[1]; 
				continue;
			}

			vertice_normal[i][j] = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face, uchar, i, j*3 );
			curr[0] = (uchar)((vertice_normal[i][j].x+1)*127);
			curr[1] = (uchar)((vertice_normal[i][j].y+1)*127);
			curr[2] = (uchar)((vertice_normal[i][j].z+1)*127);

			//if(i==567 && j==353)
			//{
			//	printf("vertice 0: %f %f %f vertice 1: %f %f %f vertice 2: %f %f %f \n", input_points[1].x, input_points[1].y, input_points[1].z, 
			//		input_points[0].x, input_points[0].y, input_points[0].z, input_points[2].x, input_points[2].y, input_points[2].z);
			//	printf("vertex normal: %f %f %f\n", vertex_normal.x, vertex_normal.y, vertex_normal.z );

			//}
		}
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face, uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
		vertice_normal[i][j] = vertice_normal[i][j-1];

	}
	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face, uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face, uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];

		vertice_normal[i][j] = vertice_normal[i-1][j];
	}
}

void cube_normal_calculation(CvMat *depth_face[6], IplImage *normal_face[6])
{
	int i,j;				//pixel position on face image
	float x, y, z;			//Cartesian coord

	CvPoint3D32f input_points[3];
	CvPoint3D32f vertex_normal;
	uchar *curr, *prec;

	CvSize face_size = cvGetSize(depth_face[0]);

	//For top image
	z=1.0;
	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			x=((float)i/face_size.height * (-2.0)) + 1.0;
			y=((float)j/face_size.width * (2.0)) - 1.0;
			input_points[1].z = cvGetReal2D( depth_face[0], i, j );
			input_points[1].x = x * abs(input_points[1].z);
			input_points[1].y = y * abs(input_points[1].z);

			x=((float)(i+1)/face_size.height * (-2.0)) + 1.0;
			input_points[0].z = cvGetReal2D( depth_face[0], i+1, j );
			input_points[0].x = x * abs(input_points[0].z);
			input_points[0].y = y * abs(input_points[0].z);
			
			x=((float)i/face_size.height * (-2.0)) + 1.0;
			y=((float)(j+1)/face_size.width * (2.0)) - 1.0;
			input_points[2].z = cvGetReal2D( depth_face[0], i, j );
			input_points[2].x = x * abs(input_points[2].z);
			input_points[2].y = y * abs(input_points[2].z);

			vertex_normal = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face[0], uchar, i, j*3 );
			curr[0] = (uchar)((vertex_normal.x+1)*128);
			curr[1] = (uchar)((vertex_normal.y+1)*128);
			curr[2] = (uchar)((vertex_normal.z+1)*128);
		}
		
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face[0], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}
	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face[0], uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face[0], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}


	//For front image
	x=1.0;
	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			z=((float)i/face_size.height * (-2.0)) + 1.0;
			y=((float)j/face_size.width * (-2.0)) + 1.0;
			input_points[1].x = cvGetReal2D( depth_face[1], i, j );
			input_points[1].z = z * abs(input_points[1].x);
			input_points[1].y = y * abs(input_points[1].x);

			z=((float)(i+1)/face_size.height * (-2.0)) + 1.0;
			input_points[0].x = cvGetReal2D( depth_face[1], i+1, j );
			input_points[0].z = z * abs(input_points[0].x);
			input_points[0].y = y * abs(input_points[0].x);

			z=((float)i/face_size.height * (-2.0)) + 1.0;
			y=((float)(j+1)/face_size.width * (-2.0)) + 1.0;
			input_points[2].x = cvGetReal2D( depth_face[1], i, j+1 );
			input_points[2].z = z * abs(input_points[2].x);
			input_points[2].y = y * abs(input_points[2].x);

			vertex_normal = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face[1], uchar, i, j*3 );
			curr[0] = (uchar)((vertex_normal.x+1)*128);
			curr[1] = (uchar)((vertex_normal.y+1)*128);
			curr[2] = (uchar)((vertex_normal.z+1)*128);
		}
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face[1], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}
	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face[1], uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face[1], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}

	//For left image
	y = -1.0;
	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			z=((float)i/face_size.height * (-2.0)) + 1.0;
			x=((float)j/face_size.width * (-2.0)) + 1.0;
			input_points[1].y = cvGetReal2D( depth_face[2], i, j );
			input_points[1].z = z * abs(input_points[1].y);
			input_points[1].x = x * abs(input_points[1].y);

			z=((float)(i+1)/face_size.height * (-2.0)) + 1.0;
			input_points[0].y = cvGetReal2D( depth_face[2], i+1, j );
			input_points[0].z = z * abs(input_points[0].y);
			input_points[0].x = x * abs(input_points[0].y);

			z=((float)i/face_size.height * (-2.0)) + 1.0;
			x=((float)(j+1)/face_size.width * (-2.0)) + 1.0;
			input_points[2].y = cvGetReal2D( depth_face[2], i, j+1 );
			input_points[2].z = z * abs(input_points[2].y);
			input_points[2].x = x * abs(input_points[2].y);

			vertex_normal = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face[2], uchar, i, j*3 );
			curr[0] = (uchar)((vertex_normal.x+1)*128);
			curr[1] = (uchar)((vertex_normal.y+1)*128);
			curr[2] = (uchar)((vertex_normal.z+1)*128);
		}
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face[2], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}

	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face[2], uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face[2], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}


	//For back image
	x = -1.0;
	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			z=((float)i/face_size.height * (-2.0)) + 1.0;
			y=((float)j/face_size.width * (2.0)) - 1.0;
			input_points[1].x = cvGetReal2D( depth_face[3], i, j );
			input_points[1].z = z * abs(input_points[1].x);
			input_points[1].y = y * abs(input_points[1].x);
			
			z=((float)(i+1)/face_size.height * (-2.0)) + 1.0;
			input_points[0].x = cvGetReal2D( depth_face[3], i+1, j );
			input_points[0].z = z * abs(input_points[0].x);
			input_points[0].y = y * abs(input_points[0].x);

			z=((float)i/face_size.height * (-2.0)) + 1.0;
			y=((float)(j+1)/face_size.width * (2.0)) - 1.0;
			input_points[2].x = cvGetReal2D( depth_face[3], i, j+1 );
			input_points[2].z = z * abs(input_points[2].x);
			input_points[2].y = y * abs(input_points[2].x);

			vertex_normal = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face[3], uchar, i, j*3 );
			curr[0] = (uchar)((vertex_normal.x+1)*128);
			curr[1] = (uchar)((vertex_normal.y+1)*128);
			curr[2] = (uchar)((vertex_normal.z+1)*128);
		}
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face[3], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}
	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face[3], uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face[3], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];

	}


	//For right image
	y=1.0;
	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			z=((float)i/face_size.height * (-2.0)) + 1.0;
			x=((float)j/face_size.width * (2.0)) - 1.0;
			input_points[1].y = cvGetReal2D( depth_face[4], i, j );
			input_points[1].z = z * abs(input_points[1].y);
			input_points[1].x = x * abs(input_points[1].y);

			z=((float)(i+1)/face_size.height * (-2.0)) + 1.0;
			input_points[0].y = cvGetReal2D( depth_face[4], i+1, j );
			input_points[0].z = z * abs(input_points[0].y);
			input_points[0].x = x * abs(input_points[0].y);

			z=((float)i/face_size.height * (-2.0)) + 1.0;
			x=((float)(j+1)/face_size.width * (2.0)) - 1.0;
			input_points[2].y = cvGetReal2D( depth_face[4], i, j+1 );
			input_points[2].z = z * abs(input_points[2].y);
			input_points[2].x = x * abs(input_points[2].y);

			vertex_normal = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face[4], uchar, i, j*3 );
			curr[0] = (uchar)((vertex_normal.x+1)*128);
			curr[1] = (uchar)((vertex_normal.y+1)*128);
			curr[2] = (uchar)((vertex_normal.z+1)*128);
		}
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face[4], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}
	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face[4], uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face[4], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}

	//For bottom image
	z = -1.0;
	for(i=0; i<face_size.height-1; i++)
	{
		for(j=0; j<face_size.width-1; j++)
		{
			x=((float)i/face_size.height * (2.0)) - 1.0;
			y=((float)j/face_size.width * (2.0)) - 1.0;
			input_points[1].z = cvGetReal2D( depth_face[5], i, j );
			input_points[1].x = x * abs(input_points[1].z);
			input_points[1].y = y * abs(input_points[1].z);

			x=((float)(i+1)/face_size.height * (2.0)) - 1.0;
			input_points[0].z = cvGetReal2D( depth_face[5], i+1, j );
			input_points[0].x = x * abs(input_points[0].z);
			input_points[0].y = y * abs(input_points[0].z);

			x=((float)i/face_size.height * (2.0)) - 1.0;
			y=((float)(j+1)/face_size.width * (2.0)) - 1.0;
			input_points[2].z = cvGetReal2D( depth_face[5], i, j+1 );
			input_points[2].x = x * abs(input_points[2].z);
			input_points[2].y = y * abs(input_points[2].z);

			vertex_normal = calculate_normal(input_points);

			curr = &CV_IMAGE_ELEM( normal_face[5], uchar, i, j*3 );
			curr[0] = (uchar)((vertex_normal.x+1)*128);
			curr[1] = (uchar)((vertex_normal.y+1)*128);
			curr[2] = (uchar)((vertex_normal.z+1)*128);
		}
		j=face_size.width-1;
		prec =  &CV_IMAGE_ELEM( normal_face[5], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}
	i=face_size.height-1;
	for(j=0; j<face_size.width; j++)
	{
		curr = &CV_IMAGE_ELEM( normal_face[5], uchar, i-1, j*3 );
		prec =  &CV_IMAGE_ELEM( normal_face[5], uchar, i, j*3 );
		prec[0]=curr[0]; prec[1]=curr[1]; prec[2]=curr[2];
	}

}


CvPoint3D32f vertex_calculation(double disp, int i, int j )
{
	int k;
	double radius;
	CvPoint3D32f vert;
	double unit_h = 1.0/(img_size.height);
	double unit_w = 2.0/(img_size.width);

	float latitude	= i*unit_h * CV_PI;
	float longitude	= j*unit_w * CV_PI;
	double angle_disp	= (disp/disp_scale + disp_offset) * unit_h * CV_PI;  //model reconstruction

	if (latitude+angle_disp <0)
		angle_disp=0.01;

	if(angle_disp==0 || disp==0)
		radius=max_radius;
	else
		radius	= baseline/((sin(latitude)/tan(latitude+angle_disp))-cos(latitude));
		
	if(radius > max_radius)
		radius=max_radius;

	if(radius < 0.0)
		radius=max_radius;

	vert.x = (-1)*radius*sin(latitude)*cos(CV_PI-longitude);
	vert.y = (-1)*radius*sin(latitude)*sin(CV_PI-longitude);
	vert.z = radius*cos(latitude);

	return vert;
}

CvPoint3D32f calculate_normal(CvPoint3D32f vetice_point[])
{
	CvPoint3D32f	normal;

	CvMat *vect1 = cvCreateMat( 3, 1, CV_32FC1 );
	CvMat *vect2 = cvCreateMat( 3, 1, CV_32FC1 );
	CvMat *vect3 = cvCreateMat( 3, 1, CV_32FC1 );
				
	cvSetReal2D(vect1,0,0, vetice_point[2].x - vetice_point[1].x);
	cvSetReal2D(vect1,1,0, vetice_point[2].y - vetice_point[1].y);
	cvSetReal2D(vect1,2,0, vetice_point[2].z - vetice_point[1].z);
	cvSetReal2D(vect2,0,0, vetice_point[1].x - vetice_point[0].x);
	cvSetReal2D(vect2,1,0, vetice_point[1].y - vetice_point[0].y);
	cvSetReal2D(vect2,2,0, vetice_point[1].z - vetice_point[0].z);

	cvCrossProduct(vect1, vect2, vect3);

	double sum = cvNorm(vect3);

	normal.x = cvGetReal2D(vect3,0,0)/sum;
	normal.y = cvGetReal2D(vect3,1,0)/sum;
	normal.z = cvGetReal2D(vect3,2,0)/sum;

	return normal;
}

void normal_classification(IplImage *input, IplImage *output)
{
	int i,j,k;				
	float x, y, z;			
	uchar min, max;

	uchar *curr, *prec;

	CvSize face_size = cvGetSize(input);

	for(i=0; i<face_size.height; i++)
	{
		for(j=0; j<face_size.width; j++)
		{
			max=0;
			min=255;
			
			curr = &CV_IMAGE_ELEM( input, uchar, i, j*3 );
			prec = &CV_IMAGE_ELEM( output, uchar, i, j*3 );

			for(k=0; k<3; k++)
			{
				if(curr[k]>max)
					max=curr[k];
				if(curr[k]<min)
					min=curr[k];
			}

			if((254-min)>max)
			{
				for(k=0; k<3; k++)
				{
					if(curr[k]==min)
						prec[k]=0;
					else
						prec[k]=127;
				}
			}
			else
			{
				for(k=0; k<3; k++)
				{
					if(curr[k]==max)
						prec[k]=254;
					else
						prec[k]=127;
				}
			}
		}
	}
}


IplImage* img_resize(IplImage* src_img, int new_width,int new_height)
{
    IplImage* des_img;
    des_img=cvCreateImage(cvSize(new_width,new_height),src_img->depth,src_img->nChannels);
    cvResize(src_img,des_img,CV_INTER_LINEAR);
    return des_img;
} 