//// Object label 12 Unknow is moved to 0 none, then 12 is used for 'speaker'
//// v1.1 - Deal with grey scale disparity
////		Coordinate changed from left/back/up in Plane_Reconstruction V3.3 to Right/Front/Up in RoomModel BlockReconstruction V1.0
//// v2.0 - Modified for RoomModelReconstruction-SegNet-V1.0
////		Object labels have been changed. Speaker to 14

#include <opencv2/core.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc.hpp>
#include <iostream>

using namespace cv;
using namespace std;

#include "variable.h"
#include "IO.h"
#include "region.h"
#include "jWrite.h"


int main(int argc, char** argv)
{
	if (argc != 8)
	{
		cout << "Usage: " << argv[0] << " [Base Path] [Top Image] [Disparity Image] [Cuboid info file] [baseline diatance (in m)] [Output name (Prefix)] [Save Option]" << endl;
		cout << "	Save Option: 0: OBJ with JASON  1: OBJ with obj colours and JASON  2: OBJ with texture 3: OFF with obj colours" << endl;
		return -1;
	}

	BASE_PATH = argv[1];
	TOP_IMAGE_NAME = argv[2];
	DISP_IMAGE_NAME = argv[3];
	CUBOID_INFO_NAME = argv[4];
	baseline = atof(argv[5]);
	OUTPUT_PREFIXE = argv[6];
	OBJ_SAVE = atoi(argv[7]);

	cube_info = new float*[MAX_OBJECTS];
	new_region = new float*[MAX_OBJECTS];
	for (int i = 0; i<MAX_OBJECTS; i++)
	{
		cube_info[i] = new float[9];	//[0]type (0:Invalid 1:Cuboid 2:aligned plane 3: Arb. plane) [1-3]Min x,y,z [4-6]max x,y,z [7] object index [8]Material index
		new_region[i] = new float[5];
	}


	// load cube info file
	FILE *input1;
	num_cube = 0;
	float cube_resolusion;

	if ((fopen_s(&input1, (BASE_PATH+ CUBOID_INFO_NAME).c_str(), "r")) != 0)
	{
		cout << "Could not open or find the image" << CUBOID_INFO_NAME << endl;
		exit(0);
	}

	fscanf(input1, "%d %f", &num_cube, &cube_resolusion);

	for (int i = 0; i < num_cube; i++)
	{
		for (int j = 0; j < 9; j++)
			fscanf(input1, "%f ", &cube_info[i][j]);

		if (cube_info[i][0] == 2)
		{
			room_min.x = cube_info[i][1];
			room_min.y = cube_info[i][3];
			room_min.z = cube_info[i][5];
			room_max.x = cube_info[i][2];
			room_max.y = cube_info[i][4];
			room_max.z = cube_info[i][6];
		}
		
	}

	fclose(input1);


	//Load image
	img_t = imread((BASE_PATH + TOP_IMAGE_NAME));
	if (img_t.empty()) // Check for invalid input
	{
		cout << "Could not open or find the image" << TOP_IMAGE_NAME << endl;
		return -1;
	}

	Mat img_disparity = imread((BASE_PATH + DISP_IMAGE_NAME), -1);
	if (img_disparity.empty()) 
	{
		cout << "Could not open or find the image" << DISP_IMAGE_NAME << endl;
		return -1;
	}

	disp_ch = img_disparity.channels();

	cout << "Disparity channel: " << disp_ch << endl;

	//Disparity conversion
	img_size = img_t.size();
	Mat disparity(img_disparity.size(), CV_32FC1);

	if (disp_ch == 3)
	{
		disp_scale = 1000;

		for (int i = 0; i < img_t.rows; i++)
			for (int j = 0; j < img_t.cols; j++)
			{
				disparity.at<float>(i, j) = (img_disparity.at<Vec3b>(i, j)[0] * 65536 + img_disparity.at<Vec3b>(i, j)[1] * 256 + img_disparity.at<Vec3b>(i, j)[2]);
			}
	}
	else
	{
		disp_scale = 2;

		for (int i = 0; i < img_t.rows; i++)
			for (int j = 0; j < img_t.cols; j++)
			{
				disparity.at<float>(i, j) = img_disparity.at<uchar>(i, j);
			}
	}

	//UI
	Mat img_resize;
	Size reformed_size;
	float reform_factor;

	if (img_t.cols > 1700)
		reform_factor = 2;
	else
		reform_factor = 1;

	reformed_size.height = img_size.height / reform_factor; 
	reformed_size.width = img_size.width / reform_factor;

	resize(img_t, img_resize, reformed_size);
		

	reset_new_region();
	manual_region_processing(img_resize);


	if (reform_factor != 1)
	{
		for (int i = 0; i < rect_counter; i++)
			for (int j = 1; j<5; j++)
				new_region[i][j] *= reform_factor;
	}


	////New cuboid info generation for each new_region

	max_vertice = img_size.height * img_size.width;
	v_2D = new CvPoint2D32f[max_vertice];
	vertex = new CvPoint3D32f[max_vertice];
	new_cube_info = new float[9];

	for (int i = 0; i < rect_counter; i++)
	{
		generate_new_cuboid(disparity, i);

		for (int j = 1; j < 7; j++)
			new_cube_info[j] = floor((new_cube_info[j] / cube_resolusion) + 0.5) * cube_resolusion; // quantization to the cube-resolution
		
		for (int j = 0; j < 9; j++)
			cube_info[num_cube][j] = new_cube_info[j];

		num_cube++;
	}


	printf("Number of new objects: %d \n", rect_counter);
	printf("Number of total objects: %d \n", num_cube);


	//Memory for final data

	cube_valid_face = new int*[MAX_OBJECTS];
	cube_volume = new float[MAX_OBJECTS];

	for (int i = 0; i<MAX_OBJECTS; i++)
	{
		cube_valid_face[i] = new int[7];		//[0] empty [1]Min x [2]Max_x [3]Min y [4]max y [5]min z [6] max z
	}

	for (int i = 0; i<num_cube + 1; i++)
		for (int j = 0; j<7; j++)
			cube_valid_face[i][j] = 1;

	//Cube range refinement to fit into the room boundary
	cube_boundary_refinement(num_cube);

	for (int i = 0; i<num_cube; i++)
		cube_volume[i] = (cube_info[i][2] - cube_info[i][1])*(cube_info[i][4] - cube_info[i][3])*(cube_info[i][6] - cube_info[i][5]);

	//if (CUBE_REFINEMENT)		//adjust the range of cubes so that they don't cross each other
	//{
		//cube_range_refinement(num_cube);  // Not implemented yet
	//}


	//overallping plane removal
	if (REMOVE_OVERAPPING_FACES)		//adjust the range of cubes so that they don't cross each other
	{
		remove_overlapping_faces(num_cube);
	}


	//Mesh generation
	printf("Mesh generation\n");

	new_max_vertice = MAX_VERTEX;
	new_max_face = new_max_vertice * 3;
	new_vertex = new CvPoint3D32f[new_max_vertice];
	vertex_visibility = new int[new_max_vertice];
	new_color = new CvPoint3D32f[new_max_vertice];
	new_alpha = new uchar[new_max_vertice];
	new_uv = new CvPoint2D32f[new_max_face];
	new_face = cvCreateMat(new_max_face, 6, CV_32S);
	new_rectangle = cvCreateMat(new_max_face, 8, CV_32S);
	new_valid_face = new int[new_max_face];
	
	for (int i = 0; i<new_max_vertice; i++)
		new_valid_face[i] = 1;

	for (int i = 0; i<new_max_vertice; i++)
		new_alpha[i] = 255;

	for (int i = 0; i<OBJ_CLASS_NUMBER; i++)
		obj_counter[i] = 0;

	int		obj_index, mat_index, mat_label;

	//Room boundary generation		// extract the room bounday first 
	for (int i = 0; i < num_cube; i++)
	{
		if (cube_info[i][0] == 2)
		{
			obj_index = 5;  //Floor but other faces will have walls and ceiling in the final output

			obj_counter[obj_index]++;
			obj_list[no_object][0] = obj_index;
			obj_list[no_object][1] = obj_counter[obj_index];
			obj_list[no_object][2] = new_total_f;		//start rectagle 
			cube_rectangle_generation_boundary(cube_info, i);
			obj_list[no_object][3] = new_total_f - 1;	//end rectagle 
			int		mat_index = cube_info[no_object][12];
			int		mat_label;
			mat_list[no_object][0] = obj_index;

			mat_index = mat_index / 2;	//material[0] (none) doesn't exist

			for (int j = 1; j<12; j++)
			{
				mat_label = mat_index % 2;
				mat_list[no_object][j] = mat_label;
				mat_index = mat_index / 2;
			}

			no_object++;
		}
	}

	//Object rectangles generation
	for (int i = 0; i<num_cube; i++)
	{
		if (cube_info[i][0] != 1)
			continue;

		int		obj_index = cube_info[i][7];
		obj_counter[obj_index]++;
		obj_list[no_object][0] = obj_index;
		obj_list[no_object][1] = obj_counter[obj_index];
		obj_list[no_object][2] = new_total_f;		//start rectagle 
		cube_rectangle_generation(cube_info, i);
		obj_list[no_object][3] = new_total_f - 1;	//end rectagle 

		int		mat_index = cube_info[i][8];
		int		mat_label;
		mat_list[no_object][0] = obj_index;

		mat_index = mat_index / 2;	//material[0] (none) doesn't exist

		for (int j = 1; j<12; j++)
		{
			mat_label = mat_index % 2;
			mat_list[no_object][j] = mat_label;
			mat_index = mat_index / 2;
		}

		no_object++;
	}




	//JSON buffer
	jwOpen(JSON_buffer, JSON_buflen, JW_OBJECT, JW_PRETTY);		// start JSON root object
	char object_name[512], object_full_name[512];
	no_object = 0;
	int err;

	for (int i = 0; i<num_cube; i++)	//For room bounday
	{
		if (cube_info[i][0] == 2)
		{
			//floor
			jwObj_object("Floor_1");					// object in the root object
				jwObj_string("Category", "Floor");
				jwObj_string("Material", "Wood");
				jwObj_string("Material", "Flat");
				jwObj_double("Centre_x", (cube_info[i][2] + cube_info[i][1])/2.0);
				jwObj_double("Centre_y", (cube_info[i][4] + cube_info[i][3]) / 2.0);
				jwObj_double("Centre_z", (cube_info[i][5]));
				jwObj_double("Length_x", cube_info[i][2] - cube_info[i][1]);
				jwObj_double("Length_y", cube_info[i][4] - cube_info[i][3]);
				jwObj_double("Length_z", 0.0);
				jwObj_double("absorption", 0.05);
				jwObj_double("scattering", 0.02);
			jwEnd();

			//ceiling
			jwObj_object("Ceiling_1");					// object in the root object
				jwObj_string("Category", "Ceiling");
				jwObj_string("Material", "Painted");
				jwObj_double("Centre_x", (cube_info[i][2] + cube_info[i][1]) / 2.0);
				jwObj_double("Centre_y", (cube_info[i][4] + cube_info[i][3]) / 2.0);
				jwObj_double("Centre_z", (cube_info[i][6]));
				jwObj_double("Length_x", cube_info[i][2] - cube_info[i][1]);
				jwObj_double("Length_y", cube_info[i][4] - cube_info[i][3]);
				jwObj_double("Length_z", 0.0);
				jwObj_double("absorption", 0.05);
				jwObj_double("scattering", 0.02);
			jwEnd();

			//Wall - at y_min
			jwObj_object("Wall_1");					// object in the root object
				jwObj_string("Category", "Wall");
				jwObj_string("Material", "Painted");
				jwObj_string("Material", "Wood");
				jwObj_string("Material", "Flat");
				jwObj_double("Centre_x", (cube_info[i][2] + cube_info[i][1]) / 2.0);
				jwObj_double("Centre_y", cube_info[i][4]);
				jwObj_double("Centre_z", (cube_info[i][6] + cube_info[i][5]) / 2.0);
				jwObj_double("Length_x", cube_info[i][2] - cube_info[i][1]);
				jwObj_double("Length_y", 0.0);
				jwObj_double("Length_z", cube_info[i][6] - cube_info[i][5]);
				jwObj_double("absorption", 0.05);
				jwObj_double("scattering", 0.02);
			jwEnd();

			//Wall - at x_max
			jwObj_object("Wall_2");					// object in the root object
				jwObj_string("Category", "Wall");
				jwObj_string("Material", "Painted");
				jwObj_string("Material", "Wood");
				jwObj_string("Material", "Flat");
				jwObj_double("Centre_x", cube_info[i][2]);
				jwObj_double("Centre_y", (cube_info[i][4] + cube_info[i][3]) / 2.0);
				jwObj_double("Centre_z", (cube_info[i][6] + cube_info[i][5]) / 2.0);
				jwObj_double("Length_x", 0.0);
				jwObj_double("Length_y", cube_info[i][4] - cube_info[i][3]);
				jwObj_double("Length_z", cube_info[i][6] - cube_info[i][5]);
				jwObj_double("absorption", 0.05);
				jwObj_double("scattering", 0.02);
			jwEnd();

			//Wall - at y_max
			jwObj_object("Wall_3");					// object in the root object
				jwObj_string("Category", "Wall");
				jwObj_string("Material", "Painted");
				jwObj_string("Material", "Wood");
				jwObj_string("Material", "Flat");
				jwObj_double("Centre_x", (cube_info[i][2] + cube_info[i][1]) / 2.0);
				jwObj_double("Centre_y", cube_info[i][6]);
				jwObj_double("Centre_z", (cube_info[i][6] + cube_info[i][5]) / 2.0);
				jwObj_double("Length_x", cube_info[i][2] - cube_info[i][1]);
				jwObj_double("Length_y", 0.0);
				jwObj_double("Length_z", cube_info[i][6] - cube_info[i][5]);
				jwObj_double("absorption", 0.05);
				jwObj_double("scattering", 0.02);
			jwEnd();

			//Wall - at x_min
			jwObj_object("Wall_4");					// object in the root object
			jwObj_string("Category", "Wall");
				jwObj_string("Material", "Painted");
				jwObj_string("Material", "Wood");
				jwObj_string("Material", "Flat");
				jwObj_double("Centre_x", cube_info[i][1]);
				jwObj_double("Centre_y", (cube_info[i][4] + cube_info[i][3]) / 2.0);
				jwObj_double("Centre_z", (cube_info[i][6] + cube_info[i][5]) / 2.0);
				jwObj_double("Length_x", 0.0);
				jwObj_double("Length_y", cube_info[i][4] - cube_info[i][3]);
				jwObj_double("Length_z", cube_info[i][6] - cube_info[i][5]);
				jwObj_double("absorption", 0.05);
				jwObj_double("scattering", 0.02);
			jwEnd();

			no_object++;
		}
	}


	for (int i = 0; i<num_cube; i++)
	{
		if (cube_info[i][0] != 1)
			continue;

		sprintf(object_full_name, "%s_%d", obj_name[obj_list[i][0]].c_str(), obj_list[i][1]);
		sprintf(object_name, "%s", obj_name[obj_list[i][0]].c_str());

		jwObj_object(object_full_name);					// object in the root object
		jwObj_string("Category", object_name);

		if (cube_info[i][12] == 0)
			jwObj_string("Material", "Unknown");
		else
		{
			for (int k = 1; k<12; k++)
			{
				if (mat_list[i][k] == 1)
				{
					sprintf(object_name, "%s", mat_name[k].c_str());
					jwObj_string("Material", object_name);
				}
			}
		}

		jwObj_double("Centre_x", (cube_info[i][2] + cube_info[i][1]) / 2.0);
		jwObj_double("Centre_y", (cube_info[i][4] + cube_info[i][3]) / 2.0);
		jwObj_double("Centre_z", (cube_info[i][6] + cube_info[i][5]) / 2.0);
		jwObj_double("Length_x", (cube_info[i][2] - cube_info[i][1]));
		jwObj_double("Length_y", (cube_info[i][4] - cube_info[i][3]));
		jwObj_double("Length_z", (cube_info[i][6] - cube_info[i][5]));
		jwObj_double("absorption", 0.05);
		jwObj_double("scattering", 0.02);
		jwEnd();

		no_object++;
	}


	err = jwClose();								// close and get error code


	//Saving data
	printf("Saving JSON file\n");
	FILE *output;
	if ((fopen_s(&output, (BASE_PATH + OUTPUT_PREFIXE + ".JSON").c_str(), "w")) != 0)
	{
		cout << "Could not open the JSON file to write" << endl;
		exit(0);
	}


	fprintf(output, "%s\n", JSON_buffer);
	fclose(output);


	printf("Saving cube info\n");

	FILE *output1;
	if ((fopen_s(&output1, (BASE_PATH + OUTPUT_PREFIXE + "_cuboid.txt").c_str(), "w")) != 0)
	{
		cout << "Could not open the cuboid info file to write" << endl;
		exit(0);
	}

	fprintf(output1, "%d %f\n", num_cube, cube_resolusion);

	for (int i = 0; i < num_cube; i++)
	{
		for (int j = 0; j < 9; j++)
			fprintf(output1, "%f ", cube_info[i][j]);
		fprintf(output1, "\n");
	}

	fclose(output1);


	if (OBJ_SAVE == 0)
		save_obj_rect_object(new_vertex, obj_list, no_object, new_rectangle, new_valid_face, new_total_v, new_total_f);
	else if (OBJ_SAVE == 1)
		save_obj_rect_object_colour(new_vertex, new_color, obj_list, no_object, new_rectangle, new_valid_face, new_total_v, new_total_f);
	else if (OBJ_SAVE == 2)
		save_obj_rect(new_vertex, new_uv, new_rectangle, new_valid_face, new_total_v, new_total_f);
	else if (OBJ_SAVE == 3)
		save_off_rect_object(new_vertex, new_color, new_alpha, new_rectangle, new_valid_face, new_total_v, new_total_f);


	printf("Done!!!\n");
	return 0;
}

