using namespace cv;
using namespace std;


bool leftDown = false, leftup = false;
Point cor1, cor2;
Rect box;
Mat temp_img;
int assigned_label;
int rect_counter;

void reset_new_region()
{
	for (int i = 0; i<MAX_OBJECTS; i++)
	{
		for (int j = 0; i<5; i++)
			new_region[i][j] = 0.0;
	}

}

void mouseHandler(int event, int x, int y, int flags, void* param)
{
	if (event == EVENT_LBUTTONDOWN)
	{
		leftDown = true;
		cor1.x = x;
		cor1.y = y;
	}
	if (event == EVENT_LBUTTONUP)
	{
		if (abs(x - cor1.x)>5 && abs(y - cor1.y)>5) //checking whether the region is too small
		{
			leftup = true;
			cor2.x = x;
			cor2.y = y;
		}
		else
		{
			cout << "Select a region more than 5 pixels" << endl;
		}
	}

	if (leftDown == true && leftup == false) //when the left button is down
	{
		Point pt;
		pt.x = x;
		pt.y = y;
		Mat temp_img2 = temp_img.clone();
		rectangle(temp_img2, cor1, pt, Scalar(0, 0, 255)); //drawing a rectangle continuously
		imshow("Org", temp_img2);

	}
	if (leftDown == true && leftup == true) //when the selection is done
	{

		box.width = abs(cor1.x - cor2.x);
		box.height = abs(cor1.y - cor2.y);
		box.x = min(cor1.x, cor2.x);
		box.y = min(cor1.y, cor2.y);
		rectangle(temp_img, cor1, cor2, Scalar(0, 0, 255)); 

		imshow("Org", temp_img);

		new_region[rect_counter][0] = assigned_label;
		new_region[rect_counter][1] = box.x;
		new_region[rect_counter][2] = box.y;
		new_region[rect_counter][3] = box.x + box.width;
		new_region[rect_counter][4] = box.y + box.height;

		rect_counter++;
		
		leftDown = false;
		leftup = false;
	}
}


void manual_region_processing(const Mat& img)
{
	temp_img= img.clone();

	assigned_label = 14;		//default: Loud Speaker
	rect_counter = 0;

	printf("Assign object labels \n"
		"\t ESC - break and exit\n"
		"\t SPACE - Done, move to next step\n"
		"\t o - restore original image\n"
		"\t 1 - Speaker, 2 - sofa, 3 - chair, 4 - table, 5 - TV, 6 - furniture, 7 - object \n");

	namedWindow("Org", WINDOW_AUTOSIZE);
	moveWindow("Org", 0, 0);
	imshow("Org", temp_img);

	cvSetMouseCallback("Org", mouseHandler, 0);

	for (;;)
	{
		int c = cvWaitKey(0);

		if ((char)c == ' ')	break;

		if ((char)c == 27)
		{
			exit(-1);
		}

		if ((char)c == 'o')
		{
			temp_img = img.clone();
			reset_new_region();
			assigned_label = 0;
			rect_counter = 0;
			imshow("Org", temp_img);
		}

		if ((char)c == '1')
		{
			printf("speaker\n");
			assigned_label = 14;
		}

		if ((char)c == '2')
		{
			printf("sofa\n");
			assigned_label = 9;
		}

		if ((char)c == '3')
		{
			printf("chair\n");
			assigned_label = 4;
		}

		if ((char)c == '4')
		{
			printf("table\n");
			assigned_label = 10;
		}

		if ((char)c == '5')
		{
			printf("TV\n");
			assigned_label = 11;
		}

		if ((char)c == '6')
		{
			printf("furniture\n");
			assigned_label = 6;
		}

		if ((char)c == '7')
		{
			printf("object\n");
			assigned_label = 7;
		}

	}
}


CvPoint3D32f Cartesian2Spherical(CvPoint3D32f point_3D)
{
	CvPoint3D32f spherical_3D;

	spherical_3D.x = sqrt(point_3D.x * point_3D.x + point_3D.y * point_3D.y + point_3D.z * point_3D.z);		//r
	spherical_3D.y = acos(point_3D.z / spherical_3D.x);														//theta
	spherical_3D.z = CV_PI - atan(point_3D.y / point_3D.x);													//phi

	return spherical_3D;
}

int point_cloud_generation (Mat& disparity, int x1, int y1, int x2, int y2)
{
	int total_v = (x2 - x1)*(y2 - y1);
	int i, j, k;
	double longitude, latitude, radius, angle_disp;
	Size size = disparity.size();
	double unit_h = 1.0 / (size.height);
	double unit_w = 2.0 / (size.width);
	int num_vertex = 0;

	for (i = y1; i< y2; i++)
		for(j = x1; j < x2; j++)
		{
			if (disparity.at<float>(i, j) == 0)
				continue;

			latitude = i*unit_h * CV_PI;
			longitude = j*unit_w * CV_PI;

			angle_disp = (disparity.at<float>(i, j) / disp_scale + disp_offset) * unit_h * CV_PI;  

			if (latitude + angle_disp <0)
				angle_disp = 0.01;

			if (angle_disp == 0 || disparity.at<float>(i, j) == 0)
			{
				radius = max_radius;
				disparity.at<float>(i, j) = 0;
			}
			else if (i<inf_border)
			{
				radius = max_radius;
				disparity.at<float>(i, j) = 0;
			}
			else if (i> size.height - inf_border)
			{
				radius = max_radius;
				disparity.at<float>(i, j) = 0;
			}
			else
			{
				radius = baseline / ((sin(latitude) / tan(latitude + angle_disp)) - cos(latitude));
			}

			if (radius > max_radius)
				radius = max_radius;

			if (radius < 0.0)
			{
				radius = max_radius;
				disparity.at<float>(i, j) = 0;
			}

			vertex[num_vertex].x = radius*sin(latitude)*cos(CV_PI - longitude);
			vertex[num_vertex].y = radius*sin(latitude)*sin(CV_PI - longitude);
			vertex[num_vertex].z = radius*cos(latitude);
			num_vertex++;
		}
	return num_vertex;
}


void generate_new_cuboid(Mat& disparity, int num)
{
	CvPoint3D32f min(1000.0, 1000.0, 1000.0), max(-1000.0, -1000.0, -1000.0), avg(0.0, 0.0, 0.0); //range of point clouds (x,y,z)

	int num_vertex = point_cloud_generation(disparity, new_region[num][1], new_region[num][2], new_region[num][3], new_region[num][4]);

	//set the range
	for (int i = 0; i < num_vertex; i++)
	{
		if (vertex[i].x < min.x)
			min.x = vertex[i].x;
		if (vertex[i].y < min.y)
			min.y = vertex[i].y;
		if (vertex[i].z < min.z)
			min.z = vertex[i].z;
		if (vertex[i].x > max.x)
			max.x = vertex[i].x;
		if (vertex[i].y > max.y)
			max.y = vertex[i].y;
		if (vertex[i].z > max.z)
			max.z = vertex[i].z;
	}

	new_cube_info[0] = 1;
	new_cube_info[1] = min.x;
	new_cube_info[2] = max.x;
	new_cube_info[3] = min.y;
	new_cube_info[4] = max.y;
	new_cube_info[5] = min.z;
	new_cube_info[6] = max.z;
	new_cube_info[7] = new_region[num][0];
	new_cube_info[8] = 0;

	printf("Object label (14 = Loud Speaker): %.0f \n", new_cube_info[7]);
	printf("Center (x,y,z): %.2f %.2f %.2f    Length (x,y,z): %.2f %.2f %.2f  \n",
		(new_cube_info[2] + new_cube_info[1]) / 2.0, (new_cube_info[4] + new_cube_info[3]) / 2.0, (new_cube_info[6] + new_cube_info[5]) / 2.0,
		(new_cube_info[2] - new_cube_info[1]), (new_cube_info[4] - new_cube_info[3]), (new_cube_info[6] - new_cube_info[5]));
}

void cube_boundary_refinement(int num_cube)
{
	for (int i = 0; i<num_cube; i++)
	{
		if (cube_info[i][1] <= room_min.x)
		{
			cube_info[i][1] = room_min.x;
			cube_valid_face[i][1] = 0;
		}
		if (cube_info[i][3] <= room_min.y)
		{
			cube_info[i][3] = room_min.y;
			cube_valid_face[i][3] = 0;
		}
		if (cube_info[i][5] <= room_min.z)
		{
			cube_info[i][5] = room_min.z;
			cube_valid_face[i][5] = 0;
		}

		if (cube_info[i][2] >= room_max.x)
		{
			cube_info[i][2] = room_max.x;
			cube_valid_face[i][2] = 0;
		}

		if (cube_info[i][4] >= room_max.y)
		{
			cube_info[i][4] = room_max.y;
			cube_valid_face[i][4] = 0;
		}
		if (cube_info[i][6] >= room_max.z)
		{
			cube_info[i][6] = room_max.z;
			cube_valid_face[i][6] = 0;
		}
	}
}


void remove_overlapping_faces(int num_cube)
{
	for (int i = 0; i<num_cube; i++)
	{
		for (int j = 0; j<num_cube; j++)
		{
			if (j == i)
				continue;

			//x min plane
			if (cube_info[i][1] == cube_info[j][1] || cube_info[i][1] == cube_info[j][2])
			{
				if (cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][1] = 0;
				}
			}

			//x max plane
			if (cube_info[i][2] == cube_info[j][1] || cube_info[2][1] == cube_info[j][2])
			{
				if (cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][2] = 0;
				}
			}

			//y min plane
			if (cube_info[i][3] == cube_info[j][3] || cube_info[i][3] == cube_info[j][4])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][3] = 0;
				}
			}

			//y max plane
			if (cube_info[i][4] == cube_info[j][3] || cube_info[i][4] == cube_info[j][4])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][4] = 0;
				}
			}

			//z min plane
			if (cube_info[i][5] == cube_info[j][5] || cube_info[i][5] == cube_info[j][6])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4])
				{
					cube_valid_face[i][5] = 0;
				}
			}

			//z max plane
			if (cube_info[i][6] == cube_info[j][5] || cube_info[i][6] == cube_info[j][6])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4])
				{
					cube_valid_face[i][6] = 0;
				}
			}
		}
	}
}


void cube_rectangle_generation_boundary(float **cube_info, int cube_no)  //For room boundary generation. should have normals into the room. 
{
	int		temp_i;
	int		num_vertex = 0;
	int		num_rectangle = 0;
	int		temp_counter = 0;
	int		obj_index = 5;		//fixed as "Floor"

	CvPoint3D32f	point_3D[8];
	CvPoint3D32f	spherical_3D;
	CvPoint3D32f	min_point, max_point;

	CvPoint3D32f	temp_point3D;

	CvPoint			point_2D;

	uchar	src[3];

	CvPoint3D32f plane_point;

	float	unit_h = 1.0 / (img_size.height);
	float	unit_w = 2.0 / (img_size.width);

	int		mode = (int)cube_info[cube_no][0];

	min_point.x = cube_info[cube_no][1];
	min_point.y = cube_info[cube_no][3];
	min_point.z = cube_info[cube_no][5];
	max_point.x = cube_info[cube_no][2];
	max_point.y = cube_info[cube_no][4];
	max_point.z = cube_info[cube_no][6];

	//make 8 vertice of cuboid
	point_3D[0] = min_point;

	point_3D[1].x = max_point.x;
	point_3D[1].y = min_point.y;
	point_3D[1].z = min_point.z;

	point_3D[2].x = max_point.x;
	point_3D[2].y = min_point.y;
	point_3D[2].z = max_point.z;

	point_3D[3].x = min_point.x;
	point_3D[3].y = min_point.y;
	point_3D[3].z = max_point.z;

	point_3D[4].x = min_point.x;
	point_3D[4].y = max_point.y;
	point_3D[4].z = min_point.z;

	point_3D[5].x = max_point.x;
	point_3D[5].y = max_point.y;
	point_3D[5].z = min_point.z;

	point_3D[6] = max_point;

	point_3D[7].x = min_point.x;
	point_3D[7].y = max_point.y;
	point_3D[7].z = max_point.z;

	// 8 vertices generation
	for (int i = 0; i<8; i++)
	{

		new_vertex[new_total_v + num_vertex] = point_3D[i];

		vertex_visibility[new_total_v + num_vertex] = 1;

		spherical_3D = Cartesian2Spherical(point_3D[i]);

		if (point_3D[i].x > 0 && point_3D[i].y < 0)
			spherical_3D.z -= CV_PI;
		if (point_3D[i].x > 0 && point_3D[i].y > 0)
			spherical_3D.z += CV_PI;
		if (point_3D[i].x == 0 && point_3D[i].y <= 0)
			spherical_3D.z = CV_PI / 2;
		if (point_3D[i].x == 0 && point_3D[i].y > 0)
			spherical_3D.z = CV_PI * 3 / 2;
		if (point_3D[i].y == 0 && point_3D[i].x > 0)
			spherical_3D.z = 0.0;


		point_2D.y = (int)floor(spherical_3D.y / (CV_PI*unit_h) + 0.5);
		point_2D.x = (int)floor(spherical_3D.z / (CV_PI*unit_w) + 0.5);

		if (point_2D.y<0)	point_2D.y = 0;
		if (point_2D.x<0)	point_2D.x = 0;
		if (point_2D.y >= img_size.height)		point_2D.y = img_size.height - 1;
		if (point_2D.x >= img_size.width)		point_2D.x = img_size.width - 1;

		new_uv[new_total_v + num_vertex] = cvPoint2D32f(point_2D.x / (double)img_size.width, (img_size.height - point_2D.y - 1) / (double)img_size.height);
		src[0] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[0];
		src[1] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[1];
		src[2] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[2];

		new_color[new_total_v + num_vertex] = cvPoint3D32f((double)obj_color_table[obj_index][0], (double)obj_color_table[obj_index][1], (double)obj_color_table[obj_index][2]);

		num_vertex++;
	}


	// 6 rentangles generation with normal information (to the inside) with right hand rule for Boundary

	//	if (cube_valid_face[cube_no][5] == 1)		//Floor
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 0);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 1);

		num_rectangle++;
	}


	//	if (cube_valid_face[cube_no][6] == 1)			// Ceiling
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 3);

		num_rectangle++;
	}

//	cube_valid_face[cube_no][3] == 1)		//Wall_1
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 1);

		num_rectangle++;
	}

	//	if (cube_valid_face[cube_no][2] == 1)		//Wall_2
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 2);

		num_rectangle++;
	}

//	if (cube_valid_face[cube_no][4] == 1)		//Wall_3
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 7);

		num_rectangle++;
	}

//	if (cube_valid_face[cube_no][1] == 1)		//Wall_4
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 0);

		num_rectangle++;
	}

	new_total_v += num_vertex;
	new_total_f += num_rectangle;
}


void cube_rectangle_generation(float **cube_info, int cube_no)
{
	int		temp_i;
	int		num_vertex = 0;
	int		num_rectangle = 0;
	int		temp_counter = 0;
	int		obj_index = cube_info[cube_no][7];

	CvPoint3D32f	point_3D[8];
	CvPoint3D32f	spherical_3D;
	CvPoint3D32f	min_point, max_point;

	CvPoint3D32f	temp_point3D;

	CvPoint			point_2D;

	uchar	src[3];

	CvPoint3D32f plane_point;

	float	unit_h = 1.0 / (img_size.height);
	float	unit_w = 2.0 / (img_size.width);

	int		mode = (int)cube_info[cube_no][0];

	min_point.x = cube_info[cube_no][1];
	min_point.y = cube_info[cube_no][3];
	min_point.z = cube_info[cube_no][5];
	max_point.x = cube_info[cube_no][2];
	max_point.y = cube_info[cube_no][4];
	max_point.z = cube_info[cube_no][6];

	//make 8 vertice of cuboid
	point_3D[0] = min_point;

	point_3D[1].x = max_point.x;
	point_3D[1].y = min_point.y;
	point_3D[1].z = min_point.z;

	point_3D[2].x = max_point.x;
	point_3D[2].y = min_point.y;
	point_3D[2].z = max_point.z;

	point_3D[3].x = min_point.x;
	point_3D[3].y = min_point.y;
	point_3D[3].z = max_point.z;

	point_3D[4].x = min_point.x;
	point_3D[4].y = max_point.y;
	point_3D[4].z = min_point.z;

	point_3D[5].x = max_point.x;
	point_3D[5].y = max_point.y;
	point_3D[5].z = min_point.z;

	point_3D[6] = max_point;

	point_3D[7].x = min_point.x;
	point_3D[7].y = max_point.y;
	point_3D[7].z = max_point.z;

	// 8 vertices generation
	for (int i = 0; i<8; i++)
	{

		new_vertex[new_total_v + num_vertex] = point_3D[i];

		vertex_visibility[new_total_v + num_vertex] = 1;

		spherical_3D = Cartesian2Spherical(point_3D[i]);

		if (point_3D[i].x > 0 && point_3D[i].y < 0)
			spherical_3D.z -= CV_PI;
		if (point_3D[i].x > 0 && point_3D[i].y > 0)
			spherical_3D.z += CV_PI;
		if (point_3D[i].x == 0 && point_3D[i].y <= 0)
			spherical_3D.z = CV_PI / 2;
		if (point_3D[i].x == 0 && point_3D[i].y > 0)
			spherical_3D.z = CV_PI * 3 / 2;
		if (point_3D[i].y == 0 && point_3D[i].x > 0)
			spherical_3D.z = 0.0;


		point_2D.y = (int)floor(spherical_3D.y / (CV_PI*unit_h) + 0.5);
		point_2D.x = (int)floor(spherical_3D.z / (CV_PI*unit_w) + 0.5);

		if (point_2D.y<0)	point_2D.y = 0;
		if (point_2D.x<0)	point_2D.x = 0;
		if (point_2D.y >= img_size.height)		point_2D.y = img_size.height - 1;
		if (point_2D.x >= img_size.width)		point_2D.x = img_size.width - 1;

		new_uv[new_total_v + num_vertex] = cvPoint2D32f(point_2D.x / (double)img_size.width, (img_size.height - point_2D.y - 1) / (double)img_size.height);
		src[0] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[0];
		src[1] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[1];
		src[2] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[2];

		new_color[new_total_v + num_vertex] = cvPoint3D32f((double)obj_color_table[obj_index][0], (double)obj_color_table[obj_index][1], (double)obj_color_table[obj_index][2]);

		num_vertex++;
	}


	// 6 rentangles generation with normal information (out of cube) with right hand rule

	if (cube_valid_face[cube_no][3] == 1)		//side 1
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 3);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][4] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 5);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][1] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 0);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 7);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][6] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 6);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][2] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 5);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][5] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 0);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 4);

		num_rectangle++;
	}


	new_total_v += num_vertex;
	new_total_f += num_rectangle;
}