#pragma once

CvPoint3D32f vertex_point(int row, int col)
{
	CvPoint3D32f point_3D = cvPoint3D32f(0, 0, 0);
	double longitude, latitude, radius, angle_disp;

	latitude = row*unit_h * CV_PI;
	longitude = col*unit_w * CV_PI;

	if (disparity.at<float>(row, col) == 0)
		return point_3D;

	angle_disp = (disparity.at<float>(row, col) / disp_scale + disp_offset) * unit_h * CV_PI;

	if (latitude + angle_disp <0)
		angle_disp = 0.01;

	if (angle_disp == 0)
	{
		radius = max_radius;
		disparity.at<float>(row, col) = 0;
	}
	else
		radius = baseline / ((sin(latitude) / tan(latitude + angle_disp)) - cos(latitude));

	if (radius > max_radius || radius < 0.0)
	{
		radius = max_radius;
		disparity.at<float>(row, col) = 0;
	}

	point_3D.x = radius*sin(latitude)*cos(CV_PI - longitude);
	point_3D.y = radius*sin(latitude)*sin(CV_PI - longitude);
	point_3D.z = radius*cos(latitude);

	return point_3D;
}

void room_boundary_reconstruction(Mat& obj_index, Mat& disparity)
{
	const int hist_size = 3000;
	int hist_min_max[6][hist_size]; // x_min, x_max, y_min, y_max, z_min, z_max
	int hist_min_max_filtered[6][hist_size]; // x_min, x_max, y_min, y_max, z_min, z_max
	int max_index[6], max_value[6];
	int temp;

	for (int i = 0; i < 6; i++)
	{
		max_index[i] = 0;
		max_value[i] = 0;

		for (int j = 0; j < hist_size; j++)
		{
			hist_min_max[i][j] = 0;
		}
	}

	CvPoint3D32f point_3D;
	room_min.x = room_min.y = room_min.z = 0.0;
	room_max.x = room_max.y = room_max.z = 0.0;


	// Ceiling
	for (int i = 0; i < obj_index.rows /3 ; i++)
		for (int j = 0; j < obj_index.cols; j++)
			if (obj_index.at<uchar>(i, j) == 3 && disparity.at<float>(i, j) != 0) 
			{
				point_3D = vertex_point(i, j);
				temp = (int)(point_3D.z * 100 + 0.5);

				if (temp <= 0 || temp > hist_size - 1)
					continue;
				else
					hist_min_max[5][temp]++;
			}

	// Floor
	for (int i = 2 * obj_index.rows / 3 ; i < obj_index.rows; i++)
		for (int j = 0; j < obj_index.cols; j++)
			if (obj_index.at<uchar>(i, j) == 5 && disparity.at<float>(i, j) != 0)		
			{
				point_3D = vertex_point(i, j);
				temp = (int)((-1.0) * point_3D.z * 100 + 0.5);

				if (temp <= 0 || temp > hist_size - 1)
					continue;
				else
					hist_min_max[4][temp]++;
			}


	////left (-x) wall
	for (int i = obj_index.rows / 4 ; i < obj_index.rows * 3/4; i++)
		for (int j = 0; j < obj_index.cols / 8; j++)
			if ((obj_index.at<uchar>(i, j) == 12 || obj_index.at<uchar>(i, j) == 8) && disparity.at<float>(i, j) != 0)		//wall
			{
				point_3D = vertex_point(i, j);
				int temp = (int)(point_3D.x * 100 + 0.5);

				if (temp == 0 || temp > hist_size - 1 || temp < (-1.0)* hist_size + 1)
					continue;

				else if (temp > 0)
					hist_min_max[1][temp]++;
				else
				{
					temp = (-1.0) * temp;
					hist_min_max[0][temp]++;
				}
			}

	for (int i = obj_index.rows / 4; i < obj_index.rows * 3 / 4; i++)
		for (int j = obj_index.cols * 7/8; j < obj_index.cols ; j++)
			if ((obj_index.at<uchar>(i, j) == 12 || obj_index.at<uchar>(i, j) == 8) && disparity.at<float>(i, j) != 0)		//wall
			{
				point_3D = vertex_point(i, j);
				temp = (int)(point_3D.x * 100 + 0.5);

				if (temp == 0 || temp > hist_size - 1 || temp < (-1.0)* hist_size + 1)
					continue;

				else if (temp > 0)
					hist_min_max[1][temp]++;
				else
				{
					temp = (-1.0) * temp;
					hist_min_max[0][temp]++;
				}
			}

	//front (+y) wall
	for (int i = obj_index.rows / 4; i < obj_index.rows * 3 / 4; i++)
		for (int j = obj_index.cols * 1/8; j < obj_index.cols * 3/8; j++)
			if ((obj_index.at<uchar>(i, j) == 12 || obj_index.at<uchar>(i, j) == 8) && disparity.at<float>(i, j) != 0)		//wall
			{
				point_3D = vertex_point(i, j);
				temp = (int)(point_3D.y * 100 + 0.5);

				if (temp == 0 || temp > hist_size - 1 || temp < (-1.0)* hist_size + 1)
					continue;
				else if (temp > 0)
				{
					//cout << "y_max " << temp << endl;
					hist_min_max[3][temp]++;
				}
				else
				{
					temp = (-1.0) * temp;
					hist_min_max[2][temp]++;
				}
			}


	
	//right (+x) wall
	for (int i = obj_index.rows / 4; i < obj_index.rows * 3 / 4; i++)
	{
		for (int j = (obj_index.cols * 3 / 8); j < (obj_index.cols * 5 / 8); j++)
		{
			if ((obj_index.at<uchar>(i, j) == 12 || obj_index.at<uchar>(i, j) == 8) && disparity.at<float>(i, j) != 0)		//wall
			{

				point_3D = vertex_point(i, j);
				temp = (int)(point_3D.x * 100 + 0.5);

				if (temp == 0 || temp > hist_size - 1 || temp < (-1.0)* hist_size + 1)
					continue;

				else if (temp > 0)
				{
					//cout << "x_max " << temp << endl;
					hist_min_max[1][temp]++;
				}
				else
				{
					temp = (-1.0) * temp;
					hist_min_max[0][temp]++;
					//cout << "x_min! " << temp1 <<  endl;
				}
			}
		}
	}

	//back (-y) wall
	for (int i = obj_index.rows / 4; i < obj_index.rows * 3 / 4; i++)
		for (int j = obj_index.cols * 5 / 8; j < obj_index.cols * 7 / 8; j++)
			if ((obj_index.at<uchar>(i, j) == 12 || obj_index.at<uchar>(i, j) == 8) && disparity.at<float>(i, j) != 0)		//wall
			{
				point_3D = vertex_point(i, j);
				int temp2 = (int)(point_3D.y * 100 + 0.5);

				if (temp2 == 0 || temp2 > hist_size - 1 || temp2 < (-1.0)* hist_size + 1)
					continue;
				else if (temp2 > 0)
					hist_min_max[3][temp2]++;
				else
				{
					temp2 = (-1.0) * temp2;
					hist_min_max[2][temp2]++;
					//cout << "y_min! " << temp2 << endl;
				}
			}

	


	//for (int i = 0; i < obj_index.rows; i++)
	//{
	//	for (int j = 0; j < obj_index.cols; j++)
	//	{
	//		//if (obj_index.at<uchar>(i, j) == 3 && disparity.at<float>(i, j) != 0) // Ceiling
	//		//{
	//		//	point_3D = vertex_point(i, j);
	//		//	int temp = (int)(point_3D.z * 100 + 0.5);

	//		//	if (temp == 0 || temp > hist_size - 1 || temp < (-1.0)* hist_size + 1)
	//		//		continue;
	//		//	else if (temp > 0)
	//		//		hist_min_max[5][temp]++;
	//		//}

	//		//else if (obj_index.at<uchar>(i, j) == 5 && disparity.at<float>(i, j) != 0)		//Floor
	//		//{
	//		//	point_3D = vertex_point(i, j);
	//		//	int temp = (int)((-1.0) * point_3D.z * 100 + 0.5);

	//		//	if (temp == 0 || temp > hist_size - 1 || temp < (-1.0)* hist_size + 1)
	//		//		continue;
	//		//	else if (temp > 0)
	//		//		hist_min_max[4][temp]++;
	//		//}

	//		if ((obj_index.at<uchar>(i, j) == 12 || obj_index.at<uchar>(i, j) == 8) && disparity.at<float>(i, j) != 0)		//Wall
	//		{
	//			point_3D = vertex_point(i, j);
	//			int temp1 = (int)(point_3D.x * 100 + 0.5);

	//			if (temp1 == 0 || temp1 > hist_size - 1 || temp1 < (-1.0)* hist_size + 1)
	//				continue;

	//			else if (temp1 > 0)
	//				hist_min_max[1][temp1]++;
	//			else
	//			{
	//				temp1 = (-1.0) * temp1;
	//				hist_min_max[0][temp1]++;
	//				//cout << "X_min! " << temp1 <<  endl;
	//			}

	//			int temp2 = (int)(point_3D.y * 100 + 0.5);

	//			if (temp2 == 0 || temp2 > hist_size - 1 || temp2 < (-1.0)* hist_size + 1)
	//				continue;
	//			else if (temp2 > 0)
	//				hist_min_max[3][temp2]++;
	//			else
	//			{
	//				temp2 = (-1.0) * temp2;
	//				hist_min_max[2][temp2]++;
	//				//cout << "Y_min! " << temp2 << endl;
	//			}
	//		}
	//	}
	//}

	//cout << "Histogram Filtering" << endl;

	//histogram filtering
	for (int i = 0; i < 6; i++)
	{
		hist_min_max_filtered[i][0] = hist_min_max[i][0];
		hist_min_max_filtered[i][1] = hist_min_max[i][1];
		hist_min_max_filtered[i][hist_size-1] = hist_min_max[i][hist_size-1];
		hist_min_max_filtered[i][hist_size - 2] = hist_min_max[i][hist_size - 2];

		for (int j = 2; j < hist_size - 2; j++)
		{
			hist_min_max_filtered[i][j] = (int)((hist_min_max[i][j - 2] + hist_min_max[i][j - 1] + hist_min_max[i][j] +
				hist_min_max[i][j + 1] + hist_min_max[i][j + 2]) / 5.0 + 0.5);
		}

	}

	// Find maximum and set as boundary
	for (int i = 0; i < 6; i++)
	{
		for (int j = 10; j < hist_size ; j++)
		{
			if (hist_min_max_filtered[i][j] > max_value[i])
			{
				max_value[i] = hist_min_max_filtered[i][j];
				max_index[i] = j;
			}
		}
	}

	//cout << "y_max index " << max_index[3] << endl;

	room_min.x = (max_index[0] * 0.01)*(-1.0);
	room_max.x = (max_index[1] * 0.01);
	room_min.y = (max_index[2] * 0.01)*(-1.0);
	room_max.y = (max_index[3] * 0.01);
	room_min.z = (max_index[4] * 0.01)*(-1.0);
	room_max.z = (max_index[5] * 0.01);

	if (room_min.x == 0)
		room_min.x = initial_room_dim_min;
	if (room_min.y == 0)
		room_min.y = initial_room_dim_min;
	if (room_min.z == 0)
		room_min.z = initial_room_dim_min;
	if (room_max.x == 0)
		room_max.x = initial_room_dim_max;
	if (room_max.y == 0)
		room_max.y = initial_room_dim_max;
	if (room_max.z == 0)
		room_max.z = initial_room_dim_max;

	//room_max.y = 3.72;	//Only for Usability 1
}

int point_cloud_generation(int region_num)
{
	int total_v;
	int i, j, k;
	CvPoint3D32f point_3D;
	int num_vertex = 0;
	int elem;
	
	for (i = 0; i< img_size.height; i++)
		for (j = 0; j < img_size.width; j++)
		{
			if (region.at<int>(i, j) != region_num || disparity.at<float>(i, j) == 0)
				continue;
			if (i<inf_border || i> img_size.height - inf_border)
				continue;

			point_3D = vertex_point(i, j);
			if (point_3D.x == 0 && point_3D.y == 0 && point_3D.z == 0)
				continue;

			vertex[num_vertex].x = point_3D.x;
			vertex[num_vertex].y = point_3D.y;
			vertex[num_vertex].z = point_3D.z;
			num_vertex++;
		}

	return num_vertex;
}

int point_cloud_generation_with_Region_erosion(int region_num, int erosion_size)
{
	int total_v;
	int i, j, k;
	CvPoint3D32f point_3D;
	int num_vertex = 0;
	int elem;
	
	Mat mask = Mat::zeros(region.size(), CV_8UC1);
	Mat mask2;

	for (i = 0; i< img_size.height; i++)
		for (j = 0; j < img_size.width; j++)
		{
			if (region.at<int>(i, j) != region_num || disparity.at<float>(i, j) == 0)
				continue;
			if (i<inf_border || i> img_size.height - inf_border)
				continue;
			mask.at<uchar>(i, j) = 255;
		}

	Mat element = getStructuringElement(MORPH_RECT, Size(erosion_size, erosion_size),	Point(-1, -1));
	erode(mask, mask2, element);


	for (i = 0; i< img_size.height; i++)
		for (j = 0; j < img_size.width; j++)
		{
			//if (region.at<int>(i, j) != region_num || disparity.at<float>(i, j) == 0)
			//	continue;
			//if (i<inf_border || i> img_size.height - inf_border)
			//	continue;

			if (mask2.at<uchar>(i, j) == 0)
					continue;

			point_3D = vertex_point(i, j);
			if (point_3D.x == 0 && point_3D.y == 0 && point_3D.z == 0)
				continue;

			vertex[num_vertex].x = point_3D.x;
			vertex[num_vertex].y = point_3D.y;
			vertex[num_vertex].z = point_3D.z;
			num_vertex++;
		}

	return num_vertex;
}

vector<float> Outlier_removal(vector<float> vect, Scalar mean, Scalar stddev, float Th)
{
	vector<float> temp_vec;

	for (int i = 0; i< vect.size(); ++i)
	{
		if (vect[i] > mean[0] - Th * stddev[0] && vect[i] < mean[0] + Th * stddev[0])
			temp_vec.push_back(vect[i]);
	}

	return temp_vec;

}


void generate_new_cuboid(int region_num)
{

	vector<float> vector_x, vector_y, vector_z;
	Scalar		sc_mean;
	Scalar		sc_stddev;
	double		min_x, min_y, min_z, max_x, max_y, max_z;


	//int num_vertex = point_cloud_generation(region_num);
	int num_vertex = point_cloud_generation_with_Region_erosion(region_num, 5);		//Reconstruction with eroded region 
	

	//printf("point generated: %d \n", num_vertex);
	//printf("region info: %d %d %d %d \n", region_info.at<int>(region_num, 0), region_info.at<int>(region_num, 1), region_info.at<int>(region_num, 2), region_info.at<int>(region_num, 3));


	for (int i = 0; i < num_vertex; i++)
	{
		vector_x.push_back(vertex[i].x);
		vector_y.push_back(vertex[i].y);
		vector_z.push_back(vertex[i].z);
	}

	Mat vect_x(vector_x);
	meanStdDev(vect_x, sc_mean, sc_stddev);
	vector<float> vector_x_new = Outlier_removal(vector_x, sc_mean, sc_stddev, point_outlier_Th);
	Mat vect_x_new(vector_x_new);
	minMaxLoc(vect_x_new, &min_x, &max_x);

	Mat vect_y(vector_y);
	meanStdDev(vect_y, sc_mean, sc_stddev);
	vector<float> vector_y_new = Outlier_removal(vector_y, sc_mean, sc_stddev, point_outlier_Th);
	Mat vect_y_new(vector_y_new);
	minMaxLoc(vect_y_new, &min_y, &max_y);

	Mat vect_z(vector_z);
	meanStdDev(vect_z, sc_mean, sc_stddev);
	vector<float> vector_z_new = Outlier_removal(vector_z, sc_mean, sc_stddev, point_outlier_Th);
	Mat vect_z_new(vector_z_new);
	minMaxLoc(vect_z_new, &min_z, &max_z);

	new_cube_info[0] = 1;
	new_cube_info[1] = min_x;
	new_cube_info[2] = max_x;
	new_cube_info[3] = min_y;
	new_cube_info[4] = max_y;
	new_cube_info[5] = min_z;
	new_cube_info[6] = max_z;
	new_cube_info[7] = region_info.at<int>(region_num,2);
	new_cube_info[8] = region_info.at<int>(region_num,3);

}


void cube_boundary_refinement(int num_cube)
{
	for (int i = 0; i<num_cube; i++)
	{
		//Out of room rayout
		if (cube_info[i][1] >= room_max.x || cube_info[i][3] >= room_max.y || cube_info[i][5] >= room_max.z || cube_info[i][2] <= room_min.x || cube_info[i][4] <= room_min.y || cube_info[i][6] <= room_min.z )
			cube_info[i][0] = 0;

		if (cube_info[i][1] <= room_min.x)
		{
			cube_info[i][1] = room_min.x;
			cube_valid_face[i][1] = 0;
		}
		if (cube_info[i][3] <= room_min.y)
		{
			cube_info[i][3] = room_min.y;
			cube_valid_face[i][3] = 0;
		}
		if (cube_info[i][5] <= room_min.z)
		{
			cube_info[i][5] = room_min.z;
			cube_valid_face[i][5] = 0;
		}

		if (cube_info[i][2] >= room_max.x)
		{
			cube_info[i][2] = room_max.x;
			cube_valid_face[i][2] = 0;
		}

		if (cube_info[i][4] >= room_max.y)
		{
			cube_info[i][4] = room_max.y;
			cube_valid_face[i][4] = 0;
		}
		if (cube_info[i][6] >= room_max.z)
		{
			cube_info[i][6] = room_max.z;
			cube_valid_face[i][6] = 0;
		}
	}
}

void cube_extension_to_ground(int num_cube)
{
	//By ratio
	//float min_gap_z = (room_max.z - room_min.z)*0.3;
	//float min_gap_x = (room_max.x - room_min.x)*0.2;
	//float min_gap_y = (room_max.y - room_min.y)*0.2;

	// By values
	float min_gap_z = 1.0;
	float min_gap_x = 0.8;
	float min_gap_y = 0.8;


	for (int i = 0; i<num_cube; i++)
	{
		if (cube_info[i][5] <= room_min.z + min_gap_z)
		{
			cube_info[i][5] = room_min.z;
			cube_valid_face[i][5] = 0;
		}

		if (cube_info[i][1] <= room_min.x + min_gap_x)
		{
			cube_info[i][1] = room_min.x;
			cube_valid_face[i][1] = 0;
		}

		if (cube_info[i][3] <= room_min.y + min_gap_y)
		{
			cube_info[i][3] = room_min.y;
			cube_valid_face[i][3] = 0;
		}

		if (cube_info[i][2] >= room_max.x - min_gap_x)
		{
			cube_info[i][2] = room_max.x;
			cube_valid_face[i][2] = 0;
		}

		if (cube_info[i][4] >= room_max.y - min_gap_y)
		{
			cube_info[i][4] = room_max.y;
			cube_valid_face[i][4] = 0;
		}
	}
}

void remove_overlapping_faces(int num_cube)
{
	for (int i = 0; i<num_cube; i++)
	{
		for (int j = 0; j<num_cube; j++)
		{
			if (j == i)
				continue;

			//x min plane
			if (cube_info[i][1] == cube_info[j][1] || cube_info[i][1] == cube_info[j][2])
			{
				if (cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][1] = 0;
				}
			}

			//x max plane
			if (cube_info[i][2] == cube_info[j][1] || cube_info[2][1] == cube_info[j][2])
			{
				if (cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][2] = 0;
				}
			}

			//y min plane
			if (cube_info[i][3] == cube_info[j][3] || cube_info[i][3] == cube_info[j][4])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][3] = 0;
				}
			}

			//y max plane
			if (cube_info[i][4] == cube_info[j][3] || cube_info[i][4] == cube_info[j][4])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][5] >= cube_info[j][5] && cube_info[i][6] <= cube_info[j][6])
				{
					cube_valid_face[i][4] = 0;
				}
			}

			//z min plane
			if (cube_info[i][5] == cube_info[j][5] || cube_info[i][5] == cube_info[j][6])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4])
				{
					cube_valid_face[i][5] = 0;
				}
			}

			//z max plane
			if (cube_info[i][6] == cube_info[j][5] || cube_info[i][6] == cube_info[j][6])
			{
				if (cube_info[i][1] >= cube_info[j][1] && cube_info[i][2] <= cube_info[j][2]
					&& cube_info[i][3] >= cube_info[j][3] && cube_info[i][4] <= cube_info[j][4])
				{
					cube_valid_face[i][6] = 0;
				}
			}
		}
	}
}



CvPoint3D32f Cartesian2Spherical(CvPoint3D32f point_3D)
{
	CvPoint3D32f spherical_3D;

	spherical_3D.x = sqrt(point_3D.x * point_3D.x + point_3D.y * point_3D.y + point_3D.z * point_3D.z);		//r
	spherical_3D.y = acos(point_3D.z / spherical_3D.x);														//theta
	spherical_3D.z = CV_PI - atan(point_3D.y / point_3D.x);													//phi

	return spherical_3D;
}

void cube_rectangle_generation_boundary(float **cube_info, int cube_no)  //For room boundary generation. should have normals into the room. 
{
	int		temp_i;
	int		num_vertex = 0;
	int		num_rectangle = 0;
	int		temp_counter = 0;
	int		obj_index = 5;		//fixed as "Floor"

	CvPoint3D32f	point_3D[8];
	CvPoint3D32f	spherical_3D;
	CvPoint3D32f	min_point, max_point;

	CvPoint3D32f	temp_point3D;

	CvPoint			point_2D;

	uchar	src[3];

	CvPoint3D32f plane_point;

	float	unit_h = 1.0 / (img_size.height);
	float	unit_w = 2.0 / (img_size.width);

	int		mode = (int)cube_info[cube_no][0];

	min_point.x = cube_info[cube_no][1];
	min_point.y = cube_info[cube_no][3];
	min_point.z = cube_info[cube_no][5];
	max_point.x = cube_info[cube_no][2];
	max_point.y = cube_info[cube_no][4];
	max_point.z = cube_info[cube_no][6];

	//make 8 vertice of cuboid
	point_3D[0] = min_point;

	point_3D[1].x = max_point.x;
	point_3D[1].y = min_point.y;
	point_3D[1].z = min_point.z;

	point_3D[2].x = max_point.x;
	point_3D[2].y = min_point.y;
	point_3D[2].z = max_point.z;

	point_3D[3].x = min_point.x;
	point_3D[3].y = min_point.y;
	point_3D[3].z = max_point.z;

	point_3D[4].x = min_point.x;
	point_3D[4].y = max_point.y;
	point_3D[4].z = min_point.z;

	point_3D[5].x = max_point.x;
	point_3D[5].y = max_point.y;
	point_3D[5].z = min_point.z;

	point_3D[6] = max_point;

	point_3D[7].x = min_point.x;
	point_3D[7].y = max_point.y;
	point_3D[7].z = max_point.z;

	// 8 vertices generation
	for (int i = 0; i<8; i++)
	{

		new_vertex[new_total_v + num_vertex] = point_3D[i];

		vertex_visibility[new_total_v + num_vertex] = 1;

		spherical_3D = Cartesian2Spherical(point_3D[i]);

		if (point_3D[i].x > 0 && point_3D[i].y < 0)
			spherical_3D.z -= CV_PI;
		if (point_3D[i].x > 0 && point_3D[i].y > 0)
			spherical_3D.z += CV_PI;
		if (point_3D[i].x == 0 && point_3D[i].y <= 0)
			spherical_3D.z = CV_PI / 2;
		if (point_3D[i].x == 0 && point_3D[i].y > 0)
			spherical_3D.z = CV_PI * 3 / 2;
		if (point_3D[i].y == 0 && point_3D[i].x > 0)
			spherical_3D.z = 0.0;


		point_2D.y = (int)floor(spherical_3D.y / (CV_PI*unit_h) + 0.5);
		point_2D.x = (int)floor(spherical_3D.z / (CV_PI*unit_w) + 0.5);

		if (point_2D.y<0)	point_2D.y = 0;
		if (point_2D.x<0)	point_2D.x = 0;
		if (point_2D.y >= img_size.height)		point_2D.y = img_size.height - 1;
		if (point_2D.x >= img_size.width)		point_2D.x = img_size.width - 1;

		new_uv[new_total_v + num_vertex] = cvPoint2D32f(point_2D.x / (double)img_size.width, (img_size.height - point_2D.y - 1) / (double)img_size.height);
		
		if(new_uv[new_total_v + num_vertex].x < 0.5)
			new_uv[new_total_v + num_vertex].x += 0.5;
		else
			new_uv[new_total_v + num_vertex].x -= 0.5;


		//src[0] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[0];
		//src[1] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[1];
		//src[2] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[2];

		new_color[new_total_v + num_vertex] = cvPoint3D32f((double)obj_color_table[obj_index][0], (double)obj_color_table[obj_index][1], (double)obj_color_table[obj_index][2]);

		//cout << "generated boundary vertex " << i << endl;
		//cout << "3D point " << point_3D[i].x << ", " << point_3D[i].y << ", " << point_3D[i].z << endl;
		//cout << "UV point " << point_2D.x << ", " << point_2D.y << endl;


		num_vertex++;
	}


	// 6 rentangles generation with normal information (to the inside) with right hand rule for Boundary

	//	if (cube_valid_face[cube_no][5] == 1)		//Floor
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 0);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 1);

		num_rectangle++;
	}


	//	if (cube_valid_face[cube_no][6] == 1)			// Ceiling
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 3);

		num_rectangle++;
	}

	//	cube_valid_face[cube_no][3] == 1)		//Wall_1
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 1);

		num_rectangle++;
	}

	//	if (cube_valid_face[cube_no][2] == 1)		//Wall_2
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 2);

		num_rectangle++;
	}

	//	if (cube_valid_face[cube_no][4] == 1)		//Wall_3
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 7);

		num_rectangle++;
	}

	//	if (cube_valid_face[cube_no][1] == 1)		//Wall_4
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 0);

		num_rectangle++;
	}

	new_total_v += num_vertex;
	new_total_f += num_rectangle;
}


void cube_rectangle_generation(float **cube_info, int cube_no)
{
	int		temp_i;
	int		num_vertex = 0;
	int		num_rectangle = 0;
	int		temp_counter = 0;
	int		obj_index = cube_info[cube_no][7];

	CvPoint3D32f	point_3D[8];
	CvPoint3D32f	spherical_3D;
	CvPoint3D32f	min_point, max_point;

	CvPoint3D32f	temp_point3D;

	CvPoint			point_2D;

	CvPoint3D32f plane_point;

	uchar	src[3];

	float	unit_h = 1.0 / (img_size.height);
	float	unit_w = 2.0 / (img_size.width);

	int		mode = (int)cube_info[cube_no][0];

	min_point.x = cube_info[cube_no][1];
	min_point.y = cube_info[cube_no][3];
	min_point.z = cube_info[cube_no][5];
	max_point.x = cube_info[cube_no][2];
	max_point.y = cube_info[cube_no][4];
	max_point.z = cube_info[cube_no][6];

	//cout << "object index " << obj_index << endl;
	//cout << "min point " << min_point.x << ", " << min_point.y << ", " << min_point.z << endl;
	//cout << "max point " << max_point.x << ", " << max_point.y << ", " << max_point.z << endl;

	//make 8 vertice of cuboid
	point_3D[0] = min_point;

	point_3D[1].x = max_point.x;
	point_3D[1].y = min_point.y;
	point_3D[1].z = min_point.z;

	point_3D[2].x = max_point.x;
	point_3D[2].y = min_point.y;
	point_3D[2].z = max_point.z;

	point_3D[3].x = min_point.x;
	point_3D[3].y = min_point.y;
	point_3D[3].z = max_point.z;

	point_3D[4].x = min_point.x;
	point_3D[4].y = max_point.y;
	point_3D[4].z = min_point.z;

	point_3D[5].x = max_point.x;
	point_3D[5].y = max_point.y;
	point_3D[5].z = min_point.z;

	point_3D[6] = max_point;

	point_3D[7].x = min_point.x;
	point_3D[7].y = max_point.y;
	point_3D[7].z = max_point.z;

	// 8 vertices generation
	for (int i = 0; i<8; i++)
	{

		new_vertex[new_total_v + num_vertex] = point_3D[i];

		vertex_visibility[new_total_v + num_vertex] = 1;

		spherical_3D = Cartesian2Spherical(point_3D[i]);

		if (point_3D[i].x > 0 && point_3D[i].y < 0)
			spherical_3D.z -= CV_PI;
		if (point_3D[i].x > 0 && point_3D[i].y > 0)
			spherical_3D.z += CV_PI;
		if (point_3D[i].x == 0 && point_3D[i].y <= 0)
			spherical_3D.z = CV_PI / 2;
		if (point_3D[i].x == 0 && point_3D[i].y > 0)
			spherical_3D.z = CV_PI * 3 / 2;
		if (point_3D[i].y == 0 && point_3D[i].x > 0)
			spherical_3D.z = 0.0;



		point_2D.y = (int)floor(spherical_3D.y / (CV_PI*unit_h) + 0.5);
		point_2D.x = (int)floor(spherical_3D.z / (CV_PI*unit_w) + 0.5);

		if (point_2D.y<0)	point_2D.y = 0;
		if (point_2D.x<0)	point_2D.x = 0;
		if (point_2D.y >= img_size.height)		point_2D.y = img_size.height - 1;
		if (point_2D.x >= img_size.width)		point_2D.x = img_size.width - 1;

		new_uv[new_total_v + num_vertex] = cvPoint2D32f(point_2D.x / (double)img_size.width, (img_size.height - point_2D.y - 1) / (double)img_size.height);

		if (new_uv[new_total_v + num_vertex].x < 0.5)
			new_uv[new_total_v + num_vertex].x += 0.5;
		else
			new_uv[new_total_v + num_vertex].x -= 0.5;

		//src[0] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[0];
		//src[1] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[1];
		//src[2] = img_t.at<Vec3b>(new_uv[new_total_v + num_vertex].y, new_uv[new_total_v + num_vertex].x)[2];

		new_color[new_total_v + num_vertex] = cvPoint3D32f((double)obj_color_table[obj_index][0], (double)obj_color_table[obj_index][1], (double)obj_color_table[obj_index][2]);

		//cout << "generated boundary vertex " << i << endl;
		//cout << "3D point " << point_3D[i].x << ", " << point_3D[i].y << ", " << point_3D[i].z << endl;
		//cout << "UV point " << point_2D.x << ", " << point_2D.y << endl;

		num_vertex++;
	}


	// 6 rentangles generation with normal information (out of cube) with right hand rule
	if (cube_valid_face[cube_no][3] == 1)		//side 1
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 3);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][4] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 5);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][1] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 4);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 0);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 7);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][6] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 7);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 3);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 6);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][2] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 6);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 2);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 5);

		num_rectangle++;
	}

	if (cube_valid_face[cube_no][5] == 1)
	{
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 0, new_total_v + 5);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 1, new_total_v + 1);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 2, new_total_v + 0);
		cvSetReal2D(new_rectangle, new_total_f + num_rectangle, 3, new_total_v + 4);

		num_rectangle++;
	}


	new_total_v += num_vertex;
	new_total_f += num_rectangle;
}