# Spherical2Cube-NonSquare.py
# Decomposition of spherical image sequence into 6 rectangular face images 

# python Spherical2Cube-NonSquare.py [Input img] [Ouput img width] [Output img height]


import numpy as np
import cv
import sys

args = sys.argv[0:]

if len(sys.argv) < 4:
	print "Usage: python Spherical2Cube-NonSquare.py [Input img] [Ouput img width] [Output img height]"
	sys.exit()

input_name = args[1]
face_width = int(args[2])
face_height = int(args[3])
side_range = (face_width / float(face_height))	#over projection range for output width

PI = 3.141592

img_org = cv.imread(input_name)
width, height = cv.GetSize(img_org)
img_out = np.zeros((face_height,face_width,3),np.uint8)

#Initial shift as done in the original projection
init_shift = width/4;
img = np.zeros((height,width,3),np.uint8)
if init_shift < 0:
	offset = -1*init_shift
	img[0:height,0:width-offset] = img_org[0:height,offset:width]
	img[0:height,width-offset:width] = img_org[0:height,0:offset]

else:
	offset = init_shift
	img[0:height,0:offset] = img_org[0:height,width-offset:width]
	img[0:height,offset:width] = img_org[0:height,0:width-offset]
	

#For top image
z= 1.0
for i in range(0, face_height):
	x=(float(i)/face_height * (-2.0)) + 1.0

	for j in range(0, face_width):
		y=(float(j)/face_width * (-2.0 * side_range)) + side_range

		r = np.sqrt(x*x+y*y+z*z)
		theta = np.arccos(z/r)
		if x==0:
			phi= 0.5*PI
		else:
			phi = np.arctan(y/x)

		if i>= face_height/2: 
			phi=phi+PI
		
		a = int(theta / PI * height + 0.5)
		b = width -1 - int(phi / (2*PI) * width + 0.5)

		if b < 0:	
			b = b+width
		if b > (width-1):
			b = b-width
		
		if a<0:	
			a=0
		if a > height-1:
			a=height-1

		img_out[i,j] = img[a,b]

out_filename = "Cubic_0.png"
cv.imwrite(out_filename, img_out)		

#For bottom image
z= -1.0
for i in range(0, face_height):
	x=(float(i)/face_height * (2.0)) - 1.0

	for j in range(0, face_width):
		y=(float(j)/face_width * (2.0 * side_range)) - side_range

		r = np.sqrt(x*x+y*y+z*z)
		theta = np.arccos(z/r)
		if x==0:
			phi= 0.5*PI
		else:
			phi = np.arctan(y/x)

		if i<= face_height/2: 
			phi=phi+PI
		
		a = int(theta / PI * height + 0.5)
		b = width -1 - int(phi / (2*PI) * width + 0.5)

		if b < 0:	
			b = b+width
		if b > (width-1):
			b = b-width
		
		if a<0:	
			a=0
		if a > height-1:
			a=height-1
		
		img_out[i,j] = img[a,b]

out_filename = "Cubic_5.png"
cv.imwrite(out_filename, img_out)	

#For front image
x=1.0
for i in range(0, face_height):
	z=(float(i)/face_height * (-2.0)) + 1.0

	for j in range(0, face_width):
		y=(float(j)/face_width * (-2.0 * side_range)) + side_range

		r = np.sqrt(x*x+y*y+z*z)
		theta = np.arccos(z/r)
		if x==0:
			phi= 0.5*PI
		else:
			phi = np.arctan(y/x)

		a = int(theta / PI * height + 0.5)
		b = width -1 - int(phi / (2*PI) * width + 0.5)

		if b < 0:	
			b = b+width
		if b > (width-1):
			b = b-width
		
		img_out[i,j] = img[a,b]

out_filename = "Cubic_1.png"
cv.imwrite(out_filename, img_out)


#For left image
y= -1.0
for i in range(0, face_height):
	z=(float(i)/face_height * (-2.0)) + 1.0

	for j in range(0, face_width):
		x=(float(j)/face_width * (-2.0 * side_range)) + side_range

		r = np.sqrt(x*x+y*y+z*z)
		theta = np.arccos(z/r)
		if x==0:
			phi= 0.5*PI
		else:
			phi = np.arctan(y/x)

		if phi > 0: 
			phi=phi+PI
		
		a = int(theta / PI * height + 0.5)
		b = width -1 - int(phi / (2*PI) * width + 0.5)

		if b < 0:	
			b = b+width
		if b > (width-1):
			b = b-width
		
		img_out[i,j] = img[a,b]

out_filename = "Cubic_2.png"
cv.imwrite(out_filename, img_out)	
	

#For back image
x=-1.0
for i in range(0, face_height):
	z=(float(i)/face_height * (-2.0)) + 1.0

	for j in range(0, face_width):
		y=(float(j)/face_width * (2.0* side_range)) - side_range

		r = np.sqrt(x*x+y*y+z*z)
		theta = np.arccos(z/r)
		if x==0:
			phi= 0.5*PI
		else:
			phi = np.arctan(y/x)
			
		phi = phi+PI;

		a = int(theta / PI * height + 0.5)
		b = width -1 - int(phi / (2*PI) * width + 0.5)

		if b < 0:	
			b = b+width
		if b > (width-1):
			b = b-width
		
		img_out[i,j] = img[a,b]

out_filename = "Cubic_3.png"
cv.imwrite(out_filename, img_out)
	
#For right image
y= 1.0
for i in range(0, face_height):
	z=(float(i)/face_height * (-2.0)) + 1.0

	for j in range(0, face_width):
		x=(float(j)/face_width * (2.0* side_range)) - side_range

		r = np.sqrt(x*x+y*y+z*z)
		theta = np.arccos(z/r)
		if x==0:
			phi= 0.5*PI
		else:
			phi = np.arctan(y/x)

		if phi < 0: 
			phi=phi+PI
		
		a = int(theta / PI * height + 0.5)
		b = width -1 - int(phi / (2*PI) * width + 0.5)

		if b < 0:	
			b = b+width
		if b > (width-1):
			b = b-width
		
		img_out[i,j] = img[a,b]

out_filename = "Cubic_4.png"
cv.imwrite(out_filename, img_out)	

		
print("Done!\n")