# ConvertSegNetResultsToSpherical.py
# Convert SegNet results to Spherical image inputs for geometry reconstruction.

#13 class + speaker:
#1: bed 2: books 3: ceiling 4: chair 5: floor 6: furniture 7: objects 8: picture 9: sofa 10: table
#11: tv 12: wall 13: window 14:speaker

#37 class:
#1: wall 2: floor 3: cabinet 4: bed 5: chair 6: sofa 7: table 8: door 9: window 10: bookshelf
#11: picture 12: counter 13: blinds 14: desk 15: shelves 16: curtain 17: dresser 18: pillow 19: mirror 20: floor_mat
#21: clothes 22: ceiling 23: books 24: fridge 25: tv 26: paper 27: towel 28: shower_curtain 29: box 30: whiteboard
#31: person 32: night_stand 33: toilet 34: sink 35: lamp 36: bathtub 37: bag


### OpenCV use BGR format
Colour_list = []
Colour_list.append([0,0,0])			#0  none
Colour_list.append([0,0,128])		#1  bed
Colour_list.append([0,128,0])		#2  books
Colour_list.append([0,128,128])		#3  ceiling
Colour_list.append([128,0,0])		#4  chair
Colour_list.append([128,0,128])		#5  floor
Colour_list.append([128,128,0])		#6  furniture
Colour_list.append([128,128,128])	#7  objects
Colour_list.append([128,128,255])	#8  picture
Colour_list.append([128,255,128])	#9  sofa
Colour_list.append([128,255,255])	#10  table
Colour_list.append([255,128,128])	#11  tv
Colour_list.append([255,255,128])	#12  wall
Colour_list.append([255,255,255])	#13  window
Colour_list.append([255,128,255])	#14  Speaker

from os import path
import numpy as np
import PIL.Image
import math
import cv
import sys


args = sys.argv[0:]

if len(sys.argv) < 4:
	print "Usage: python ConvertSegNetResultsToSpherical.py INPUT_IMAGE_WIDTH INPUT_IMAGE_HEIGHT SPH_IMAGE_WIDTH SPH_IMAGE_HEIGHT"
	sys.exit()


### Global variables
input_img_width = int(args[1]) #480
input_img_height = int(args[2]) #360
face_img_size = input_img_height
sph_img_width = int(args[3])  #1600 or 2690
sph_img_height = int(args[4])  #800 or 1345
PI = 3.141592


#### Spherical image composition function from cubic images

def composition(no_cat,colour_flag,filtering):

	if colour_flag==0:
		obj_image = np.zeros([6, input_img_height, input_img_width]).astype(np.uint8)
		obj_ext_image = np.zeros([6, input_img_height, input_img_width]).astype(np.uint8)
		face_image = np.zeros([6, face_img_size, face_img_size]).astype(np.uint8)
		sphecial_rgb_image = np.zeros([sph_img_height, sph_img_width]).astype(np.uint8)
		img_out = np.zeros([sph_img_height, sph_img_width]).astype(np.uint8)
	else:
		obj_image = np.zeros([6, input_img_height, input_img_width, 3]).astype(np.uint8)
		obj_ext_image = np.zeros([6, input_img_height, input_img_width, 3]).astype(np.uint8)
		face_image = np.zeros([6, face_img_size, face_img_size, 3]).astype(np.uint8)
		sphecial_rgb_image = np.zeros([sph_img_height, sph_img_width, 3]).astype(np.uint8)
		img_out = np.zeros([sph_img_height, sph_img_width, 3]).astype(np.uint8)
	

	for k in range(6):
		if colour_flag==0:
			filename = "%d" % (k) + "_%d.png" %(no_cat)
			print(" input file name = %s " % filename)
			obj_image[k] = cv.imread(filename,0)
			
		else:
			filename = "%d" % (k) + "_%d_rgb.png" %(no_cat)
			print(" input file name = %s " % filename)
			obj_image[k] = cv.imread(filename,1)
		
		offset_x = (input_img_width-input_img_height)/2
		
		face_image[k,0:input_img_height, 0:input_img_height] = obj_image[k, 0:input_img_height, offset_x:input_img_width-offset_x]

	
		
	########
	## composition of spherical image

	
	#For top image
	z=1.0;

	for a in range(sph_img_height*2/5):
		theta = float(a)/sph_img_height * PI

		for b in range(sph_img_width):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001 
			r= z/np.cos(theta)

			x= r * np.sin(theta) * np.cos(phi)
			y= r * np.sin(theta) * np.sin(phi)
			#z= r * cos(theta)

			i= int((x - 1.0)/(-2.0) * face_img_size)
			j= int( (y + 1.0)/2.0 * face_img_size )

			if j<0: 
				j=0
			if j>face_img_size-1: 
				j=face_img_size-1

			if i>=0 and i<face_img_size:
				if(filtering == 0):	
					sphecial_rgb_image[a,b] = face_image[0,i,j]
				else:	#change to ceiling
					sphecial_rgb_image[a,b] = 3
					
			
	#For bottom image
	z = -1.0;
	for a  in range(sph_img_height*3/5, sph_img_height):
		theta = float(a)/sph_img_height * PI

		for b in range(sph_img_width):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001	

			r= z/ np.cos(theta)

			x= r * np.sin(theta) * np.cos(phi)
			y= r * np.sin(theta) * np.sin(phi)
			#z= r * np.cos(theta)

			i= int((x + 1.0)/(2.0) * face_img_size)
			j= int( (y + 1.0)/2.0 * face_img_size)

			if j<0: 
				j=0
			if j>face_img_size-1: 
				j=face_img_size-1
			if i>=0 and i<face_img_size:
				if(filtering == 0):	
					sphecial_rgb_image[a,b] = face_image[5,i,j]
				else:	#change to ceiling
					temp = face_image[5,i,j]
					if (temp == 0) or (temp == 3) or (temp == 8) or (temp == 11) or (temp == 12) or (temp == 13):
						sphecial_rgb_image[a,b] = 5
					else:
						sphecial_rgb_image[a,b] = temp
	
				

	#for front image			
	x = 1.0;
	for a  in range(sph_img_height/5, sph_img_height*4/5):
		theta = float(a)/sph_img_height * PI

		for b in range(sph_img_width/8):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001

			r= x / np.sin(theta) / np.cos(phi)

			#x= r * sin(theta) * cos(phi)
			y= r * np.sin(theta) * np.sin(phi)
			z= r * np.cos(theta)

			i= int((z - 1.0)/(-2.0) * face_img_size)
			j= int( (y - 1.0)/(-2.0) * face_img_size )

			if j<0: 
				j=0
			if j>face_img_size-1:
				j=face_img_size-1

			if i>=0 and i<face_img_size:
				sphecial_rgb_image[a,b] = face_image[1,i,j]

		for b in range(sph_img_width*7/8, sph_img_width):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001 ;

			r= x / np.sin(theta) / np.cos(phi);

			#x= r * sin(theta) * cos(phi);
			y= r * np.sin(theta) * np.sin(phi);
			z= r * np.cos(theta);

			i= int((z - 1.0)/(-2.0) * face_img_size);
			j= int( (y - 1.0)/(-2.0) * face_img_size);

			if j<0: 
				j=0
			if j>face_img_size-1:
				j=face_img_size-1

			if i>=0 and i<face_img_size:
				sphecial_rgb_image[a,b] = face_image[1,i,j]
		
	#For left image
	y = -1.0;

	for a  in range(sph_img_height/5, sph_img_height*4/5):
		theta = float(a)/sph_img_height * PI

		for b in range(sph_img_width/8, sph_img_width*3/8):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001

			r= y / np.sin(theta) / np.sin(phi)

			x= r * np.sin(theta) * np.cos(phi)
			#y= r * np.sin(theta) * np.sin(phi)
			z= r * np.cos(theta)

			i= int((z - 1.0)/(-2.0) * face_img_size)
			j= int( (x - 1.0)/(-2.0) * face_img_size )

			if j<0: 
				j=0
			if j>face_img_size-1:
				j=face_img_size-1

			if i>=0 and i<face_img_size:
				sphecial_rgb_image[a,b] = face_image[2,i,j]

		
	#For back image
	x = -1.0;
	for a  in range(sph_img_height/5, sph_img_height*4/5):
		theta = float(a)/sph_img_height * PI

		for b in range(sph_img_width*3/8, sph_img_width*5/8):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001	

			r= x / np.sin(theta) /np.cos(phi)

			#x= r * sin(theta) * cos(phi)
			y= r * np.sin(theta) * np.sin(phi)
			z= r * np.cos(theta)

			i= int((z - 1.0)/(-2.0) * face_img_size)
			j= int( (y + 1.0)/(2.0) * face_img_size )

			if j<0: 
				j=0
			if j>face_img_size-1:
				j=face_img_size-1

			if i>=0 and i<face_img_size:
				sphecial_rgb_image[a,b] = face_image[3,i,j]

	#For right image
	y = 1.0;
	for a  in range(sph_img_height/5, sph_img_height*4/5):
		theta = float(a)/sph_img_height * PI

		for b in range(sph_img_width*5/8, sph_img_width*7/8):
			phi = 2*PI - (float(b)/sph_img_width * 2 * PI) - 0.00001		

			r= y / np.sin(theta) / np.sin(phi)
			
			x= r * np.sin(theta) * np.cos(phi)
			#y= r * sin(theta) * sin(phi)
			z= r * np.cos(theta)

			i= int((z - 1.0)/(-2.0) * face_img_size);
			j= int( (x + 1.0)/(2.0) * face_img_size );

			if j<0: 
				j=0
			if j>face_img_size-1:
				j=face_img_size-1

			if i>=0 and i<face_img_size:
				sphecial_rgb_image[a,b] = face_image[4,i,j]

				

	#Output - Spherical image should be shifted by (-init_shift (= img_size.width/4)) set in the original cubic projection
	shift = (-1)*sph_img_width/4 
	
	if shift < 0:
		offset = -1*shift
		img_out[0:sph_img_height,0:sph_img_width-offset] = sphecial_rgb_image[0:sph_img_height,offset:sph_img_width]
		img_out[0:sph_img_height,sph_img_width-offset:sph_img_width] = sphecial_rgb_image[0:sph_img_height,0:offset]

	else:
		offset = shift
		img_out[0:sph_img_height,0:offset] = sphecial_rgb_image[0:sph_img_height,sph_img_width-offset:sph_img_width]
		img_out[0:sph_img_height,offset:sph_img_width] = sphecial_rgb_image[0:sph_img_height,0:sph_img_width-offset]

	return img_out

	

	
################################################
#######
####### Start main function


print("Processing 13 cat result files......\n")	
img_index = composition(13,0,1)		#Filtering option: change top and bottom index to ceiling and floor
out_filename = "13_obj_index.png"
cv.imwrite(out_filename, img_index)	

out_colour = np.zeros([sph_img_height, sph_img_width, 3]).astype(np.uint8)

for i  in range(sph_img_height):
	for j in range(sph_img_width):
		ind = int(img_index[i,j])
		if ind>14:
			ind=0
		out_colour[i,j]= np.uint8(Colour_list[ind])
out_filename = "13_obj_index_colour.png"
cv.imwrite(out_filename, out_colour)				




print("Done!\n")